#region Originating script: 'C:\devhome\Powershell\KempTechPowershellModule\Kemp.LoadBalancer.Powershell.ps1'
#
# $Id: Kemp.LoadBalancer.Powershell.psm1 15367 2017-06-12 23:42:19Z fcarpin $
#
####################
# MODULE VARIABLES #
####################
[System.Reflection.Assembly]::LoadWithPartialName("system.web") | Out-Null

New-Variable -Name LoadBalancerAddress -Scope Script -Force
New-Variable -Name LBAccessPort -Scope Script -Force -Value 443

New-Variable -Name Cred -Scope Script -Force

New-Variable -Name SubjectCN -Scope Script -Force
New-Variable -Name CertificateStoreLocation -Scope Script -Force -Value $null

$IgnoredParameters = @("Verbose","Debug","WarningAction","WarningVariable","ErrorAction",
	"ErrorVariable","OutVariable","OutBuffer","WhatIf","Confirm","RuleType",
	"LoadBalancer", "LBPort", "Credential", "SubjectCN", "CertificateStoreLocation")

$ParamReplacement = @{VSIndex = "vs";VirtualService = "vs";Protocol = "prot";
	RSIndex = "rs";RealServer = "rs";RealServerPort = "rsport";
	RuleName = "rule";InterfaceID = "iface";IPAddress = "addr";
	NameServer="nameserver";NamServer="namserver";HAMode="hamode";Partner='partner';Hcp="hcp";Location="location";
	GeoTraffic="geotraffic";Mapaddress="mapaddress";Mapport="mapport";Cluster="clust";
	KempId="kempid";Password="password";OrderId="orderid";
	GeoInterfaceId = "geosshiface";}

$SystemRuleType = @{MatchContentRule=0;AddHeaderRule=1;DeleteHeaderRule=2;ReplaceHeaderRule=3;ModifyUrlRule=4}
$RestoreType = @{Base=1;VS=2;Geo=3}

# Internal use only
Function closeApiConnections($reader, $stream, $response)
{
	if ($reader)
 	{
		Write-Verbose "deleting reader"
		$reader.Close()
		$reader.Dispose()
	}

	if ($stream)
 	{
		Write-Verbose "deleting stream"
		$stream.Close()
		$stream.Dispose()
	}

	if ($response)
 	{
		Write-Verbose "deleting response"
		$response.Close()
		$response.Dispose()
	}
}

# Internal use only
Function sendRequest2LM($cmd, $propTable, $LmIP, $LmPort, $Cred, $SubjectCN, $CertStore, $FilePath)
{
	if (([String]::IsNullOrEmpty($LmPort))) {
		$LmPort = 443
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertStore)))
	 	{
			$response = Send-LBMessage -Command $cmd -ParameterValuePair $propTable -LoadBalancer $LmIP -LBPort $LmPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertStore -File $FilePath
		}
		else {
			$response = Send-LBMessage -Command $cmd -ParameterValuePair $propTable -LoadBalancer $LmIP -LBPort $LmPort -SubjectCN $SubjectCN -File $FilePath
		}
	}
	else {
		$response = Send-LBMessage -Command $cmd -ParameterValuePair $propTable -LoadBalancer $LmIP -LBPort $LmPort -Credential $Cred -File $FilePath
	}
	$response
}

# Internal use only
Function isSetEmtpyString($paramName)
{
	$retValue = $false

	switch ($paramName)
	{
		{ ($_ -in "NetConsole") } {
				$retValue = $true
				break
		}
		{ ($_ -in "geoclients", "admingw", "geopartners") } {
			$retValue = $true
			break
		}
		{ ($_ -in "SyslogInfo", "SyslogCritical", "SyslogNotice", "SyslogEmergency", "SyslogWarn", "SyslogError") } {
			$retValue = $true
			break
		}
		{ ($_ -in "LDAPServer", "LDAPBackupServer") } {
			$retValue = $true
			break
		}
		default {
		 	$retValue = $false
			break
	 	}
	}
	$retValue
}

# Internal use only
Function isLoginParam($paramName)
{
	$retValue = $false
	if ($paramName -eq "Credential" -or $paramName -eq "SubjectCN" -or $paramName -eq "Certificate" -or $paramName -eq "CertificateStoreLocation")
	{
		$retValue = $true
	}
	if ($paramName -eq "LoadBalancer" -or $paramName -eq "LBPort")
	{
		$retValue = $true
	}
	$retValue
}

# Internal use only
Function Get-LoginCertificate($certStoreLoc, $certCN)
{
	if ($certStoreLoc -eq "" -or $certStoreLoc -eq $false)
	{
		$certStoreLoc = "Cert:\CurrentUser\My"
		Write-Verbose "Certificate Store Location NOT specified. Using the default one [$certStoreLoc]"
	}
	else {
		Write-Verbose "Certificate Store Location specified. Searching the Certificate in [$certStoreLoc]"
	}
	if ($certCN -eq "" -or $certCN -eq $false)
	{
		throw "Missing username"
	}

	$certs_list = Get-ChildItem -Recurse -Path $certStoreLoc

	$found = $false
	$certificate = $null
	$location = "Cert:\"	# Certificates base dir
	foreach ($item in $certs_list) {
		$item.Subject | Where {$check = $_ -Like "*CN=$certCN, OU=support*"}
		if ($check) {
			$index = $item.PSPath.IndexOf("::")
			$location += ($item.PSPath).Substring($index + 2)
			$certificate = $item
			$found = $true
			Write-Verbose "Certificate Store Location [$location]"
			Write-Verbose "Certificate Common Name    [$certCN]"
			break
		}
	}
	$certificate
}

# Internal use only
Function GetValueFromString($str)
{
	# str must be something="somevalue"
	$pos = $str.IndexOf("=")
	if ($pos -gt 0) {
		$pos += 2
		$tmp = $str.Substring($pos)
		$tmp = $tmp.Substring(0, $tmp.Length-1)
		$tmp
	}
}

# Internal use only
Function SetPsAnswer($LmResponse)
{
	if ($LmResponse) {
		if ($LmResponse.Contains("Content-Type: text/plain; charset=utf-8")) {
			$LmResponse = $LmResponse -replace "Content-Type: text/plain; charset=utf-8", ""
		}

		$pos1 = $LmResponse.Indexof("stat=")
		$pos2 = $LmResponse.Indexof("code=")

		if ($pos1 -gt 0 -and $pos2 -gt 0 -and $pos2 -gt $pos1) {
			$t_stat = $LmResponse.Substring($pos1, $pos2 - $pos1 - 1)
			$stat = GetValueFromString $t_stat
			Write-Verbose "t_stat [$t_stat], stat [$stat]"

			$tmp = $LmResponse.Substring($pos1)
			$pos3 = $tmp.Indexof("code=")
			$pos4 = $tmp.IndexOf(">")
			if ($pos3 -gt 0 -and $pos4 -gt 0 -and $pos4 -gt $pos3) {
				$t_code = $tmp.Substring($pos3, $pos4 - $pos3)
				$code = GetValueFromString $t_code
			}
			else {
				$code = ""
			}
			Write-Verbose "t_code [$t_code], code [$code]"

			$pos5 = $tmp.IndexOf("<Success>")
			$pos6 = $tmp.IndexOf("</Success>")
			if ($pos5 -gt 0 -and $pos6 -gt 0 -and $pos6 -gt $pos5) {
				$t_success = $tmp.Substring($pos5 + 9, $pos6 - $pos5 - 9)
				$success = $t_success.trim()
			}
			else {
				$success = ""
			}
			#Write-Verbose "t_success [$t_success], success [$success]"
		}
		else {
			$stat = ""
			$code = ""
			$success = ""
		}

		$tmpHs = [ordered]@{}
		$tmpHs.Add("stat", $stat)
		$tmpHs.Add("code", $code)
		$tmpHs.Add("success", $success)
		$dataObj = New-Object -TypeName PSObject -Prop $tmpHs

		$tmpResHs = [ordered]@{}
		$tmpResHs.response = $dataObj

		$result = New-Object -TypeName PSObject -Prop $tmpResHs
		$result
	}
}

# Internal use only
Function CleanLoggingResult($LmResponse)
{
	$tmp = $LmResponse
	if ($tmp.Contains("Content-Type: text/plain; charset=utf-8")) {
		$tmp = $tmp -replace "Content-Type: text/plain; charset=utf-8", ""
	}
	$tmp
}

#region - LBCommunication

# Internal usage only
Function WriteOutputFile($response, $fileName)
{
	$datalength = $response.ContentLength
	$reader     = $response.GetResponseStream()

	$writer = New-Object io.FileStream $fileName, "Create"
	$buffer = New-Object byte[] 4096

	do {
		$count = $reader.Read($buffer, 0, $buffer.length)
		$writer.Write($buffer, 0, $count)
	} while ($count -gt 0)

	$writer.Flush()
	$writer.Close()
}

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Send-LBMessage
{
 	[CmdletBinding(DefaultParameterSetName="SingleParam")]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[string]$Command,

		[Parameter(Position=1,ParameterSetName="SingleParam")]
		[string]$ParameterName,
		[Parameter(Position=2,ParameterSetName="SingleParam")]
		[string]$ParameterValue,
		[Parameter(Position=1,ParameterSetName="MultiParam")]
		[hashtable]$ParameterValuePair,
		[Parameter(Position=2,ParameterSetName="MultiParam")]
		[string]$File,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.PSCredential]$Credential = $script:cred,

		[ValidateNotNullOrEmpty()]
		[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[ValidateNotNullOrEmpty()]
		[String]$SubjectCN = $script:SubjectCN,

		[int]$LBPort = $LBAccessPort,
		[switch]$Output
	)
	Write-Verbose "Entering Send-LBMessage"

	$sb = New-Object Text.StringBuilder

	if($Command -eq 'enableapi')
	{
		$sb.Append("https://$LoadBalancer`:$LBPort/progs/doconfig/enableapi/set/yes") | out-null
	}
	Elseif($Command -eq 'disableapi')
	{
		$sb.Append("https://$LoadBalancer`:$LBPort/progs/doconfig/enableapi/set/no") | out-null
	}
	Elseif($Command -eq 'isapienabled')
	{
		$sb.Append("https://$LoadBalancer`:$LBPort/access/get?param=version") | out-null
	}
	Elseif($Command -eq 'isapienabled2')
	{
		$Command = 'isapienabled'
		$orig_command = 'isapienabled2'
		$sb.Append("https://$LoadBalancer`:$LBPort/access/get?param=version") | out-null
	}
	Elseif($Command -eq 'set_initial_passwd2')
	{
		$Command = 'set_initial_passwd'
		$orig_command = 'set_initial_passwd2'
		$sb.Append("https://$LoadBalancer`:$LBPort/access/$Command`?") | out-null
	}
	else {
		if ($Command -eq 'alsilicense2')
		{
			$Command = 'alsilicense'
			$orig_command = 'alsilicense2'
		}
		$sb.Append("https://$LoadBalancer`:$LBPort/access/$Command`?") | out-null
	}

	switch ($PsCmdlet.ParameterSetName)
	{
		"SingleParam"
		{
			if($ParameterValue)
			{
				$sb.Append("param=$ParameterName&value=$ParameterValue") | out-null
			}
			Elseif($Command -eq "get")
			{
				# The "get" commands require "param=Name".
				$sb.Append("param=$ParameterName") | out-null
			}
			else {
				$check = isSetEmtpyString $ParameterName
				if (-not $check)
				{
					# No value and omit the "param=".
					$sb.Append("$ParameterName") | out-null
				}
				else {
					$sb.Append("param=$ParameterName&value=$ParameterValue") | out-null
				}
			}
			break
		}
		"MultiParam"
		{
			if ($Command -eq "isapienabled" -or $Command -eq "isapienabled") {
				break
			}
			if ($Command.Contains("set?param=enableapi&value=yes")) {
				break
			}
			foreach ($key in $ParameterValuePair.keys)
			{
				Write-Verbose "key=`"$key`" - value=`"$($ParameterValuePair[$key])`""
				$sb.Append("$key=$($ParameterValuePair[$key])&") | out-null
			}
			break
		}
		default
		{
		}
	}
	$uri = $sb.ToString() -replace "\&$"
	Write-Verbose "Uri = $uri"
	$uriCopy = $uri
	$lastCharFromUri = $uriCopy.substring($uriCopy.length - 1, 1)
	if ($lastCharFromUri -eq "?" -and $Command.Contains("set?param=enableapi&value=yes")) {
		$uri = $uri.Substring(0, $uri.Length-1)
		Write-Verbose "Uri = $uri"
	}
	$response = $null
	try {
		Write-Verbose "Creating request [$uri]"
		[System.Net.ServicePointManager]::Expect100Continue = $true
		[System.Net.ServicePointManager]::MaxServicePointIdleTime = 10000
		[System.Net.ServicePointManager]::ServerCertificateValidationCallback = {$true}
		[System.Net.ServicePointManager]::SecurityProtocol = 'Tls11','Tls12'

		$request = [System.Net.HttpWebRequest]::Create($uri)
		$request.UserAgent = "KempLoadBalancerPowershellModule"
		$request.KeepAlive = $false

		if (-not ([String]::IsNullOrEmpty($SubjectCN))) {
			Write-Verbose "Running the API command using the specified certificate as login credential"
			$loginCert = Get-LoginCertificate $CertificateStoreLocation $SubjectCN
			if ($loginCert -ne $null) {
				Write-Verbose "adding the certificate to the request."
				$tmp = $request.ClientCertificates.Add($loginCert)
			}
			else {
				Throw "ERROR: Can't find certificate with $SubjectCN as CN"
			}
		}
		else {
			Write-Verbose "Running the API command using the specified login/password as login credential"
			$request.Credentials = $Credential
		}

		if (($File) -and (-not ($Output)))
		{
			if ((-not (Test-Path -Path $file)) -and (-not ($Output)))
			{
				Write-Error -Message "You have specified an input file that does not exist." -RecommendedAction "Make sure the path is correct and that the file exists." -TargetObject $file
			}
			Write-Verbose "There is a file to upload."
			$datalength = (Get-Item -Path $File).Length
			Write-Verbose "File length: $datalength"
			$request.SendChunked = $true
			$request.KeepAlive = $false
			#$request.protocolversion = [System.Net.HttpVersion]::Version10
			Write-Verbose "Send Chunked: $($request.SendChunked)"
			$request.method = "POST"
			$request.ContentType = "application/x-www-form-urlencoded"
			Write-Verbose "Reading file..."
			$fileStream = New-Object IO.FileStream($File, "Open", "Read")
			$binaryReader = New-Object IO.BinaryReader($fileStream)
			$data = $binaryReader.ReadBytes([int]$datalength)
			Write-Verbose "Cleaning up readers."
			$binaryReader.close()
			Write-Verbose "Getting request stream"
			$stream = $request.GetRequestStream()
			$binaryWriter = New-Object IO.BinaryWriter($stream)
			Write-Verbose "Writing the data to the stream"
			$binaryWriter.Write($data, 0, $data.length)
			$binaryWriter.Flush()
			$binaryWriter.Close()
		}
		Write-Verbose "Trying to get response"
		$response = $request.GetResponse()
		Write-Verbose "Response received."
		$HTTP_STATUS = $response.StatusCode
		Write-Verbose "HTTP STATUS: $HTTP_STATUS"

		if ($Command -eq 'isapienabled')
		{
			if ($orig_command -eq 'isapienabled2')
			{
				if ($response.StatusCode -eq 404 -or $response.StatusCode -eq 401)
				{
					Write-Verbose "API is Disabled"
				}
				else {
					Write-Verbose "API is Enabled"
				}
			}
			else {
				if ($response.StatusCode -eq 404 -or $response.StatusCode -eq 401)
				{
					Write-Output "API is Disabled"
				}
				else {
					Write-Output "API is Enabled"
				}
			}
		}
	}
	catch [Exception]
	{
		$response  = $_.Exception.InnerException.Response
		$response2 = $_.Exception.Message
		Write-Verbose "response2 [$response2]"
		if ($orig_command -eq 'isapienabled2')
		{
			Write-Verbose "Exception catched [$response2]"
		}
		else {
			Write-Output $_.Exception.Message
		}
		if (!$response)
		{
			if ($Command -eq 'isapienabled' -and $orig_command -ne 'isapienabled2')
			{
				Write-Output "API is disabled"
			}
			$response = $_.Exception.Response
		}
		if ($orig_command -eq 'isapienabled2')
		{
			# (1) The remote server returned an error: (404) Not Found
			# (2) Unable to connect to the remote server.
			$errorMsg = [string]$($_.Exception.Message)
			Write-Verbose "Error Message [$errorMsg]"

			if ($errorMsg -eq "The remote server returned an error: (404) Not Found.")
			{
				throw "API is disabled."
			}
			elseif ($errorMsg.Contains("ERROR: Can't find certificate"))
			{
				throw $errorMsg
			}
			elseif ($errorMsg.Contains("Method Not Allowed"))
			{
				throw $errorMsg
			}
			else {
				throw "Unable to connect to the remote server: check username/password and/or LM IP/PORT."
			}
		}
		if ($Command -eq "logging/netstat") {
			if ($response2.Contains("protocol violation")) {
				Write-Output "ERROR: $response2"
			}
		}
		#$result.response
	}
	finally
	{
		try {
			$stream = $response.GetResponseStream()
		}
		catch {
			if ($response2.Contains("ERROR: Can't find certificate")) {
				Throw $response2
			}
		}
		if($response.contenttype -eq "text/xml")
		{
			$Encoding = [System.Text.Encoding]::GetEncoding("utf-8")
			$reader = New-Object system.io.StreamReader($stream, $Encoding)
			
			if($Command -eq 'readeula' -or $Command -eq 'accepteula')
			{
				Write-Verbose "Skip XML processing in the case of eula commands."
				$result = $reader.REadToEnd()
				Write-Output $result
			}
			Else {
				if ($Command -ne "logging/ps") {
					Write-Verbose "Converting response to XML."
					[xml]$result = $reader.REadToEnd()
				}
				else {
					Write-Verbose "logging/* case. Cmd [$Command]"
					[string]$tmp = $reader.REadToEnd()
					$result = SetPsAnswer $tmp
				}
				Write-Verbose "Checking for success."
				if ($result.response.success)
				{
					Write-Verbose "This is a success response. Cmd [$Command]"
					if ($Command -eq "alsilicensetypes" -or $Command -eq "aslgetlicensetypes") {
						Write-Output $result
					}
					elseif([string]::IsNullOrEmpty($result.response.success.data) -and ($Command -ne "logging/ping" -or $Command -ne "logging/ping6"))
					{
						Write-Verbose "There is no data, so just write the code."
						if ($Command -eq "DownloadWafRules" -or $Command -eq "Update-WafRulesDatabase")
						{
							Write-Verbose "No writing response code"
						}
						elseif ($Command -eq "ManInstallWafRules" -or $Command -eq "Install-WafRulesDatabase")
						{
							Write-Verbose "No writing response code"
						}
						elseif ($Command -like "set?param=enableapi&value=no&")
						{
							Write-Verbose "No writing response code"
						}
						elseif ($Command -eq "resolvenow")
						{
							Write-Verbose "No writing response code"
						}
						elseif ($Command -eq "logging/ping" -or $Command -eq "logging/ping6")
						{
							# TODO: this is a check to due a REST API BUG
							Write-Output $result.response.success
						}
						elseif ($Command.Contains("logging/"))
						{
							[string]$tmp = $result.response.success
							$answer = CleanLoggingResult $tmp
							Write-Output $answer
						}
						else {
							if ($orig_command -eq "alsilicense2") {
								Write-Output $result.response.code
							}
							else {
								Write-Output $result.response.code
							}
						}
					}
					elseif ($Command -eq "logging/ping" -or $Command -eq "logging/ping6")
					{
						# TODO: this is a check to due a REST API BUG
						Write-output $result.response.success
					}
					else {
						Write-Verbose "There is data to return."
						write-output $result.response.success.data
					}
				}
				else {
					Write-Verbose "This is a failure response, and here is the response."
					if ($Command -eq "set_initial_passwd" -and $orig_command -eq "set_initial_passwd2")
					{
						throw "Method Not Allowed."
					}
					if ($Command -eq "geoacl/getsettings")
					{
						throw "$($result.response.error)"
					}
					if ($Command -eq "geoacl/setautoupdate" -or $Command -eq "geoacl/setautoinstall" -or $Command -eq "geoacl/setinstalltime" -or $Command -eq "geoacl/installnow" -or $Command -eq "geoacl/updatenow" -or $Command -eq "geoacl/downloadlist" -or $Command -eq "geoacl/downloadchanges")
					{
						$errorMsg = $($result.response.error)
						if ($errorMsg -eq "Command not available.")
						{
							throw "operation not permitted."
						}
						else {
							throw "$($result.response.error)"
						}
					}
					if ($Command -eq "geoacl/addcustom")
					{
						$errorMsg = $($result.response.error)
						if ($errorMsg -eq "addr: CIDR format out of range" -or $errorMsg -eq "addr: Invalid address format")
						{
							throw "$errorMsg"
						}
						elseif ($errorMsg -like "* is already present in the white list")
						{
							$smp = $errorMsg.IndexOf(":")
							$errMsg  = $errorMsg.substring($smp + 2)
							throw "$errMsg"	#the address/network is already present in the white list."
						}
						else {
							throw "Internal error."
						}
					}
					if ($Command -eq "geoacl/removecustom")
					{
						throw "the address/network is NOT present in the whitelist."
					}
					if ($Command -eq "modiface") {
						closeApiConnections $reader $stream $response
					}
					if ($Command -eq "alsilicense") {
						Throw $($result.Response.Error)
					}
					Write-Output $result.Response
					Write-Output $result.Response.Error
				}
			}
		}
		if ($response.ContentType -eq "application/octet-stream" -or $response.ContentType -eq "application/x509-cert" -or $Command -eq "exportvstmplt" -or $response.ContentType -eq "application/vnd.tcpdump.pcap")
		{
			if ($response.StatusCode -eq 200)
			{
				$check = ""
				$fileCheck = Test-Path -Path $file

				# 1) The output file does not exist            ---> write it
				# 2) The output file exists and force is true  ---> write it
				# 3) The output file exists and force is false ---> don't write it

				if (-not $fileCheck) {
					# 1
					WriteOutputFile $response $file
				}
				else {
					if ($Force) {
						# 2
						WriteOutputFile $response $file
					}
					else {
						# 3
						if ($Command -eq "tcpdump" -or $Command -eq "downloadsamlspcert") {
							$check = "ERROR: The specified file already exists. To use the same filename, either delete the file or use the -Force switch."
						}
						else {
							$recActStr = "To use the same filename, either delete the file or use the -Force switch"
							Write-Error -Message "The specified file already exists." -RecommendedAction "$recActStr" -TargetObject $file
						}
					}
				}

				if ($Command -eq "tcpdump") {
					if ($check -eq "") {
						Write-Verbose "command successfully executed"
						Write-Output $response.StatusCode
					}
					else {
						Write-Verbose $check
						Write-Output $check
					}
				}

				if ($Command -eq "downloadsamlspcert") {
					if ($check -eq "") {
						Write-Verbose "command successfully executed"
						$ans = [string]$response.StatusCode
						Write-Output $ans
					}
					else {
						Write-Verbose $check
						Write-Output $check
					}
				}
			}
		}
		if (-not ([String]::IsNullOrEmpty($response2)) -and ($response2.Contains("Unauthorized") -or $response2.Contains("Unable to connect")))
		{
			Write-Output $response2
		}
		Write-Verbose "cleaning connection objects"
		if ($request)
		{
			if ($loginCert)
			{
				$request.ClientCertificates.Remove($loginCert)
				$request.ClientCertificates.Clear()
			}
		}
		if ($reader)
		{
			$reader.Close()
			$reader.Dispose()
		}
		if ($stream)
		{
			$stream.Close()
			$stream.Dispose()
		}
		if ($fileStream)
		{
			$fileStream.Close()
			$fileStream.Dispose()
		}
		if ($response)
		{
			$response.Close()
			$response.Dispose()
		}
	}
}

# Internal usage only
Function enableAPIViaCertificate($LMAddress, $LBPort, $SubjectCN, $CertStore)
{
	Write-Verbose "Enabling API using certificate."
	if ($LBPort -eq "") {
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation))) {
			$en = Enable-SecAPIAccess -LoadBalancer $LMAddress -SubjectCN $SubjectCN -CertificateStoreLocation $CertStore
		}
		else {
			$en = Enable-SecAPIAccess -LoadBalancer $LMAddress -SubjectCN $SubjectCN
		}
	}
	else {
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation))) {
			$en = Enable-SecAPIAccess -LoadBalancer $LMAddress -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertStore
		}
		else {
			$en = Enable-SecAPIAccess -LoadBalancer $LMAddress -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	$en
}

# Internal usage only
Function enableAPIViaCredential($LMAddress, $LBPort, $Credential)
{
	Write-Verbose "Enabling API via credential."
	if ($LBPort -eq "") {
		$en = Enable-SecAPIAccess -LoadBalancer $LMAddress -Credential $Credential
	}
	else {
		$en = Enable-SecAPIAccess -LoadBalancer $LMAddress -Credential $Credential -LBPort $LBPort
	}
	$en
}

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Initialize-Lm
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$Address,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3, 65530)]
		[int]$LBPort = 443,

		[Parameter(ParameterSetName="Credential")]
		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = [System.Management.Automation.PSCredential]::Empty,

		[Parameter(ParameterSetName="Certificate")]
		[ValidateNotNullOrEmpty()]
		[String]$CertificateStoreLocation = $null,

		[Parameter(ParameterSetName="Certificate")]
		[ValidateNotNullOrEmpty()]
		[String]$SubjectCN = $null
	)

	$netAssembly = [Reflection.Assembly]::GetAssembly([System.Net.Configuration.SettingsSection])
	if ($netAssembly) {
		$bindingFlags = [Reflection.BindingFlags] "Static,GetProperty,NonPublic"
		$settingsType = $netAssembly.GetType("System.Net.Configuration.SettingsSectionInternal")

		$instance = $settingsType.InvokeMember("Section", $bindingFlags, $null, $null, @())
		if ($instance) {
			$bindingFlags = "NonPublic","Instance"
			$useUnsafeHeaderParsingField = $settingsType.GetField("useUnsafeHeaderParsing", $bindingFlags)
			if ($useUnsafeHeaderParsingField) {
				$useUnsafeHeaderParsingField.SetValue($instance, $true)
			}
		}
	}

	$cString = "Unless overridden using a local"
	if ($LBPort) {
		$script:LBAccessPort = $LBPort;
		Write-Output "$cString -Port parameter, all commands will now use HTTPS Port $($script:LBAccessPort)."
	}

	if ($Address) {
		if (-not (Test-ServerConnection -ComputerName $Address -Port $script:LBAccessPort)) {
			Throw "Unable to resolve or connect to $Address`:$LBPort"
			return
		}
		$script:LoadBalancerAddress = $Address
		Write-Output "$cString -LoadBalancer parameter, all commands will now be directed to $($script:LoadBalancerAddress)."
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN))) {
		$tempSubjecCN = $script:SubjectCommonName
		$script:SubjectCN = $SubjectCN
		Write-Output "$cString -SubjectCN parameter, all commands will now use `"$($script:SubjectCN)`" as login certificate."

		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation))) {
			$tempCertStoreLoc = $script:CertStoreLoc
			$script:CertificateStoreLocation = $CertificateStoreLocation
			Write-Output "$cString -CertificateStoreLocation parameter, all commands will now use `"$($script:CertificateStoreLocation)`" as location to search the login certificate."
		}
		$script:cred = $null
		Write-Output "Previous credential settings has been removed."
	}
	else {
		if ($Credential -ne [System.Management.Automation.PSCredential]::Empty) {
			$script:cred = $Credential
			$script:SubjectCN = $null
			$script:CertificateStoreLocation = $null
			Write-Output "$cString -Credential parameter, all commands will now use `"$($script:cred.Username)`" as user."
			Write-Output "Previous certificate settings has been removed."
		}
		else {
			if (-not ([String]::IsNullOrEmpty($script:cred.Username))) {
				Write-Output "No changes made to stored credentials. All commands use `"$($script:cred.Username)`" as user."
			}
			else {
				Write-Output "No login method is set."
			}
		}
	}
}
New-Alias -Name Initialize-LoadBalancer -value Initialize-Lm -Description "Alias for Initialize-LoadBalancer command"
Export-ModuleMember -function Initialize-Lm -Alias Initialize-LoadBalancer

#Function Test-ServerConnection
Function Test-LmServerConnection
{
	[CmdletBinding()]
	Param(
		[string]$ComputerName,
		[Int32]$Port
	)
	$connection = New-Object Net.Sockets.TcpClient
	try {
		Write-Verbose -Message "Connecting to $ComputerName on $Port..."
		$connection.connect($ComputerName, $Port)
		Write-Verbose -Message "Connection Status: $($connection.Connected)"
		if ($connection.Connected)
		{
			return $true
		}
	}
	catch {
		Write-Verbose "$($_.Exception.Message)"
	}
	return $false
}
New-Alias -Name Test-ServerConnection -value Test-LmServerConnection -Description "Alias for Test-ServerConnection command"
Export-ModuleMember -function Test-LmServerConnection -Alias Test-ServerConnection

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Convert-XMLToPSObject
{
	[CmdletBinding()]
	Param(
		[Parameter(ValueFromPipeline=$true)]
		[System.Xml.XmlElement[]]$Element
	)
	
	BEGIN
	{
		$objectProperties = @{}
	}
	
	PROCESS
	{
		if($_)
		{
			$Element | ForEach-Object {
				Write-Verbose "Element = $($_.Name)"
				foreach ($child in $_.get_ChildNodes())
				{
					$property = $child.get_Name()
					$value = ""
					Write-Verbose "Child Type = $($child.GetType().Name)"
					if ($child -is [System.Xml.XmlElement])
					{
						Write-Verbose "Trying to get child nodes."
						$value = @($child.get_ChildNodes() | ForEach-Object {$_.InnerText}) -join ","
					}
					else
					{
						Write-Verbose "Caught exception."
						$value = $child.InnerText
					}
					Write-Verbose "Property: $property; Value: $value"
					if ($value -ne "-")
					{
						if ($objectProperties.ContainsKey($property))
						{
							$objectProperties[$property] += ",$value"
						}
						else
						{
							$objectProperties[$property] = $value
						}
					}
				}
				$objectProperties.Remove("#whitespace")
				New-Object -TypeName PSObject -Property $objectProperties
				$objectProperties.Clear()
			}
		}
	}
}

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Convert-BoundParameters
{
	[CmdletBinding()]
	Param(
		[hashtable]$hashtable
	)

	$propertyTable = @{}

	foreach ($param in $IgnoredParameters)
	{
		$hashtable.Remove($param)
	}

	foreach ($param in $hashtable.keys)
	{
		if ($hashtable[$param] -is [bool])
		{
			$propertyTable.Add(($param), $hashtable[$param] -as [int])
		}
		else {
			$value = [System.Web.HttpUtility]::UrlEncode($hashtable[$param])
			if ($param -eq "BondMode")
			{
				if ($hashtable[$param].ToLowerInvariant() -eq "802.3ad") {$value = 4}
				if ($hashtable[$param].ToLowerInvariant() -eq "active-backup") {$value = 1}
			}

			if ($ParamReplacement.Keys -contains $param)
			{
				# Special case: Add the "!" prefix to the RSIndex value.
				if ($param -eq "RSIndex")
				{
					$value = "!" + $value
				}
				$param = $ParamReplacement[$param]
			}
			$propertyTable.Add($param, $value)
		}
	}
	return $propertyTable
}

# use internal
Function isSecApiEnabled($LoadBalancer, $port, $SubjectCN, $CertificateStoreLocation, $Credential)
{
	if (([String]::IsNullOrEmpty($port)))
	{
		$port = 443
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$isApiEn = Test-SecAPIAccess -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $port
		}
		else {
			$isApiEn = Test-SecAPIAccess -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $port
		}
	}
	else {
		$isApiEn = Test-SecAPIAccess -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $port
	}
	$isApiEn
}

# use internal
Function setEnableApiUrl($port)
{
	$sb = New-Object Text.StringBuilder

	if ($port -eq "")
 	{
		[void]$sb.Append("https://$LoadBalancer/progs/doconfig/enableapi/set/yes")
	}
	else {
		[void]$sb.Append("https://$LoadBalancer`:$port/progs/doconfig/enableapi/set/yes")
	}
	$uri = $sb.ToString() -replace "\&$"
	$uri
}

#Function EnableAPI
Function Enable-SecAPIAccess
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[ValidateRange(1,65535)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[Parameter(ParameterSetName="Credential")]
		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$ErrorActionPreference = "Stop"
	$response = $null

	if ($LBPort -eq "")
	{
		$LBPort = 443
	}

	Write-Verbose "LM port: $LBPort"
	Write-Verbose "Testing if the APIs are already enabled"
	$isApiEn = isSecApiEnabled  $LoadBalancer $LBPort $SubjectCN $CertificateStoreLocation $Credential
	if ($isApiEn.ReturnCode -eq 200) {
		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "API already enabled"
		$tempApiRetObj.Data = $null

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	else {
		if ( ($isApiEn.Response).Contains("Method Not Allowed") )
		{
			# ERROR: the LM is not licensed yet
			Write-Verbose "ERROR: the LM is not licensed yet [$($isApiEn.Response)]."
			$tempApiRetObj = [ordered]@{}
			$tempApiRetObj.PSTypeName = "KempAPI"
			$tempApiRetObj.ReturnCode = 400
			$tempApiRetObj.Response = "The LM is not licensed yet."
			$tempApiRetObj.Data = $null

			$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj

			return $apiRetObject
		}
		if ( ($isApiEn.Response).Contains("check username") )
		{
			throw "ERROR: $($isApiEn.Response)"
		}
		Write-Verbose "APIs are NOT enabled. Going to enable"

		Start-Sleep -s 3

		Write-Verbose "OK, moving on"
		$cmdToExec = "set?param=enableapi&value=yes"
		try {
			$response = sendRequest2LM $cmdToExec $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

			if (-not ([String]::IsNullOrEmpty($response))) {

				Write-Verbose "Enable API answer [$response]"

				if ($response -eq "ok") {
					# OK
					$tempApiRetObj = [ordered]@{}
					$tempApiRetObj.PSTypeName = "KempAPI"
					$tempApiRetObj.ReturnCode = 200
					$tempApiRetObj.Response = "API successfully enabled"
					$tempApiRetObj.Data = $null

					$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj

					$apiRetObject
				}
				else {
					# ERROR
					$tempApiRetObj = [ordered]@{}
					$tempApiRetObj.PSTypeName = "KempAPI"
					$tempApiRetObj.ReturnCode = 422
					$tempApiRetObj.Response = "API not enabled. Error: " + $response
					$tempApiRetObj.Data = $null

					$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj

					$apiRetObject
				}
			}
			else {
				Throw "ERROR: NULL answer"
			}
		}
		catch {
			Throw $_.Exception.Message
		}
	}
}
New-Alias -Name EnableAPI -value Enable-SecAPIAccess -Description "Alias for EnableAPI command"
Export-ModuleMember -function Enable-SecAPIAccess -Alias EnableAPI

#Function DisableAPI
Function Disable-SecAPIAccess
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[ValidateRange(1,65535)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[Parameter(ParameterSetName="Credential")]
		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$ErrorActionPreference = "Stop"
	if ($LBPort -eq "")
	{
		$LBPort = 443
	}

	Write-Verbose "Testing if the APIs are already disabled"
	$isApiEn = isSecApiEnabled  $LoadBalancer $LBPort $SubjectCN $CertificateStoreLocation $Credential
	Write-Verbose "isApiEn [$isApiEn]"
	if ($isApiEn.ReturnCode -eq 404) {

		Write-Verbose "Return code [$($isApiEn.ReturnCode)]"
		$retCode = 200
		if ($isApiEn.Response -eq "API is disabled.") {
			$msg = "API already disabled"	# or wrong credentials (login/password or certificate)"
		}
		else {
			if ($isApiEn.ReturnCode -eq 404)
			{
				$retCode = 404
			}
			else {
				$retCode = $isApiEn.ReturnCode
			}
			$msg = $isApiEn.Response
		}

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = $retCode
		$tempApiRetObj.Response = $msg
		$tempApiRetObj.Data = $null

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	else {
		$iter = 1
		if (-not ([String]::IsNullOrEmpty($SubjectCN))) {
			$iter = 5
		}
		Write-Verbose "waiting for LM"
		for ($i=1; $i -le $iter; $i++)
		{
			Start-Sleep -s 2
		}
		Write-Verbose "OK, moving on"
		$cmdToExec = "set?param=enableapi&value=no&"
		$response = sendRequest2LM $cmdToExec $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		if (-not ([String]::IsNullOrEmpty($response))) {
			Write-Verbose "answer from LM [$response]"
		}

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "API successfully disabled"
		$tempApiRetObj.Data = $null

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
}
New-Alias -Name DisableAPI -value Disable-SecAPIAccess -Description "Alias for DisableAPI command"
Export-ModuleMember -function Disable-SecAPIAccess -Alias DisableAPI

# Legacy: should be not used
Function IsAPIEnabled
{
	[CmdletBinding()]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	Send-LBMessage -Command isapienabled -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
}
Export-ModuleMember -function IsAPIEnabled

Function Test-SecAPIAccess
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[ValidateRange(1,65535)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if ($LBPort -eq "") {
		$LBPort = 443
	}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	try {
		$response = sendRequest2LM "isapienabled2" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		[string]$r_string = $response | Convert-XmlToPSObject

		$eqp = $r_string.IndexOf("=")
		$lbp = $r_string.IndexOf("}")

		$version = $r_string.substring($eqp + 1, $lbp - $eqp - 1)

		$tempApiVs = @{}
		$tempApiVs.PSTypeName = "Version"
		$tempApiVs.ApiVersion = $version

		$apiVers = New-Object -TypeName PSObject -Prop $tempApiVs

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "API is enabled"
		$tempApiRetObj.Data = $apiVers
	}
	catch [Exception] {
		$exceptionMsg = [string]$($_.Exception.Message)
		Write-Verbose "[Test-SecAPIAccess] Exception catched [$exceptionMsg]"
		if ($exceptionMsg -eq "The remote server returned an error: (404) Not Found.")
		{
			$errMsg = "API are disabled"
		}
		elseif ($exceptionMsg.Contains("ERROR: Can't find certificate"))
		{
			throw $exceptionMsg
		}
		else {
			$errMsg = $exceptionMsg
		}

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 404
		$tempApiRetObj.Response = $errMsg	# "API are disabled"
		$tempApiRetObj.Data = $null
	}
	finally {
		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
}
Export-ModuleMember -function Test-SecAPIAccess

#endregion - LBCommunication

#region - Virtual Service

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Get-AllParameters
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = @{}

	sendRequest2LM "getall" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
Export-ModuleMember -function Get-AllParameters

#Function Get-Parameter
Function Get-LmParameter
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Param,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "get" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Get-Parameter -value Get-LmParameter -Description "Alias for Get-Parameter command"
Export-ModuleMember -function Get-LmParameter -Alias Get-Parameter

#Function Set-Parameter
Function Set-LmParameter
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Param,

		[Parameter(Mandatory=$true,Position=1)]
		[AllowEmptyString()]
		[String]$Value,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN

	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "set" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Set-Parameter -value Set-LmParameter -Description "Alias for Set-Parameter command"
Export-ModuleMember -function Set-LmParameter -Alias Set-Parameter

Function ListIfconfig
{
	#[CmdletBinding()]
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = @{}

	$response = sendRequest2LM "listifconfig" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response | Convert-XmlToPSObject
}
Export-ModuleMember -function ListIfconfig

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function New-VirtualService
Function New-AdcVirtualService
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[string]$VirtualService,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[Parameter(Mandatory=$true,Position=2)]
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[ValidateRange(0,6)]
		[Int16]$AddVia,
		[string]$Template,
		[bool]$Cache = $false,
		[string]$CertFile,
		[ValidateSet("tcp","icmp","https","http","smtp","nntp","ftp","telnet","pop3","imap","rdp","ldap","none")]
		[string]$CheckType = "tcp",
		[string]$CheckHost,
		[string]$CheckPattern,
		[string]$CheckUrl,
		[string]$CheckHeaders,
		[string]$LdapEndpoint,
		[string]$MatchLen,
		[ValidateRange(0,1)]
		[int]$CheckUse1_1 = -1,
		[Int32]$CheckPort,
		[bool]$EnhancedHealthChecks,
		[Int32]$RsMinimum,
		[ValidateRange(0,2)]
		[Int16]$ClientCert = 0,
		[bool]$Compress = $false,
		[string]$Cookie,
		[ValidateRange(0,100)]
		[Int16]$CachePercent = 0,
		[string]$DefaultGW,
		[bool]$Enable = $true,
		[string]$ErrorCode = 0,
		[string]$ErrorUrl,
		[ValidateRange(3,65530)]
		[Int32]$PortFollow,
		[bool]$ForceL7 = $true,
		[ValidateRange(0,86400)]
		[Int32]$Idletime,
		[String[]]$LocalBindAddresses,
		[ValidateSet("gen","http","tls","ts")]
		[string]$VSType,
		[string]$Nickname,
		[ValidateSet("ssl","cookie","active-cookie","cookie-src","active-cook-src","cookie-hash",
		            "url","query-hash","host","header","super","super-src","src","rdp","rdp-src",
		            "rdp-sb","udpsip", "none")]
		[string]$Persist,
		[ValidateRange(0, 604800)]
		[Int32]$PersistTimeout,
		[string]$QueryTag,
		[string]$CipherSet,
		[bool]$SSLReencrypt,
		[bool]$SSLReverse,
		[ValidateSet("","http","https")]
		[string]$SSLRewrite,
		[string]$ReverseSNIHostname,
		[ValidateSet("rr","wrr","lc","wlc","fixed","adaptive","sh")]
		[string]$Schedule,
		[ValidateRange(0,5)]
		[Int16]$ServerInit,
		[bool]$SSLAcceleration,
		[string]$StandByAddr,
		[string]$StandByPort,
		[Int32]$TransactionLimit,
		[bool]$Transparent,
		[bool]$SubnetOriginating,
		[bool]$UseforSnat,
		[ValidateSet("0","1","2","4","8")]
		[string]$QoS,
		[int32]$CheckUseGet,
		[ValidateRange(0,7)]
		[Int16]$Verify,
		[string]$ExtraHdrKey,
		[string]$ExtraHdrValue,
		[string]$AllowedHosts,
		[string]$AllowedDirectories,
		[string]$AllowedGroups,
		[string]$GroupSIDs,
		[bool]$IncludeNestedGroups,
		[bool]$DisplayPubPriv,
		[bool]$DisablePasswordForm,
		[string]$Domain,
		[string]$AltDomains,
		[string]$Logoff,
		[ValidateRange(0,7)]
		[Int16]$ESPLogs,
		[string]$SMTPAllowedDomains,
		[bool]$ESPEnabled,
		[string]$UserPwdChangeUrl,
		[string]$UserPwdChangeMsg,
		[ValidateRange(0,5)]
		[Int16]$InputAuthMode,
		[ValidateRange(0,2)]
		[Int16]$OutputAuthMode,
		[ValidateRange(0,1)]
		[Int16]$StartTLSMode,
		[string]$ExtraPorts,
		[string]$AltAddress,
		[bool]$MultiConnect,
		[string]$SingleSignOnDir,
		[string]$OCSPVerify,
		[Int32]$FollowVSID,
		[ValidateRange(0, 14)]
		[int]$TlsType = 0,
		[string]$CheckPostData,
		[string]$CheckCodes,
		[string]$PreProcPrecedence,
		[Int16]$PreProcPrecedencePos,
		[string]$RequestPrecedence,
		[Int16]$RequestPrecedencePos,
		[string]$ResponsePrecedence,
		[Int16]$ResponsePrecedencePos,
		[string]$RsRulePrecedence,
		[Int16]$RsRulePrecedencePos,
		[bool]$NeedHostName,
		[string]$CopyHdrFrom = "",
		[string]$CopyHdrTo = "",

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	if ($CheckUse1_1 -ne -1) {
		$propertyTable.Remove("CheckUse1_1")
		if ($CheckUse1_1 -eq 1) {
			$propertyTable.Add("CheckUse1.1", 1)
		}
		else {
			$propertyTable.Add("CheckUse1.1", 0)
		}
	}
	sendRequest2LM "addvs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name New-VirtualService -value New-AdcVirtualService -Description "Alias for New-VirtualService command"
Export-ModuleMember -function New-AdcVirtualService -Alias New-VirtualService

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function New-SubVirtualService
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$VirtualService,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[Parameter(Mandatory=$true,Position=2)]
		[ValidateSet("tcp","udp")]
		[string]$Protocol,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = @{}
	$propertyTable.Add("vs", $VirtualService)
	$propertyTable.Add("port", $Port)
	$propertyTable.Add("prot", $Protocol)
	$propertyTable.Add("createsubvs", "")

	sendRequest2LM "modvs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
Export-ModuleMember -function New-SubVirtualService

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function New-SubVirtualServiceByID
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[Int32]$VSIndex,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = @{}
	$propertyTable.Add("vs", $VSIndex)
	$propertyTable.Add("createsubvs", "")

	sendRequest2LM "modvs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
Export-ModuleMember -function New-SubVirtualServiceByID

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-VirtualService
Function Remove-AdcVirtualService
{
	[cmdletbinding(SupportsShouldProcess=$true, DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Position=0,Mandatory=$true,ParameterSetName="IPAddress",ValueFromPipelineByPropertyName=$true)]
		[Parameter(ParameterSetName="Credential")]
		[Parameter(ParameterSetName="Certificate")]
		[Alias("VSAddress")]
		[string]$VirtualService,

		[Parameter(Position=1,Mandatory=$true,ParameterSetName="IPAddress",ValueFromPipelineByPropertyName=$true)]
		[Parameter(ParameterSetName="Credential")]
		[Parameter(ParameterSetName="Certificate")]
		[ValidateRange(3,65530)]
		[Int32]$Port,

		[Parameter(Position=2,Mandatory=$true,ParameterSetName="IPAddress",ValueFromPipelineByPropertyName=$true)]
		[Parameter(ParameterSetName="Credential")]
		[Parameter(ParameterSetName="Certificate")]
		[ValidateSet("tcp","udp")]
		[string]$Protocol,

		[Parameter(Position=0,Mandatory=$true,ParameterSetName="Index",ValueFromPipelineByPropertyName=$true)]
		[Parameter(ParameterSetName="Credential")]
		[Parameter(ParameterSetName="Certificate")]
		[Int32]$VSIndex = -1,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[switch]$Force
	)

	PROCESS
	{
		if ($VSIndex -ge 0) {
			Write-Verbose "VSIndex not null [$VSIndex]"
			if ($VirtualService) {
				Throw "The VSIndex and VirtualService parameters are mutually exclusive."
			}
			if ($Port) {
				Throw "The VSIndex and Port parameters are mutually exclusive."
			}
			if ($Protocol) {
				Throw "The VSIndex and Protocol parameters are mutually exclusive."
			}
			$cmdCase = "Index"
		}
		Elseif ($VirtualService) {
			Write-Verbose "VSIndex not set [$VSIndex]"
			if (!$Port -or !$Protocol) {
				Throw "The VirtualService, Port and Protocol parameters must be used together."
			}
			$cmdCase = "IPAddress"
		}
		else {
			Write-Verbose "No parameters"
			Throw "Either the VirtualService or VSIndex parameter is required."
		}

		switch ($cmdCase)
		{
			"IPAddress"
			{
				Write-Verbose "IPAddress case"
				if (-not ([String]::IsNullOrEmpty($SubjectCN)))
				{
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
					{
						$service = Get-AdcVirtualService -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $LBPort | Where-Object{(($_.VSAddress -eq $VirtualService) -and ($_.VSPort -eq $Port) -and ($_.Protocol -eq $Protocol))}
					}
					else {
						$service = Get-AdcVirtualService -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $LBPort | Where-Object{(($_.VSAddress -eq $VirtualService) -and ($_.VSPort -eq $Port) -and ($_.Protocol -eq $Protocol))}
					}
				}
				else {
					$service = Get-AdcVirtualService -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort | Where-Object{(($_.VSAddress -eq $VirtualService) -and ($_.VSPort -eq $Port) -and ($_.Protocol -eq $Protocol))}
				}
			}
			"Index"
			{
				Write-Verbose "Index case"
				if (-not ([String]::IsNullOrEmpty($SubjectCN)))
				{
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
					{
						$service = Get-AdcVirtualService -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $LBPort | Where-Object{$_.Index -eq $VSIndex}
					}
					else {
						$service = Get-AdcVirtualService -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $LBPort | Where-Object{$_.Index -eq $VSIndex}
					}
				}
				else {
					$service = Get-AdcVirtualService -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort | Where-Object{$_.Index -eq $VSIndex}
				}
			}
			default {
				Throw "Not valid case."
			}
		}

		if ($service)
		{
			Write-Verbose "Working with $($service.VSAddress)"
			$propertyTable = @{
				vs=$($service.VSAddress)
				port=$($service.VSPort)
				prot=$($service.Protocol)
			}

			if (($Force) -or ($PsCmdlet.ShouldProcess("$($VirtualService)/$($Port)/$($Protocol)", "Remove Virtual Service")))
			{
				Write-Verbose "Committing change."
				$response = sendRequest2LM "delvs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
				$response
			}
		}
		else {
			switch ($cmdCase)
			{
				"IPAddress"
				{
					Write-Error "The Virtual Service at `"$($propertyTable.vs)`" using $($propertyTable.prot)/$($propertyTable.port) could not be found. Has it already been deleted?"
					break
				}
				"Index"
				{
					Write-Error "The Virtual Service with index `"$VSIndex`" could not be found."
				}
				default {
					Throw "Unknown error."
				}
			}
		}
	}
}
New-Alias -Name Remove-VirtualService -value Remove-AdcVirtualService -Description "Alias for Remove-VirtualService command"
Export-ModuleMember -function Remove-AdcVirtualService -Alias Remove-VirtualService

#Function Remove-VirtualServiceByID
Function Remove-AdcVirtualServiceByID
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[Alias("Vs")]
		[Int32]$VSIndex,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "delvs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Remove-VirtualServiceByID -value Remove-AdcVirtualServiceByID -Description "Alias for Remove-VirtualServiceByID command"
Export-ModuleMember -function Remove-AdcVirtualServiceByID -Alias Remove-VirtualServiceByID

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-VirtualService
Function Get-AdcVirtualService
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[String]$VirtualService,
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[ValidateSet("tcp","udp")]
		[String]$Protocol,
		[Int32]$VSIndex,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if ($VSIndex) {
		if ($VirtualService) {
			Throw "The VSIndex and VirtualService parameters are mutually exclusive."
		}
		if ($Port) {
			Throw "The VSIndex and Port parameters are mutually exclusive."
		}
		if ($Protocol) {
			Throw "The VSIndex and Protocol parameters are mutually exclusive."
		}
	} Elseif ($VirtualService) {
		if (!$Port -or !$Protocol) {
			Throw "The VirtualService, Port and Protocol parameters must be used together."
		}
	} else {
		if ($Port -or $Protocol) {
			Throw "The VirtualService, Port and Protocol parameters must be used together."
		}
	}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if ($VirtualService -or $VSIndex) {
		$response = sendRequest2LM "showvs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		if ($response -is "String") {
			$response
		}
	 	else {
			$response | Convert-XmlToPSObject
		}
	}
 	else {	
		$response = sendRequest2LM "listvs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		$response.VS | Convert-XmlToPSObject
	}
}
New-Alias -Name Get-VirtualService -value Get-AdcVirtualService -Description "Alias for Get-VirtualService command"
Export-ModuleMember -function Get-AdcVirtualService -Alias Get-VirtualService

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-VirtualService
Function Set-AdcVirtualService
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$VirtualService,
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Int32]$VSIndex,
		[Int16]$AddVia,
		[bool]$Cache,
		[string]$CertFile,
		[ValidateSet("tcp","icmp","https","http","smtp","nntp","ftp","telnet","pop3","imap","rdp","ldap","none")]
		[string]$CheckType,
		[string]$CheckHost,
		[string]$CheckPattern,
		[string]$CheckUrl,
		[string]$CheckHeaders,
		[string]$LdapEndpoint,
		[string]$MatchLen,
		[ValidateRange(0,1)]
		[int]$CheckUse1_1 = -1,
		[Int32]$CheckPort,
		[bool]$EnhancedHealthChecks,
		[Int32]$RsMinimum,
		[ValidateRange(0,2)]
		[Int16]$ClientCert,
		[bool]$Compress,
		[string]$Cookie,
		[ValidateRange(0,100)]
		[Int16]$CachePercent,
		[string]$DefaultGW,
		[bool]$Enable,
		[string]$ErrorCode,
		[string]$ErrorUrl,
		[ValidateRange(3,65530)]
		[Int32]$PortFollow,
		[bool]$ForceL7,
		[ValidateRange(0,86400)]
		[Int32]$Idletime,
		[String[]]$LocalBindAddresses,
		[ValidateSet("gen","http","tls","ts")]
		[string]$VSType,
		[string]$Nickname,
		[ValidateSet("none", "ssl","cookie","active-cookie","cookie-src","active-cook-src","cookie-hash",
		             "url","query-hash","host","header","super","super-src","src","rdp","rdp-src",
		             "rdp-sb","udpsip")]
		[string]$Persist,
		[ValidateRange(0, 604800)]
		[Int32]$PersistTimeout,
		[string]$QueryTag,
		[string]$CipherSet,
		[bool]$SSLReencrypt,
		[bool]$SSLReverse,
		[ValidateSet("","http","https")]
		[string]$SSLRewrite,
		[string]$ReverseSNIHostname,
		[ValidateSet("rr","wrr","lc","wlc","fixed","adaptive","sh")]
		[string]$Schedule,
		[ValidateRange(0,5)]
		[Int16]$ServerInit,
		[bool]$SSLAcceleration,
		[string]$StandByAddr,
		[string]$StandByPort,
		[Int32]$TransactionLimit,
		[bool]$Transparent,
		[bool]$SubnetOriginating,
		[bool]$UseforSnat,
		[ValidateSet("0","1","2","4","8")]
		[string]$QoS,
		[int32]$CheckUseGet,
		[ValidateRange(0,7)]
		[Int16]$Verify,
		[string]$ExtraHdrKey,
		[string]$ExtraHdrValue,
		[string]$AllowedHosts,
		[string]$AllowedDirectories,
		[string]$AllowedGroups,
		[string]$GroupSIDs,
		[bool]$IncludeNestedGroups,
		[bool]$DisplayPubPriv,
		[bool]$DisablePasswordForm,
		[string]$Domain,
		[string]$AltDomains,
		[string]$Logoff,
		[ValidateRange(0,7)]
		[Int16]$ESPLogs,
		[string]$SMTPAllowedDomains,
		[bool]$ESPEnabled,
		[string]$UserPwdChangeUrl,
		[string]$UserPwdChangeMsg,
		[ValidateRange(0,5)]
		[Int16]$InputAuthMode,
		[ValidateRange(0,1)]
		[Int16]$OutputAuthMode,
		[ValidateRange(0,1)]
		[Int16]$StartTLSMode,
		[string]$ExtraPorts,
		[string]$AltAddress,
		[bool]$MultiConnect,
		[string]$SingleSignOnDir,
		[string]$OCSPVerify,
		[Int32]$FollowVSID,
		[ValidateRange(0, 14)]
		[int]$TlsType = 0,
		[string]$CheckPostData,
		[string]$CheckCodes,
		[string]$PreProcPrecedence,
		[Int16]$PreProcPrecedencePos,
		[string]$RequestPrecedence,
		[Int16]$RequestPrecedencePos,
		[string]$ResponsePrecedence,
		[Int16]$ResponsePrecedencePos,
		[string]$RsRulePrecedence,
		[Int16]$RsRulePrecedencePos,
		[bool]$NeedHostName,
		[string]$CopyHdrFrom,
		[string]$CopyHdrTo,
		[string]$ServerFbaPath,
		[string]$ServerFbaPost,
		[bool]$AllowHTTP2,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[bool]$Intercept,
		[ValidateNotNullOrEmpty()]
		[string]$InterceptOpts,
		[ValidateNotNullOrEmpty()]
		[string]$InterceptRules,
		[ValidateRange(0, 100000)]
		[int32]$AlertThreshold,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if ($VSIndex) {
		if ($VirtualService) {
			Throw "The VSIndex and VirtualService parameters are mutually exclusive."
		}
		if ($Port) {
			Throw "The VSIndex and Port parameters are mutually exclusive."
		}
		if ($Protocol) {
			Throw "The VSIndex and Protocol parameters are mutually exclusive."
		}
	}
 	Elseif ($VirtualService) {
		if (!$Port -or !$Protocol) {
			Throw "The VirtualService, Port and Protocol parameters must be used together."
		}
	}
 	else {
		Throw "Either the VirtualService or VSIndex parameter is required."
	}

	if ($ServerFbaPost -and -not ($ServerFbaPath) ) {
		setKempAPIReturnObject 400 "ERROR: ServerFbaPath must be set in order to use ServerFbaPost parameter" $null
		return
	}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	if ($CheckUse1_1 -ne -1) {
		$propertyTable.Remove("CheckUse1_1")
		if ($CheckUse1_1 -eq 1) {
			$propertyTable.Add("CheckUse1.1", 1)
		}
		else {
			$propertyTable.Add("CheckUse1.1", 0)
		}
	}

	sendRequest2LM "modvs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Set-VirtualService -value Set-AdcVirtualService -Description "Alias for Set-VirtualService command"
Export-ModuleMember -function Set-AdcVirtualService -Alias Set-VirtualService

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Get-VSTotals
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = @{}

	$response = sendRequest2LM "vstotals" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	if ($response.code -eq "fail") {
		$response.Error
	} else {
		$response.VSTotals | Add-Member -MemberType NoteProperty -Name Type -Value "VSTotals"
		$response.SUBVSTotals | Add-Member -MemberType NoteProperty -Name Type -Value "SubVSTotals"
		$response.RSTotals | Add-Member -MemberType NoteProperty -Name Type -Value "RSTotals"
		Write-Output $response.VSTotals
		Write-Output $response.SUBVSTotals
		Write-Output $response.RSTotals
	}
}
Export-ModuleMember -function Get-VSTotals

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Get-VSPacketFilterACL
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[string]$VirtualService,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[Parameter(Mandatory=$true,Position=2)]
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Parameter(Mandatory=$true,Position=3)]
		[ValidateSet("black","white")]
		[String]$Type,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = @{}
	$propertyTable.Add("listvs", $Type)
	$propertyTable.Add("vsip", $VirtualService)
	$propertyTable.Add("vsport", $Port)
	$propertyTable.Add("vsprot", $Protocol)

	try {
		$response = sendRequest2LM "aclcontrol" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$exceptionMsg = $($_.Exception.Message)
		setKempAPIReturnObject 401 "$exceptionMsg" $null
		return
	}

	if ($response.code -eq "fail") {
		$errorString = $response.Error
		setKempAPIReturnObject 401 "$errorString" $null
		return
	}

	if ($response) {
		try {
			$x = $response | Get-Member -MemberType Method "Contains"
		}
		catch {
			$exceptionMsg = $($_.Exception.Message)
			setKempAPIReturnObject 401 "$exceptionMsg" $null
			return
		}
	}
	else {
		setKempAPIReturnObject 401 "System error: this error can be caused using a wrong Virtual service/port." $null
		return
	}

	if ($x.Name) {
		if ($response -is [array]) {
			$r_response = $response[0]
		}
		else {
			$r_response = $response
		}

		if ($r_response.Contains("Unauthorized") -or
				$r_response.Contains("Invalid address format") -or
				$r_response.Contains("Unable to connect") -or
				$r_response.Contains("Can't find certificate") -or
				$r_response.Contains("The remote name could") -or
				$r_response.Contains("unknown host")) {
			$errorString = $r_response
		}
		else {
			$errorString = "Unknow error"
		}
		setKempAPIReturnObject 401 "$errorString" $null
		return
	}

	try {
		$answer = $response.VS | Convert-XmlToPSObject
	}
	catch {
		setKempAPIReturnObject 401 "the response is not an XML objecy" $null
		return
	}

	$outputParams = [ordered]@{}
	$outputParams.Add("VS_IP", $answer.VS_IP)
	$outputParams.Add("VS_Port", $answer.VS_Port)
	$outputParams.Add("VS_Protocol", $answer.VS_Protocol)

	if ($Type -eq "black") {
		$list = $($response.VS).Blacklist
		if ($list) {
			$aa = $list | Convert-XmlToPSObject
		}
	}
	else {
		$list = $($response.VS).Whitelist
		if ($list) {
			$aa = $list | Convert-XmlToPSObject
		}
	}

	if ($list) {
		$ips = @()
		$aa.PSObject.Properties | Sort-Object | foreach-object {
			$tmp = [ordered]@{}
			$name  = $_.Name 
			$value = $_.value
			$values = $value -Split ','
			$x = $tmp.Add("Name", $name)
			$x = $tmp.Add("Addr", $values[0])
			$x = $tmp.Add("Comment", $values[1])
			$obj = New-Object -TypeName PSObject -Prop $tmp
			$ips += $obj
		}

		if ($Type -eq "black") {
			$outputParams.Add("Blacklist", $ips)
		}
		else {
			$outputParams.Add("Whitelist", $ips)
		}
	}

	if (-not $list) {
		$outputParams = $null
	}
	$apiRetObject = New-Object -TypeName PSObject -Prop $outputParams

	setKempAPIReturnObject 200 "Command successfully executed" $apiRetObject

}
Export-ModuleMember -function Get-VSPacketFilterACL

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Add-VSPacketFilterAddr
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[string]$VirtualService,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[Parameter(Mandatory=$true,Position=2)]
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Parameter(Mandatory=$true,Position=3)]
		[ValidateSet("black","white")]
		[String]$Type,
		[Parameter(Mandatory=$true,Position=4)]
		[ValidateNotNullOrEmpty()]
		[String]$Address,
		[String]$Comment,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = @{}
	$propertyTable.Add("addvs", $Type)
	$propertyTable.Add("vsip", $VirtualService)
	$propertyTable.Add("vsport", $Port)
	$propertyTable.Add("vsprot", $Protocol)
	$propertyTable.Add("addr", $Address)
	$propertyTable.Add("comment", [System.Uri]::EscapeDataString("""$Comment"""))

	try {
		$answer = sendRequest2LM "aclcontrol" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$exceptionMsg = $($_.Exception.Message)
		setKempAPIReturnObject 401 "$exceptionMsg" $null
		return
	}

	if ($answer -eq "ok") {
		setKempAPIReturnObject 200 "Command successfully executed" $null
	}
	else {
		if ($answer -is [array]) {
			$len = $answer.Length
			if ($len -eq 2) {
				setKempAPIReturnObject 401 "$($answer[0])" $null
				return
			}
			elseif ($len -eq 3) {
				setKempAPIReturnObject 401 "$($answer[2])" $null
				return
			}
			else {
				setKempAPIReturnObject 401 "Unknown error." $null
				return
			}
		}
		elseif ($answer -is [string]) {
			if ($answer.Contains("violation")) {
				setKempAPIReturnObject 401 "command too long" $null
			}
			else {
				setKempAPIReturnObject 401 "$answer" $null
			}
			return
		}
		else {
			setKempAPIReturnObject 401 "Unknown error." $null
			return
		}
	}
}
Export-ModuleMember -function Add-VSPacketFilterAddr

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Remove-VSPacketFilterAddr
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[string]$VirtualService,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[Parameter(Mandatory=$true,Position=2)]
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Parameter(Mandatory=$true,Position=3)]
		[ValidateSet("black","white")]
		[String]$Type,
		[Parameter(Mandatory=$true,Position=4)]
		[ValidateNotNullOrEmpty()]
		[String]$Address,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = @{}
	$propertyTable.Add("delvs", $Type)
	$propertyTable.Add("vsip", $VirtualService)
	$propertyTable.Add("vsport", $Port)
	$propertyTable.Add("vsprot", $Protocol)
	$propertyTable.Add("addr", $Address)

	sendRequest2LM "aclcontrol" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
Export-ModuleMember -function Remove-VSPacketFilterAddr

Function ExportVSTemplate
{	
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$VirtualService,
		[Parameter(Mandatory=$true)]
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[Parameter(Mandatory=$true)]
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[String]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$params = @{}
	$params.Add("vs", $VirtualService)
	$params.Add("port", $Port)
	$params.Add("prot", $Protocol)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Send-LBMessage -Command exportvstmplt -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Send-LBMessage -Command exportvstmplt -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Send-LBMessage -Command exportvstmplt -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
Export-ModuleMember -function ExportVSTemplate

#Function UploadTemplate
Function Install-Template
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command uploadtemplate -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command uploadtemplate -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command uploadtemplate -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
	$response
}
New-Alias -Name UploadTemplate -value Install-Template -Description "Alias for UploadTemplate command"
Export-ModuleMember -function Install-Template -Alias UploadTemplate

#Function DeleteTemplate
Function Remove-Template
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "deltemplate" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DeleteTemplate -value Remove-Template -Description "Alias for DeleteTemplate command"
Export-ModuleMember -function Remove-Template -Alias DeleteTemplate

#Function ListTemplates
Function Get-Template
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "listtemplates" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.template | Convert-XmlToPSObject
}
New-Alias -Name ListTemplates -value Get-Template -Description "Alias for ListTemplates command"
Export-ModuleMember -function Get-Template -Alias ListTemplates

#endregion - Virtual Service

#region - SSO Domains

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function New-SSODomain
{
	[CmdletBinding()]
	Param(
		[Parameter(Position=0,ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("Name")]
		[string]$Domain,
		[Parameter(Position=1,ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[ValidateSet("Unencrypted","StartTLS","LDAPS")]
		[string]$TLS,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[ValidateSet("LDAP-Unencrypted","LDAP-StartTLS","LDAP-LDAPS", "RADIUS", "RSA-SECURID", "Certificate", "KCD", "RADIUS and LDAP-Unencrypted", "RADIUS and LDAP-StartTLS", "RADIUS and LDAP-LDAPS", "RSA-SECURID and LDAP-Unencrypted", "RSA-SECURID and LDAP-StartTLS", "RSA-SECURID and LDAP-LDAPS", "SAML")]
		[string]$auth_type,		
		[Parameter(Position=2,ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[String[]]$Server,
		[Parameter(Position=0,ParameterSetName="Multiple",ValueFromPipeline=$true)]
		[PSCustomObject[]]$DomainObject,
		[ValidateNotNullOrEmpty()]
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("KerberosDomain")]
		[string]$kerberos_domain,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("KerberosKDC")]
		[string]$kerberos_kdc,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("KCDUsername")]
		[string]$kcd_username,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("KCDPassword")]
		[string]$kcd_password,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("ServerSide")]
		[string]$server_side,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("CertCheck")]
		[string]$cert_check_asi,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("CertCheckCn")]
		[string]$cert_check_cn,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("LogonTranscode")]
		[bool]$Logon_Transcode,
		[Alias("LdapEndpoint")]
		[string]$ldap_endpoint,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred,

		[ValidateNotNullOrEmpty()]
		[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[ValidateNotNullOrEmpty()]
		[String]$SubjectCN = $script:SubjectCN
	)

	PROCESS
	{
		$ErrorActionPreference = "Stop"
		
		if ( (-not ([String]::IsNullOrEmpty($SubjectCN))) -and (-not ([String]::IsNullOrEmpty($Credential))) )
		{
			Throw "ERROR: Can only set one login method, certificate or credential, not both."
		}

		switch ($PsCmdlet.ParameterSetName)
		{
			"Single"
			{
				Write-Verbose "Domain = $Domain"
				if (-not ([String]::IsNullOrEmpty($SubjectCN)))
				{
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
					{
						$ssoDomainName = Get-SSODomain -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation | Where-Object{$_.Name -eq $Domain}
					}
					else {
						$ssoDomainName = Get-SSODomain -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN | Where-Object{$_.Name -eq $Domain}
					}
				}
				else {
					$ssoDomainName = Get-SSODomain -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential | Where-Object{$_.Name -eq $Domain}
				}
				if (([String]::IsNullOrEmpty($ssoDomainName)))
				{
					$propertytable = Convert-BoundParameters $psboundparameters
					$response = sendRequest2LM "adddomain" $propertytable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
					if ($response -eq "ok")
					{
						$response = sendRequest2LM "moddomain" $propertytable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
						if ($response.haschildnodes)
						{
							$response.domain | Where-Object{$_.Name -eq $Domain}
						}
					}
				}
				else {
					Write-Error "The domain name already exists."
				}
				break
			}
			"Multiple"
			{
				foreach ($object in $DomainObject)
				{
					if (-not ([String]::IsNullOrEmpty($SubjectCN)))
					{
						if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
						{
							New-SSODomain -Domain $DomainObject.Domain -TLS $DomainObject.TLS -Server $DomainObject.Server -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
						}
						else {
							New-SSODomain -Domain $DomainObject.Domain -TLS $DomainObject.TLS -Server $DomainObject.Server -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
						}
					}
					else {
						New-SSODomain -Domain $DomainObject.Domain -TLS $DomainObject.TLS -Server $DomainObject.Server -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
					}
				}
				break
			}
		}
	}
}
Export-ModuleMember -function New-SSODomain

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Remove-SSODomain
{
	[cmdletbinding(SupportsShouldProcess=$true,ConfirmImpact="High",DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0,ValueFromPipeline=$true)]
		[Alias("Name")]
		[string[]]$Domain,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[switch]$Force
	)

	PROCESS
	{
		try {
			foreach ($name in $Domain)
			{
				if (-not ([String]::IsNullOrEmpty($SubjectCN)))
				{
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
					{
						$check = Get-SSODomain -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation | Where-Object{$_.Name -eq $name}
					}
					else {
						$check = Get-SSODomain -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN | Where-Object{$_.Name -eq $name}
					}
				}
				else {
					$check = Get-SSODomain -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential | Where-Object{$_.Name -eq $name}
				}

				if ($check)
				{
					if (($Force) -or ($PsCmdlet.ShouldProcess($name, "Remove SSO Domain")))
					{
						$response = sendRequest2LM "deldomain" @{domain=$name} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
						if ($response = "ok")
						{
							Write-Output "The SSO Domain `"$name`" has been removed."
						}
					}
				}
				else {
					Write-Error "The SSO Domain `"$name`" does not exist. Have you already deleted it?"
				}
			}
		}
		catch {
			Throw "ERROR: $($_.Exception.Message)"
		}
	}
}
Export-ModuleMember -function Remove-SSODomain

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Set-SSODomain
{
	[CmdletBinding()]
	Param(
		[Parameter(Position=0,ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("Name")]
		[string]$Domain,
		[Parameter(Position=1,ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[ValidateSet("Unencrypted","StartTLS","LDAPS")]
		[string]$TLS,
		[Parameter(Position=2,ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[String[]]$Server,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[String[]]$Server2,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[ValidateSet("LDAP-Unencrypted","LDAP-StartTLS","LDAP-LDAPS", "RADIUS", "RSA-SECURID", "Certificates", "KCD", "RADIUS and LDAP-Unencrypted", "RADIUS and LDAP-StartTLS", "RADIUS and LDAP-LDAPS", "RSA-SECURID and LDAP-Unencrypted", "RSA-SECURID and LDAP-StartTLS", "RSA-SECURID and LDAP-LDAPS", "SAML")]
		[string]$auth_type,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[ValidateRange(0,999)]
		[int]$max_failed_auths,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[ValidateRange(60,86400)]
		[int]$reset_fail_tout,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[ValidateRange(60,86400)]
		[int]$unblock_tout,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[ValidateSet("Notspecified","Not specified","Principalname","Username","Usernameonly","Username only")]
		[string]$logon_fmt,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[ValidateSet("Notspecified","Not specified","Principalname","Username")]
		[string]$logon_fmt2,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[string]$logon_domain,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("stt")][ValidateSet("idle time","max duration")]
		[string]$sess_tout_type,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("stipub")][ValidateRange(60,86400)]
		[int]$sess_tout_idle_pub,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("stdpub")][ValidateRange(60,86400)]
		[int]$sess_tout_duration_pub,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("stipriv")][ValidateRange(60,86400)]
		[int]$sess_tout_idle_priv,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("stdpriv")][ValidateRange(60,86400)]
		[int]$sess_tout_duration_priv,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[string]$testuser,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[string]$testpass,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("Secret")]
		[string]$radius_shared_secret,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("KerberosDomain")]
		[string]$kerberos_domain,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("SKerberosKDC")]
		[string]$kerberos_kdc,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("KCDUsername")]
		[string]$kcd_username,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("KCDPassword")]
		[string]$kcd_password,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("ServerSide")]
		[string]$server_side,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("CertCheck")]
		[string]$cert_check_asi,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("CertCheckCn")]
		[string]$cert_check_cn,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[Alias("LogonTranscode")]
		[bool]$Logon_Transcode,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]						
		[Alias("LdapEndpoint")]
		[string]$ldap_endpoint,
		[Parameter(ParameterSetName="Single",ValueFromPipelineByPropertyName=$true)]
		[PSCustomObject[]]$DomainObject,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred,

		[ValidateNotNullOrEmpty()]
		[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[ValidateNotNullOrEmpty()]
		[String]$SubjectCN = $script:SubjectCN
	)

	BEGIN
	{
		# We allow logon_fmt values with or without a space, but the
		# RESTful API requires a space character for each of these.
		if ($logon_fmt -eq "Notspecified")
		{
			$psboundparameters.Remove("logon_fmt")
			$psboundparameters.Add("logon_fmt", "Not specified")
		}
		Elseif ($logon_fmt -eq "Usernameonly")
		{
			$psboundparameters.Remove("logon_fmt")
			$psboundparameters.Add("logon_fmt", "Username only")
		}

		if ($logon_fmt2 -eq "Notspecified")
		{
			$psboundparameters.Remove("logon_fmt2")
			$psboundparameters.Add("logon_fmt2", "Not specified")
		}
	}

	PROCESS
	{
		$ErrorActionPreference = "Stop"

		if ( (-not ([String]::IsNullOrEmpty($SubjectCN))) -and (-not ([String]::IsNullOrEmpty($Credential))) )
		{
			Throw "ERROR: Can only set one login method, certificate or credential, not both."
		}

		switch ($PsCmdlet.ParameterSetName)
		{
			"Single"
			{
				Write-Verbose "Domain = $Domain"

				if (-not ([String]::IsNullOrEmpty($SubjectCN)))
				{
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
					{
						$response = Send-LBMessage -command showdomain -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
					}
					else {
						$response = Send-LBMessage -command showdomain -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
					}
				}
				else {
					$response = Send-LBMessage -command showdomain -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
				}
				if ($response.hasChildNodes)
				{
					$ssoDomainName = $response.Domain | Where-Object{$_.Name -eq $Domain}
				}

				if (-not ([String]::IsNullOrEmpty($ssoDomainName)))
				{
					$propertytable = Convert-BoundParameters $psboundparameters
					$response = sendRequest2LM "moddomain" $propertytable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
					if ($response.haschildnodes)
					{
						$answer = $response.domain | Where-Object{$_.Name -eq $Domain}
						if (-not ([String]::IsNullOrEmpty($answer)))
						{
							$answer
						}
						else {
							Throw "ERROR: $response"
						}
					}
				}
				else {
					if ($response -is [string] -and (-not ([String]::IsNullOrEmpty($response)) -and ($response.Contains("Unauthorized") -or $response.Contains("Unable to connect"))))
					{
						Throw "ERROR: $response"
					}
					else {
						Write-Error "The domain name doesn't exist.  Did you mean to execute New-SSODomain instead?"
					}
				}
				break
			}
			"Multiple"
			{
				foreach ($object in $DomainObject)
				{
					if (-not ([String]::IsNullOrEmpty($SubjectCN)))
					{
						if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
						{
							Set-SSODomain $DomainObject.Domain -TLS $DomainObject.TLS -Server $DomainObject.Server -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
						}
						else {
							Set-SSODomain $DomainObject.Domain -TLS $DomainObject.TLS -Server $DomainObject.Server -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
						}
					}	
					else {
						Set-SSODomain $DomainObject.Domain -TLS $DomainObject.TLS -Server $DomainObject.Server -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
					}
				}
				break
			}
		}
	}
}
Export-ModuleMember -function Set-SSODomain

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Get-SSODomain
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command showdomain -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command showdomain -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command showdomain -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($response.hasChildNodes)
	{
		$response.Domain | Convert-XmlToPSObject
	}
	else {
		if (-not ([String]::IsNullOrEmpty($response)) -and $response.Contains("Unable to connect")) {
			Throw $response
		}
	}
}
Export-ModuleMember -function Get-SSODomain

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-SSODomainLockedUsers
Function Get-SSODomainLockedUser
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$false,Position=0,ValueFromPipeline=$true)]
		[Alias("Name")]
		[string]$Domain,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	PROCESS
	{
		$propertytable = Convert-BoundParameters $psboundparameters

		$response = sendRequest2LM "showdomainlockedusers" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		if ($response.hasChildNodes)
		{
			$response.LockedUsers | Convert-XmlToPSObject
		}
	}
}
New-Alias -Name Get-SSODomainLockedUsers -value Get-SSODomainLockedUser -Description "Alias for Get-SSODomainLockedUsers command"
Export-ModuleMember -function Get-SSODomainLockedUser -Alias Get-SSODomainLockedUsers

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-SSODomainUnlockUsers
Function Set-SSODomainUnlockUser
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0,ValueFromPipeline=$true)]
		[Alias("Name")]
		[string]$Domain,
		[Parameter(Mandatory=$true,Position=1,ValueFromPipeline=$true)]
		[string]$Users,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	PROCESS
	{
		$propertytable = Convert-BoundParameters $psboundparameters

		$response = sendRequest2LM "unlockdomainusers" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		if ($response.hasChildNodes)
		{
			$response.UnlockedUsers | Convert-XmlToPSObject
		}
	}
}
New-Alias -Name Set-SSODomainUnlockUsers -value Set-SSODomainUnlockUser -Description "Alias for Set-SSODomainUnlockUsers command"
Export-ModuleMember -function Set-SSODomainUnlockUser -Alias Set-SSODomainUnlockUsers

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function UploadRSAConfigurationFile
Function Install-SSORSAConfigurationFile
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command setrsaconfig -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command setrsaconfig -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command setrsaconfig -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	$response
}
New-Alias -Name UploadRSAConfigurationFile -value Install-SSORSAConfigurationFile -Description "Alias for UploadRSAConfigurationFile command"
Export-ModuleMember -function Install-SSORSAConfigurationFile -Alias UploadRSAConfigurationFile

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function UploadRSANodeSecretAndPassword
Function Install-SSORSANodeSecretAndPassword
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Password,
		[Parameter(Mandatory=$true)]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}
	$params.Add("rsanspwd",[System.Web.HttpUtility]::UrlEncode($Password))

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command setrsanodesecret -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command setrsanodesecret -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command setrsanodesecret -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	$response
}
New-Alias -Name UploadRSANodeSecretAndPassword -value Install-SSORSANodeSecretAndPassword -Description "Alias for UploadRSANodeSecretAndPassword command"
Export-ModuleMember -function Install-SSORSANodeSecretAndPassword -Alias UploadRSANodeSecretAndPassword

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-SSODomainSessions
Function Get-SSODomainSession
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[Alias("Name")]
		[string]$Domain,
		[Parameter(Mandatory=$false,Position=1)]
		[string]$User,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$params = [ordered]@{}
	$params.Add("domain", $Domain)

	if (-not ([String]::IsNullOrEmpty($User))) {
		$params.Add("user", $User)
		$ssodcmd = "ssodomain/search"
	}
	else {
		$ssodcmd = "ssodomain/queryall"
	}

	try {
		$response = sendRequest2LM $ssodcmd $params $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$response = $_.Exception.Message
	}

	if($response.code -eq "fail" -Or !$response.hasChildNodes)
	{	# ERROR
		if ($response -is [array]) {
			if ($($response[1]).Error) {
				$lmResponse = $response[0] + " " + $($response[1]).Error
				$retCode = $($response[1]).stat
			}
			else {
				$lmResponse = $response[0]
				$retCode = 401
			}
			setKempAPIReturnObject $retCode "$lmResponse" $null
		}
		else {
			setKempAPIReturnObject 401 "$response" $null
		}
	}
	else 
	{	# OK
		$Sessions = @()

		$tmpNOS = @{}
		$tmpNOS.PSTypeName = "NOS"
		$tmpNOS.Add("NumberOfSessions",$response.NumberOfSessions);
		$Sessions += New-Object -TypeName PSObject -Prop $tmpNOS
        
		if ($response.NumberOfSessions -gt 0)
		{
			if ($response.Session) {
				$sobj = $response.Session | Convert-XmlToPSObject
				ForEach($session in $sobj) {
					$tmpObj = [ordered]@{}
					$tmpObj.PSTypeName = "SD"
					$tmpObj.Add("User", $session.User)
					$tmpObj.Add("Client", $session.Client)
					$tmpObj.Add("Destination", $session.Destination)
					$tmpObj.Add("Created", $session.Created)
					$tmpObj.Add("Expires", $session.Expires)
					$tmpObj.Add("Cookie", $session.Cookie)
					$Sessions += New-Object -TypeName PSObject -Prop $tmpObj
				}
			}
			else {
				setKempAPIReturnObject 401 "System error." $null
				return
			}
		}
		setKempAPIReturnObject 200 "Command successfully executed." $Sessions
	}
}
New-Alias -Name Get-SSODomainSessions -value Get-SSODomainSession -Description "Alias for Get-SSODomainSessions command"
Export-ModuleMember -function Get-SSODomainSession -Alias Get-SSODomainSessions

#Function Stop-SSODomainSessions
Function Stop-SSODomainSession
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Domain,
		[Parameter(Mandatory=$false,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$Key,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = [ordered]@{}
	$params.Add("domain", $Domain)

	if (-not ([String]::IsNullOrEmpty($Key)))
	{
		$params.Add("key", $Key)
		$ssodcmd = "ssodomain/killsession"
	}
	else
	{
		$ssodcmd = "ssodomain/killallsessions"
	}

	try {
		$response = sendRequest2LM $ssodcmd $params $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$response = $_.Exception.Message
	}

	if($response.code -eq "fail")
	{	# ERROR
		if ($response -is [array]) {
			if ($($response[1]).Error) {
				$lmResponse = $response[0] + " " + $($response[1]).Error
				$retCode = $($response[1]).stat
			}
			else {
				$lmResponse = $response[0]
				$retCode = 401
			}
			setKempAPIReturnObject $retCode "$lmResponse" $null
		}
		else {
			setKempAPIReturnObject 401 "$response" $null
		}
	}
	else {
		setKempAPIReturnObject 200 "Command successfully executed." $null
	}
}
New-Alias -Name Stop-SSODomainSessions -value Stop-SSODomainSession -Description "Alias for Stop-SSODomainSessions command"
Export-ModuleMember -function Stop-SSODomainSession -Alias Stop-SSODomainSessions

#endregion - SSO Domains

#region - Statistics

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-Statistics
Function Get-LogStatistics
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[switch]$VirtualService,
		[switch]$Totals,
		[switch]$RealServer,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if ((!$VirtualService) -and (!$Totals) -and (!$RealServer))
	{
		$VirtualService = $true
		$Totals = $true
		$RealServer = $true
	}

	$response = sendRequest2LM "stats" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	if (($VirtualService) -and ($response.Vs))
	{
		$response.Vs | Convert-XmlToPSObject
	}

	if (($Totals) -and ($response.VStotals))
	{
		$response.VStotals | Convert-XmlToPsObject
		$response.CPU | Convert-XmlToPsObject
		$response.Network | Convert-XmlToPsObject
		$response.Memory | Convert-XmlToPsObject
		$response.TPS | Convert-XmlToPsObject
	}

	if (($RealServer) -and ($response.rs))
	{
		$response.rs | Convert-XmlToPSObject
	}

	"Change Time : " + $response.ChangeTime

}
New-Alias -Name Get-Statistics -value Get-LogStatistics -Description "Alias for Get-Statistics command"
Export-ModuleMember -function Get-LogStatistics -Alias Get-Statistics

#Function LicenseInfo
Function Get-LicenseInfo
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command licenseinfo -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command licenseinfo -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command licenseinfo -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	$response
}
New-Alias -Name LicenseInfo -value Get-LicenseInfo -Description "Alias for LicenseInfo command"
Export-ModuleMember -function Get-LicenseInfo -Alias LicenseInfo

#endregion - Statistics

#region - Real Servers

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function New-RealServer
Function New-AdcRealServer
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$VirtualService,
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Int32]$VSIndex,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$RealServer,
		[Parameter(Mandatory=$true)]
		[ValidateRange(3,65530)]
		[Int32]$RealServerPort,
		[ValidateRange(1,65530)]
		[Int32]$Weight,
		[ValidateSet("nat","route")]
		[string]$Forward = "nat",
		[bool]$Enable = $true,
		[bool]$Non_Local = $false,
		[ValidateRange(0,100000)]
		[Int64]$Limit = 0,
		[bool]$Critical,
		[ValidateRange(0,65535)]
		[Int32]$Follow,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if ($VSIndex) {
		if ($VirtualService) {
			Throw "The VSIndex and VirtualService parameters are mutually exclusive."
		}
		if ($Port) {
			Throw "The VSIndex and Port parameters are mutually exclusive."
		}
		if ($Protocol) {
			Throw "The VSIndex and Protocol parameters are mutually exclusive."
		}
	} Elseif ($VirtualService) {
		if (!$Port -or !$Protocol) {
			Throw "The VirtualService, Port and Protocol parameters must be used together."
		}
	} else {
		Throw "Either the VirtualService or VSIndex parameter is required."
	}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "addrs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	if ($response.code -eq "fail") {
		$response.Error
	} else {
		$response.RS | Convert-XmlToPSObject
	}
}
New-Alias -Name New-RealServer -value New-AdcRealServer -Description "Alias for New-RealServer command"
Export-ModuleMember -function New-AdcRealServer -Alias New-RealServer

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-RealServer
Function Remove-AdcRealServer
{
	[cmdletbinding(SupportsShouldProcess=$true,ConfirmImpact="High",DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$VirtualService,
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Int32]$VSIndex,
		[ValidateNotNullOrEmpty()]
		[string]$RealServer,
		[ValidateRange(3,65530)]
		[Int32]$RealServerPort,
		[Int32]$RSIndex,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[switch]$Force
	)

	if ($VSIndex) {
		if ($VirtualService) {
			Throw "The VSIndex and VirtualService parameters are mutually exclusive."
		}
		if ($Port) {
			Throw "The VSIndex and Port parameters are mutually exclusive."
		}
		if ($Protocol) {
			Throw "The VSIndex and Protocol parameters are mutually exclusive."
		}
	}
	Elseif ($VirtualService) {
		if (!$Port -or !$Protocol) {
			Throw "The VirtualService, Port and Protocol parameters must be used together."
		}
	}
	else {
		Throw "Either the VirtualService or VSIndex parameter is required."
	}

	if ($RSIndex) {
		if ($RealServer) {
			Throw "The RSIndex and RealServer parameters are mutually exclusive."
		}
		if ($RealServerPort) {
			Throw "The RSIndex and RealServerPort parameters are mutually exclusive."
		}
	}
	Elseif ($RealServer) {
		if (!$RealServerPort) {
			Throw "The RealServer and RealServerPort parameters must be used together."
		}
	}
	else {
		Throw "Either the RealServer or RSIndex parameter is required."
	}

	$propertytable = Convert-BoundParameters -hashtable $psboundparameters

	if (($Force) -or ($PsCmdlet.ShouldProcess($RealServer, "Remove Real Server")))
	{
		sendRequest2LM "delrs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
}
New-Alias -Name Remove-RealServer -value Remove-AdcRealServer -Description "Alias for Remove-RealServer command"
Export-ModuleMember -function Remove-AdcRealServer -Alias Remove-RealServer

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-RealServer
Function Get-AdcRealServer
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$realServers = Get-Statistics -RealServer -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$realServers = Get-Statistics -RealServer -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN
		}
	}
	else {
		$realServers = Get-Statistics -RealServer -LoadBalancer $LoadBalancer -Credential $Credential
	}

	foreach ($rs in $realServers)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$vs = Get-VirtualService -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation | Where-Object {$_.Index -eq $rs.VSIndex}
			}
			else {
				$vs = Get-VirtualService -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN | Where-Object {$_.Index -eq $rs.VSIndex}
			}
		}
		else {
			$vs = Get-VirtualService -LoadBalancer $LoadBalancer -Credential $Credential | Where-Object {$_.Index -eq $rs.VSIndex}
		}
		$rs | Add-Member -MemberType NoteProperty -Name VirtualService -Value $vs.VSAddress
		$rs | Add-Member -MemberType NoteProperty -Name VSPort -Value $vs.VSPort
		$rs | Add-Member -MemberType NoteProperty -Name Protocol -Value $vs.Protocol
		Write-Output $rs
	}
}
New-Alias -Name Get-RealServer -value Get-AdcRealServer -Description "Alias for Get-RealServer command"
Export-ModuleMember -function Get-AdcRealServer -Alias Get-RealServer

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-RealServer
Function Set-AdcRealServer
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$VirtualService,
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Int32]$VSIndex,
		[ValidateNotNullOrEmpty()]
		[string]$RealServer,
		[ValidateRange(3,65530)]
		[Int32]$RealServerPort,
		[Int32]$RSIndex,
		[ValidateRange(3,65530)]
		[Int32]$NewPort,
		[ValidateRange(1,65530)]
		[Int32]$Weight,
		[ValidateSet("nat","route")]
		[string]$Forward = "nat",
		[bool]$Enable = $true,
		[ValidateRange(0,100000)]
		[Int64]$Limit = 0,
		[bool]$Critical,
		[ValidateRange(0,65535)]
		[Int32]$Follow,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if ($VSIndex) {
		if ($VirtualService) {
			Throw "The VSIndex and VirtualService parameters are mutually exclusive."
		}
		if ($Port) {
			Throw "The VSIndex and Port parameters are mutually exclusive."
		}
		if ($Protocol) {
			Throw "The VSIndex and Protocol parameters are mutually exclusive."
		}
	}
	Elseif ($VirtualService) {
		if (!$Port -or !$Protocol) {
			Throw "The VirtualService, Port and Protocol parameters must be used together."
		}
	}
	else {
		Throw "Either the VirtualService or VSIndex parameter is required."
	}

	if ($RSIndex) {
		if ($RealServer) {
			Throw "The RSIndex and RealServer parameters are mutually exclusive."
		}
		if ($RealServerPort) {
			Throw "The RSIndex and RealServerPort parameters are mutually exclusive."
		}
	}
	Elseif ($RealServer) {
		if (!$RealServerPort) {
			Throw "The RealServer and RealServerPort parameters must be used together."
		}
	}
	else {
		Throw "Either the RealServer or RSIndex parameter is required."
	}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "modrs" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Set-RealServer -value Set-AdcRealServer -Description "Alias for Set-RealServer command"
Export-ModuleMember -function Set-AdcRealServer -Alias Set-RealServer

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Enable-RealServer
Function Enable-AdcRealServer
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[Parameter(Position=0,Mandatory=$true)]
		[string]$IPAddress,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	sendRequest2LM "enablers" @{rs=$IPAddress} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Enable-RealServer -value Enable-AdcRealServer -Description "Alias for Enable-RealServer command"
Export-ModuleMember -function Enable-AdcRealServer -Alias Enable-RealServer

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Disable-RealServer
Function Disable-AdcRealServer
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[Parameter(Position=0,Mandatory=$true)]
		[string]$IPAddress,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	sendRequest2LM "disablers" @{rs=$IPAddress} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Disable-RealServer -value Disable-AdcRealServer -Description "Alias for Disable-RealServer command"
Export-ModuleMember -function Disable-AdcRealServer -Alias Disable-RealServer

#endregion - Real Servers

#region - Rules

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml

#Function New-Rule
Function New-AdcContentRule
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true,Position=0)]
		[string]$Name,
		[ValidateSet("regex","prefix","postfix")]
		[string]$MatchType = "regex",
		[bool]$IncHost = $false,
		[bool]$NoCase = $false,
		[bool]$Negate = $false,
		[bool]$IncQuery = $false,
		[string]$Header,
		[string]$Pattern,
		[string]$Replacement,
		[Int32]$Type,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "addrule" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.childnodes
}
New-Alias -Name New-Rule -value New-AdcContentRule -Description "Alias for New-Rule command"
Export-ModuleMember -function New-AdcContentRule -Alias New-Rule

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-Rule
Function Remove-AdcContentRule
{
	[cmdletbinding(SupportsShouldProcess=$true,ConfirmImpact="High",DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true,ValueFromPipeline=$true,Position=0)]
		[string]$Name,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,
		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[switch]$Force
	)

	if (($Force) -or ($PsCmdlet.ShouldProcess($Name, "Remove Rule")))
	{
		$response = sendRequest2LM "delrule" @{name=$name} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
}
New-Alias -Name Remove-Rule -value Remove-AdcContentRule -Description "Alias for Remove-Rule command"
Export-ModuleMember -function Remove-AdcContentRule -Alias Remove-Rule

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-Rule
Function Set-AdcContentRule
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true,Position=0)]
		[string]$Name,
		[ValidateSet("regex","prefix","postfix")]
		[string]$MatchType = "regex",
		[bool]$IncHost,
		[bool]$NoCase,
		[bool]$Negate,
		[bool]$IncQuery,
		[string]$Header,
		[string]$Pattern,
		[string]$Replacement,
		[Int32]$Type,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "modrule" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.childnodes
}
New-Alias -Name Set-Rule -value Set-AdcContentRule -Description "Alias for Set-Rule command"
Export-ModuleMember -function Set-AdcContentRule -Alias Set-Rule

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-Rule
Function Get-AdcContentRule
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(ParameterSetName="Name")]
		[string]$name,

		[Parameter(ParameterSetName="Type")]
		[ValidateSet("MatchContentRule","AddHeaderRule","DeleteHeaderRule","ReplaceHeaderRule","ModifyUrlRule")]
		[string]$Type,

		[Parameter(ParameterSetName="All")]
		[switch]$All,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$ht = @{}

	if ( ([String]::IsNullOrEmpty($name)) -and ([String]::IsNullOrEmpty($Type)) )
 	{
		Write-Verbose "ALL case"
		$All = $true
	}
	else {
		if (-not ([String]::IsNullOrEmpty($name)))
		{
			Write-Verbose "Name case"
			$ht.Add("name",$Name)
		}
		elseif (-not ([String]::IsNullOrEmpty($Type)))
		{
			Write-Verbose "Type case"
			$TypeNumber = $null
			$TypeNumber = $SystemRuleType[$Type]
			$ht.Add("type",$TypeNumber)
		}
		else {
			Throw "ERROR: not valid combination"
		}
	}

	$response = sendRequest2LM "showrule" $ht $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	if ($response.code -eq "fail")
	{
		$response.Error
	}
	else {
		foreach ($ruletype in $SystemRuleType.Keys)
		{
			if ($response.$ruletype)
			{
				$response.$ruletype | Add-Member -MemberType NoteProperty -Name RuleType -Value $ruletype
			}
		}
		$response.childnodes | Sort-Object -Property RuleType | Format-List
	}
}
New-Alias -Name Get-Rule -value Get-AdcContentRule -Description "Alias for Get-Rule command"
Export-ModuleMember -function Get-AdcContentRule -Alias Get-Rule

#endregion - Rules

#region - Real Server Rule

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function New-RealServerRule
Function New-AdcRealServerRule
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$VirtualService,
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Int32]$VSIndex,
		[ValidateNotNullOrEmpty()]
		[string]$RealServer,
		[ValidateRange(3,65530)]
		[Int32]$RSPort,
		[Int32]$RSIndex,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$RuleName,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if ($VSIndex) {
		if ($VirtualService) {
			Throw "The VSIndex and VirtualService parameters are mutually exclusive."
		}
		if ($Port) {
			Throw "The VSIndex and Port parameters are mutually exclusive."
		}
		if ($Protocol) {
			Throw "The VSIndex and Protocol parameters are mutually exclusive."
		}
	} Elseif ($VirtualService) {
		if (!$Port -or !$Protocol) {
			Throw "The VirtualService, Port and Protocol parameters must be used together."
		}
	} else {
		Throw "Either the VirtualService or VSIndex parameter is required."
	}

	if ($RSIndex) {
		if ($RealServer) {
			Throw "The RSIndex and RealServer parameters are mutually exclusive."
		}
		if ($RSPort) {
			Throw "The RSIndex and RSPort parameters are mutually exclusive."
		}
	} Elseif ($RealServer) {
		if (!$RSPort) {
			Throw "The RealServer and RSPort parameters must be used together."
		}
	} else {
		Throw "Either the RealServer or RSIndex parameter is required."
	}

	$propertytable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "addrsrule" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name New-RealServerRule -value New-AdcRealServerRule -Description "Alias for New-RealServerRule command"
Export-ModuleMember -function New-AdcRealServerRule -Alias New-RealServerRule

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-RealServerRule
Function Remove-AdcRealServerRule
{
	[cmdletbinding(SupportsShouldProcess=$true,ConfirmImpact="High", DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$VirtualService,
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Int32]$VSIndex,
		[ValidateNotNullOrEmpty()]
		[string]$RealServer,
		[ValidateRange(3,65530)]
		[Int32]$RSPort,
		[Int32]$RSIndex,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$RuleName,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[switch]$Force
	)

	if ($VSIndex) {
		if ($VirtualService) {
			Throw "The VSIndex and VirtualService parameters are mutually exclusive."
		}
		if ($Port) {
			Throw "The VSIndex and Port parameters are mutually exclusive."
		}
		if ($Protocol) {
			Throw "The VSIndex and Protocol parameters are mutually exclusive."
		}
	} Elseif ($VirtualService) {
		if (!$Port -or !$Protocol) {
			Throw "The VirtualService, Port and Protocol parameters must be used together."
		}
	} else {
		Throw "Either the VirtualService or VSIndex parameter is required."
	}

	if ($RSIndex) {
		if ($RealServer) {
			Throw "The RSIndex and RealServer parameters are mutually exclusive."
		}
		if ($RSPort) {
			Throw "The RSIndex and RSPort parameters are mutually exclusive."
		}
	} Elseif ($RealServer) {
		if (!$RSPort) {
			Throw "The RealServer and RSPort parameters must be used together."
		}
	} else {
		Throw "Either the RealServer or RSIndex parameter is required."
	}

	$propertytable = Convert-BoundParameters -hashtable $psboundparameters

	if (($Force) -or ($PsCmdlet.ShouldProcess($RuleName, "Remove Real Server Rule")))
	{
		sendRequest2LM "delrsrule" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
}
New-Alias -Name Remove-RealServerRule -value Remove-AdcRealServerRule -Description "Alias for Remove-RealServerRule command"
Export-ModuleMember -function Remove-AdcRealServerRule -Alias Remove-RealServerRule

#endregion - Real Server Rule

#region - Virtual Server Rules

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function New-VirtualServerRule
Function New-AdcVirtualServerRule
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateSet("pre","response","request")]
		[string]$RuleType,
		[ValidateNotNullOrEmpty()]
		[string]$VirtualService,
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Int32]$VSIndex,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$RuleName,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if ($VSIndex) {
		if ($VirtualService) {
			Throw "The VSIndex and VirtualService parameters are mutually exclusive."
		}
		if ($Port) {
			Throw "The VSIndex and Port parameters are mutually exclusive."
		}
		if ($Protocol) {
			Throw "The VSIndex and Protocol parameters are mutually exclusive."
		}
	} Elseif ($VirtualService) {
		if (!$Port -or !$Protocol) {
			Throw "The VirtualService, Port and Protocol parameters must be used together."
		}
	} else {
		Throw "Either the VirtualService or VSIndex parameter is required."
	}

	$Command = ""
	switch ($RuleType)
	{
		"pre" {$Command = "addprerule"}
		"response" {$Command = "addresponserule"}
		"request" {$Command = "addrequestrule"}
	}

	$propertytable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM $Command $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name New-VirtualServerRule -value New-AdcVirtualServerRule -Description "Alias for New-VirtualServerRule command"
Export-ModuleMember -function New-AdcVirtualServerRule -Alias New-VirtualServerRule

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-VirtualServerRule
Function Remove-AdcVirtualServerRule
{
	[cmdletbinding(SupportsShouldProcess=$true,ConfirmImpact="High",DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateSet("pre","response","request")]
		[string]$RuleType,
		[ValidateNotNullOrEmpty()]
		[string]$VirtualService,
		[ValidateRange(3,65530)]
		[Int32]$Port,
		[ValidateSet("tcp","udp")]
		[string]$Protocol,
		[Int32]$VSIndex,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$RuleName,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[switch]$Force
	)

	if ($VSIndex) {
		if ($VirtualService) {
			Throw "The VSIndex and VirtualService parameters are mutually exclusive."
		}
		if ($Port) {
			Throw "The VSIndex and Port parameters are mutually exclusive."
		}
		if ($Protocol) {
			Throw "The VSIndex and Protocol parameters are mutually exclusive."
		}
	} Elseif ($VirtualService) {
		if (!$Port -or !$Protocol) {
			Throw "The VirtualService, Port and Protocol parameters must be used together."
		}
	} else {
		Throw "Either the VirtualService or VSIndex parameter is required."
	}

	$Command = ""
	switch ($RuleType)
	{
		"pre" {$Command = "delprerule"}
		"response" {$Command = "delresponserule"}
		"request" {$Command = "delrequestrule"}
	}

	$propertytable = Convert-BoundParameters -hashtable $psboundparameters

	if (($Force) -or ($PsCmdlet.ShouldProcess($RuleName, "Remove Virtual Server Rule")))
	{
		sendRequest2LM $Command $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
}
New-Alias -Name Remove-VirtualServerRule -value Remove-AdcVirtualServerRule -Description "Alias for Remove-VirtualServerRule command"
Export-ModuleMember -function Remove-AdcVirtualServerRule -Alias Remove-VirtualServerRule

#endregion - Virtual Server Rules

#region - Service Check Parameters

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-ServiceHealth
Function Get-AdcServiceHealth
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$response = sendRequest2LM "showhealth" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response | Convert-XmlToPSObject
}
New-Alias -Name Get-ServiceHealth -value Get-AdcServiceHealth -Description "Alias for Get-ServiceHealth command"
Export-ModuleMember -function Get-AdcServiceHealth -Alias Get-ServiceHealth

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-ServiceHealth
Function Set-AdcServiceHealth
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$RetryInterval,
		[Int16]$RetryCount,
		[Int16]$Timeout,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters $psboundparameters

	$response = sendRequest2LM "modhealth" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response | Convert-XmlToPSObject
}
New-Alias -Name Set-ServiceHealth -value Set-AdcServiceHealth -Description "Alias for Set-ServiceHealth command"
Export-ModuleMember -function Set-AdcServiceHealth -Alias Set-ServiceHealth

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-AdaptiveCheck
Function Get-AdcAdaptiveHealthCheck
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$response = sendRequest2LM "showadaptive" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response | Convert-XmlToPSObject
}
New-Alias -Name Get-AdaptiveCheck -value Get-AdcAdaptiveHealthCheck -Description "Alias for Get-AdaptiveCheck command"
Export-ModuleMember -function Get-AdcAdaptiveHealthCheck -Alias Get-AdaptiveCheck

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-AdaptiveCheck
Function Set-AdcAdaptiveHealthCheck
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$AdaptiveURL,
		[Int32]$AdaptivePort,
		[Int32]$AdaptiveInterval,
		[Int16]$MinPercent,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters $psboundparameters

	$response = sendRequest2LM "modadaptive" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response | Convert-XMLToPSObject
}
New-Alias -Name Set-AdaptiveCheck -value Set-AdcAdaptiveHealthCheck -Description "Alias for Set-AdaptiveCheck command"
Export-ModuleMember -function Set-AdcAdaptiveHealthCheck -Alias Set-AdaptiveCheck

#endregion - Service Check Parameters

#region - Certificates

# Internal use only
Function handleListCertAnswer($response)
{
	if ($response.cert) {
		# This should be a valid answer

		$certs = $response.cert
		$CertsData = @()
		ForEach($cert in $certs) {
			$tmpData = [ordered]@{}
			$tmpData.PSTypeName = "CertData"

			$cName = $cert.name
			$cModulus = $cert.modulus
			$tmpData.Add("CertName", $cName)
			$tmpData.Add("CertModulus", $cModulus)

			$data = New-Object -TypeName PSObject -Prop $tmpData
			$CertsData += $data
		}
		setKempAPIReturnObject 200 "Command successfully executed." $CertsData
	}
	else {
		if ($response -eq "ok") {
			# NO data available
			setKempAPIReturnObject 200 "Command successfully executed. No certificates are installed." $null
		}
		else {
			# This should be an error
			Write-Verbose "response: $response"
			if ($response -is [array]) {
				$errString = $response[0]
			}
			else {
				$errString = $response
			}
			setKempAPIReturnObject 401 "$errString" $null
		}
	}
	return
}

# Internal use only
Function handleReadCertAnswer($response, $certName)
{
	if ($response.Certificate) {
		# This should be a valid answer

		$cert = $response.Certificate
		$tmpData = [ordered]@{}
		$tmpData.PSTypeName = "Certificate"
		$tmpData.Add("Name", $certName)
		$tmpData.Add("Certificate", $cert)
		$data = New-Object -TypeName PSObject -Prop $tmpData

		setKempAPIReturnObject 200 "Command successfully executed." $data
	}
	else {
		Write-Verbose "response: $response"

		if ($response -is [array]) {

			$lmaType = $($response[1].GetType()).Name

			if ($lmaType -eq "XmlElement") {
				$errCode = $($response[1]).stat
				$r_answer = $response[0] + " " + $($response[1]).Error
			}
			else {
				$errCode = 401
				$r_answer = $response[0]
			}
		}
		else {
			$errCode = 401
			$r_answer = $response
		}
		setKempAPIReturnObject $errCode "$r_answer" $null
	}
	return
}

# Internal use only
Function ListTlsCertificate($propertyTable, $cmd2exec, $LoadBalancer, $LBPort, $Credential, $SubjectCN, $CertificateStoreLocation, $CertName, $Outform)
{
	if (-not ([String]::IsNullOrEmpty($CertName))) {
		$propertyTable.Remove("CertName")
		$propertyTable.Add("cert", $CertName)
	}
	if (-not ([String]::IsNullOrEmpty($Outform))) {
		$propertyTable.Remove("Outform")
		$propertyTable.Add("outform", $Outform)
	}

	try {
		$lmResponse = sendRequest2LM $cmd2exec $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		setKempAPIReturnObject 401 "$($_.Exception.Message)" $null
		return
	}

	if (([String]::IsNullOrEmpty($CertName))) {
		handleListCertAnswer $lmResponse
	}
	else {
		handleReadCertAnswer $lmResponse $CertName
	}
}

Function Get-TlsIntermediateCertificate
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$CertName,
		[string]$Outform,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (([String]::IsNullOrEmpty($CertName))) {
		ListTlsCertificate $propertyTable "listintermediate" $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation $CertName $Outform
	}
	else {
		ListTlsCertificate $propertyTable "readintermediate" $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation $CertName $Outform
	}
}
Export-ModuleMember -function Get-TlsIntermediateCertificate

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function New-Certificate
Function New-TlsCertificate
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$Name,
		[string]$Password,
		[switch]$Replace,
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{cert=$name;replace="0"}

	if ($Replace) {
		$params["replace"]="1"
	}
	if ($Password) {
		$params.Add("password",[System.Web.HttpUtility]::UrlEncode($Password))
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command addcert -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command addcert -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command addcert -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	if ($response.code -eq "fail") {
		$response.Error
	}
	else {
		$response
	}
}
New-Alias -Name New-Certificate -value New-TlsCertificate -Description "Alias for New-Certificate command"
Export-ModuleMember -function New-TlsCertificate -Alias New-Certificate

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-Certificate
Function Remove-TlsCertificate
{
	[cmdletbinding(SupportsShouldProcess=$true,ConfirmImpact="High",DefaultParameterSetName='Credential')]
	Param(
		[string]$Name,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,
		[switch]$Force
	)
	$param = @{cert=$name}

	if (($Force) -or ($PsCmdlet.ShouldProcess($Name, "Remove Certificate")))
	{
		$response = sendRequest2LM "delcert" $param $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		if ($response.code -eq "fail") {
			$response.Error
		}
		else {
			$response
		}
	}
}
New-Alias -Name Remove-Certificate -value Remove-TlsCertificate -Description "Alias for Remove-Certificate command"
Export-ModuleMember -function Remove-TlsCertificate -Alias Remove-Certificate

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function New-IntermediateCertificate
Function New-TlsIntermediateCertificate
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$Name,
		[string]$Password,
		[switch]$Replace,
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{cert=$name;replace="0"}

	if ($Replace) {
		$params["replace"]="1"
	}
	if ($Password) {
		$params.Add("password",[System.Web.HttpUtility]::UrlEncode($Password))
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command addintermediate -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command addintermediate -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command addintermediate -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($response.code -eq "fail") {
		$response.Error
	}
	else {
		$response
	}
}
New-Alias -Name New-IntermediateCertificate -value New-TlsIntermediateCertificate -Description "Alias for New-IntermediateCertificate command"
Export-ModuleMember -function New-TlsIntermediateCertificate -Alias New-IntermediateCertificate

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-IntermediateCertificate
Function Remove-TlsIntermediateCertificate
{
	[cmdletbinding(SupportsShouldProcess=$true,ConfirmImpact="High",DefaultParameterSetName='Credential')]
	Param(
		[string]$Name,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[switch]$Force
	)
	$param = @{cert=$name}

	if (($Force) -or ($PsCmdlet.ShouldProcess($Name, "Remove Intermediate Certificate")))
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command delintermediate -ParameterValuePair $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command delintermediate -ParameterValuePair $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command delintermediate -ParameterValuePair $param -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		if ($response.code -eq "fail") {
			$response.Error
		}
		else {
			$response
		}
	}
}
New-Alias -Name Remove-IntermediateCertificate -value Remove-TlsIntermediateCertificate -Description "Alias for Remove-IntermediateCertificate command"
Export-ModuleMember -function Remove-TlsIntermediateCertificate -Alias Remove-IntermediateCertificate

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Backup-Certificate
Function Backup-TlsCertificate
{
	#[CmdletBinding()]
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateScript({$_.length -gt 6})]
		[string]$Password,
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}
	$params.Add("password",[System.Web.HttpUtility]::UrlEncode($Password))
	if (-not ($Path))
	{
		$Path = "$($Env:SystemRoot)\Temp\CertificateBackup_$(Get-Date -format yyyy-MM-dd_HH-mm-ss)"
	}
	Write-Verbose "Path = $Path"

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command backupcert -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command backupcert -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command backupcert -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	$response
}
New-Alias -Name Backup-Certificate -value Backup-TlsCertificate -Description "Alias for Backup-Certificate command"
Export-ModuleMember -function Backup-TlsCertificate -Alias Backup-Certificate

#Function ListCert
Function Get-TlsCertificate
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$CertName,
		[string]$Outform,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (([String]::IsNullOrEmpty($CertName))) {
		ListTlsCertificate $propertyTable "listcert" $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation $CertName $Outform
	}
	else {
		ListTlsCertificate $propertyTable "readcert" $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation $CertName $Outform
	}
}
New-Alias -Name ListCert -value Get-TlsCertificate -Description "Alias for ListCert command"
Export-ModuleMember -function Get-TlsCertificate -Alias ListCert

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Restore-Certificate
Function Restore-TlsCertificate
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateScript({$_.length -gt 6})]
		[string]$Password,
		[string]$Path,
		[Parameter(Mandatory=$true)]
		[ValidateSet("Full","VS","Third")]
		[string]$Type,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}
	$params.Add("password",[System.Web.HttpUtility]::UrlEncode($Password))
	$params.Add("Type", $Type)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command restorecert -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command restorecert -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command restorecert -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	$response
}
New-Alias -Name Restore-Certificate -value Restore-TlsCertificate -Description "Alias for Restore-Certificate command"
Export-ModuleMember -function Restore-TlsCertificate -Alias Restore-Certificate

#endregion - Certificates

#region - Cipherset

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function ModifyCipherset
Function Set-TlsCipherSet
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$Value,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "modifycipherset" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name ModifyCipherset -value Set-TlsCipherSet -Description "Alias for ModifyCipherset command"
Export-ModuleMember -function Set-TlsCipherSet -Alias ModifyCipherset

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function GetCipherset
Function Get-TlsCipherSet
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "getcipherset" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name GetCipherset -value Get-TlsCipherSet -Description "Alias for GetCipherset command"
Export-ModuleMember -function Get-TlsCipherSet -Alias GetCipherset

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function DelCipherset
Function Remove-TlsCipherSet
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "delcipherset" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DelCipherset -value Remove-TlsCipherSet -Description "Alias for DelCipherset command"
Export-ModuleMember -function Remove-TlsCipherSet -Alias DelCipherset

#endregion - Cipherset

#region - Interfaces

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-Interface
Function Get-NetworkInterface
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID = -1,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	try {
		if ($InterfaceID -ne -1)
		{
			Write-Verbose "interface id [$InterfaceID]"
			$response = sendRequest2LM "showiface" @{iface=$InterfaceID} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		}
		else {
			Write-Verbose "interface id []"
			$response = sendRequest2LM "showiface" $null $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		}
	}
	catch {
		setKempAPIReturnObject 401 "$($_.Exception.Message)" $null
		return
	}

	if ($response.Interface)
	{
		$answer = $response.Interface | Convert-XmlToPSObject
		return $answer
	}
	else {
		Write-Verbose "response [$response]"
		if ($response.code -eq "fail")
		{
			setKempAPIReturnObject 422 "$response" $null
			return
		}
		if ($response) {
			if ($response.Contains("Unauthorized")) {
				setKempAPIReturnObject 401 "Error: Unauthorized." $null
			}
			if ($response.Contains("Unable to connect")) {
				setKempAPIReturnObject 401 "Error: Unable to connect to the remote server." $null
			}
		}
	}
}
New-Alias -Name Get-Interface -value Get-NetworkInterface -Description "Alias for Get-Interface command"
Export-ModuleMember -function Get-NetworkInterface -Alias Get-Interface

# Internal usage only
Function CheckLmConnection($LmIp, $LmNewIp, $LmPort)
{
	$check = $true
	$counter = 2
	$connStatus = $true
	while ($check) {
		if (-not (Test-ServerConnection -ComputerName $LmIp -Port $LmPort)) {
			$connStatus = $false
			Start-Sleep -s 5
			$counter -= 1
			if ($counter -eq 0) {
				$check = $false
			}
		}
		else {
			$connStatus = $true
			$check = $false
		}
	}
	if ($connStatus -eq $false) {
		$check = $true
		$counter = 5
		$LmIp = $($LmNewIp.Split('/'))[0]
		while ($check) {
			if (-not (Test-ServerConnection -ComputerName $LmIp -Port $LmPort)) {
				Start-Sleep -s 5
				$counter -= 1
				if ($counter -eq 0) {
					$check = $false
				}
			}
			else {
				$check = $false
				$connStatus = $true
			}
		}
	}
	if ($connStatus -eq $false) {
		Throw "ERROR: not able to reconnect to the LM"
	}
	else {
		$LmIp
	}
}

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-Interface
Function Set-NetworkInterface
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,
		[string]$IPAddress,
		[Int32]$MTU,
		[bool]$HACheck,
		[bool]$GWIface,
		[bool]$GeoTraffic,
		[ValidateSet("802.3ad","Active-backup")]
		[string]$BondMode,
		[string]$Partner,
		[string]$Shared,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
		}
	}
	else {
		$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($intfCheck.ReturnCode) {
		if ($intfCheck.ReturnCode -ge 400) {
			return $intfCheck
		}
		else {
			setKempAPIReturnObject 401 "The remote server returned an error" $null
			return
		}
	}

	if ($intfCheck)
	{
		$propertyTable = Convert-BoundParameters $psboundparameters
		$propertyTable.Remove("iface")
		$response = ""
		Foreach ($property in $propertyTable.keys)
		{
			if (isLoginParam $property) {
				continue
			}
			$params = @{iface=$InterfaceID}
			$params.Add($property, $propertyTable[$property])

			try {
				if (-not ([String]::IsNullOrEmpty($SubjectCN)))
				{
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
					{
						$response = Send-LBMessage -command modiface -ParameterValuePair $params -ErrorAction SilentlyContinue -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
					}
					else {
						$response = Send-LBMessage -command modiface -ParameterValuePair $params -ErrorAction SilentlyContinue -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
					}
				}
				else {
					$response = Send-LBMessage -command modiface -ParameterValuePair $params -ErrorAction SilentlyContinue -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
				}
			}
			catch {
				setKempAPIReturnObject 422 "$($_.Exception.Message)" $null
				return
			}
			<#
			if ($property -eq "addr")
			{
				$SystemIP = [System.Web.HttpUtility]::UrlEncode((Get-Interface -InterfaceID $InterfaceID).IPAddress)
				Write-Verbose "System value: $SystemIP"
				Write-Verbose "Given IP: $($propertyTable[$property])"
				if ($SystemIP -ne $($propertyTable[$property]))
				{
					Write-Error "IP Address could not be bound to interface."
					break
				}
			}
			else {
				if ($response -ne "ok")
				{
					Write-Error $response.Error
					break
				}
			}
			#>
		}
		if ($response -eq "ok")
		{
			# The machine can have now a new address
			# we need to check the connection first
			Write-Verbose "Checking LM connection . . ."
			$LoadBalancer = CheckLmConnection $LoadBalancer $IPAddress $LBPort
			Write-Verbose "Successfully connect to the LM"
			if (-not ([String]::IsNullOrEmpty($SubjectCN)))
			{
				if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
				{
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
				}
				else {
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
				}
			}
			else {
				Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
			}
		}
	}
	else {
		Write-Verbose "check [$intfCheck]"
	}
}
New-Alias -Name Set-Interface -value Set-NetworkInterface -Description "Alias for Set-Interface command"
Export-ModuleMember -function Set-NetworkInterface -Alias Set-Interface


#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Add-InterfaceAddress
Function Add-NetworkInterfaceAddress
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,
		[string]$Address,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
		}
	}
	else {
		$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($intfCheck)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command addadditional -ParameterValuePair @{iface=$InterfaceID;addr=$Address} -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command addadditional -ParameterValuePair @{iface=$InterfaceID;addr=$Address} -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command addadditional -ParameterValuePair @{iface=$InterfaceID;addr=$Address} -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}

		if ($response -eq "ok")
		{
			if (-not ([String]::IsNullOrEmpty($SubjectCN)))
			{
				if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
				{
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
				}
				else {
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
				}
			}
			else {
				Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
			}
		}
		else {
			Write-Error $response.Error
		}
	}
}
New-Alias -Name Add-InterfaceAddress -value Add-NetworkInterfaceAddress -Description "Alias for Add-InterfaceAddress command"
Export-ModuleMember -function Add-NetworkInterfaceAddress -Alias Add-InterfaceAddress

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-InterfaceAddress
Function Remove-NetworkInterfaceAddress
{
	[cmdletbinding(SupportsShouldProcess=$true,ConfirmImpact="High",DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,
		[string]$Address,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[switch]$Force
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
		}
	}
	else {
		$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($intfCheck)
	{
		if (($Force) -or ($PsCmdlet.ShouldProcess($InterfaceID, "Remove Interface Address")))
		{
			if (-not ([String]::IsNullOrEmpty($SubjectCN)))
			{
				if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
				{
					$response = Send-LBMessage -Command deladditional -ParameterValuePair @{iface=$InterfaceID;addr=$Address} -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
				}
				else {
					$response = Send-LBMessage -Command deladditional -ParameterValuePair @{iface=$InterfaceID;addr=$Address} -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
				}
			}
			else {
				$response = Send-LBMessage -Command deladditional -ParameterValuePair @{iface=$InterfaceID;addr=$Address} -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
			}

			if ($response -eq "ok")
			{
				if (-not ([String]::IsNullOrEmpty($SubjectCN)))
				{
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
					{
						Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
					}
					else {
						Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
					}
				}
				else {
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
				}
			}
			else {
				Write-Error $response.Error
			}
		}
	}
}
New-Alias -Name Remove-InterfaceAddress -value Remove-NetworkInterfaceAddress -Description "Alias for Remove-InterfaceAddress command"
Export-ModuleMember -function Remove-NetworkInterfaceAddress -Alias Remove-InterfaceAddress

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Register-BondedInterface
Function Register-NetworkBondedInterface
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
		}
	}
	else {
		$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($intfCheck)
	{
		$response = sendRequest2LM "createbond" @{iface=$InterfaceID} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		$check = $true
		$counter = 12
		while ($check)
		{
			Write-Output "Bonding: waiting to reconnect to the LM . . ."
			if (-not (Test-ServerConnection -ComputerName $LoadBalancer -Port $LBPort)) {
				Start-Sleep -s 10
				$counter -= 1
				if ($counter -eq 0)
				{
					Throw "ERROR: Unable to re-connect to the LM"
				}
			}
			else {
				if (-not ([String]::IsNullOrEmpty($SubjectCN)))
				{
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
					{
						Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
					}
					else {
						Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
					}
				}
				else {
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
				}
				$check = $false
			}
		}
	}
}
New-Alias -Name Register-BondedInterface -value Register-NetworkBondedInterface -Description "Alias for Register-BondedInterface command"
Export-ModuleMember -function Register-NetworkBondedInterface -Alias Register-BondedInterface

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Unregister-BondedInterface
Function Unregister-NetworkBondedInterface
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
		}
	}
	else {
		$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($intfCheck)
	{
		$response = sendRequest2LM "unbond" @{iface=$InterfaceID} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	
		$check = $true
		$counter = 12
		while ($check)
		{
			Write-Output "Un-Bonding: waiting to reconnect to the LM . . ."
			if (-not (Test-ServerConnection -ComputerName $LoadBalancer -Port $LBPort)) {
				Start-Sleep -s 10
				$counter -= 1
				if ($counter -eq 0)
				{
					Throw "ERROR: Unable to re-connect to the LM"
				}
			}
			else {
				if (-not ([String]::IsNullOrEmpty($SubjectCN)))
				{
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
					{
						Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
					}
					else {
						Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
					}
				}
				else {
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
				}
				$check = $false
			}
		}
	}
}
New-Alias -Name Unregister-BondedInterface -value Unregister-NetworkBondedInterface -Description "Alias for Unregister-BondedInterface command"
Export-ModuleMember -function Unregister-NetworkBondedInterface -Alias Unregister-BondedInterface

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Add-BondedInterface
Function Add-NetworkBondedInterface
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,
		[Int16]$BondID,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
		}
	}
	else {
		$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($intfCheck)
	{
		$response = sendRequest2LM "addbond" @{iface=$InterfaceID;bond=$BondID} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		if ($response -eq "ok")
		{
			if (-not ([String]::IsNullOrEmpty($SubjectCN)))
			{
				if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
				{
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
				}
				else {
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
				}
			}
			else {
				Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
			}
		}
		else {
			Write-Error $response.Error
		}
	}
}
New-Alias -Name Add-BondedInterface -value Add-NetworkBondedInterface -Description "Alias for Add-BondedInterface command"
Export-ModuleMember -function Add-NetworkBondedInterface -Alias Add-BondedInterface

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-BondedInterface
Function Remove-NetworkBondedInterface
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,
		[Int16]$BondID,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
		}
	}
	else {
		$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($intfCheck)
	{
		$response = sendRequest2LM "delbond" @{iface=$InterfaceID;bond=$BondID} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		if ($response -eq "ok")
		{
			if (-not ([String]::IsNullOrEmpty($SubjectCN)))
			{
				if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
				{
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
				}
				else {
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
				}
			}
			else {
				Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
			}
		}
		else {
			Write-Error $response.Error
		}
	}
}
New-Alias -Name Remove-BondedInterface -value Remove-NetworkBondedInterface -Description "Alias for Remove-BondedInterface command"
Export-ModuleMember -function Remove-NetworkBondedInterface -Alias Remove-BondedInterface

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Add-VLan
Function Add-NetworkVLAN
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,
		[ValidateRange(1,4095)]
		[Int16]$VLAN,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
		}
	}
	else {
		$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($intfCheck)
	{
		$response = sendRequest2LM "addvlan" @{iface=$InterfaceID;vlanid=$VLAN} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		if ($response -eq "ok")
		{
			"VLAN $VLAN created on interface $InterfaceID"
		}
		else {
			Write-Error $response.Error
		}
	}
}
New-Alias -Name Add-VLan -value Add-NetworkVLAN -Description "Alias for Add-VLan command"
Export-ModuleMember -function Add-NetworkVLAN -Alias Add-VLan

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-VLan
Function Remove-NetworkVLAN
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
		}
	}
	else {
		$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($intfCheck)
	{
		$response = sendRequest2LM "delvlan" @{iface=$InterfaceID} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		if ($response -eq "ok")
		{
			"VLAN removed"
			if (-not ([String]::IsNullOrEmpty($SubjectCN)))
			{
				if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
				{
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
				}
				else {
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
				}
			}
			else {
				Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
			}
		}
		else {
			Write-Error $response.Error
		}
	}
}
New-Alias -Name Remove-VLan -value Remove-NetworkVLAN -Description "Alias for Remove-VLan command"
Export-ModuleMember -function Remove-NetworkVLAN -Alias Remove-VLan

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Add-VxLan
Function Add-NetworkVxLAN
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,

		[ValidateRange(1,16777214)]
		[Int32]$VXLAN,

		[ValidateNotNullOrEmpty()]
		[string]$Addr,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if ((($Addr -As [IPAddress]) -As [Bool])) 
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
			}
			else {
				$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
			}
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}

		if ($intfCheck)
		{
			$sep = "."
			$parts = [System.StringSplitOptions]::RemoveEmptyEntries
			$AddrParts = $Addr.Split($sep,4,$parts)
			[int]$parti = [convert]::ToInt32($AddrParts[0], 10)

			if ($parti -gt 223 -and $parti -lt 240)
			{
				$response = sendRequest2LM "addvxlan" @{iface=$InterfaceID;vni=$VXLAN;group=$Addr} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
			}
			else {
				$response = sendRequest2LM "addvxlan" @{iface=$InterfaceID;vni=$VXLAN;remote=$Addr} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
			}

			if ($response -eq "ok")
			{
				"VXLAN $VXLAN created on interface $InterfaceID"
			}
			else {
				Write-Error $response.Error
			}
		}
	}
	else {
		Write-Error "No valid group or remote IP address given"
	}
}
New-Alias -Name Add-VxLan -value Add-NetworkVxLAN -Description "Alias for Add-VxLan command"
Export-ModuleMember -function Add-NetworkVxLAN -Alias Add-VxLan

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-VxLan
Function Remove-NetworkVxLAN
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$InterfaceID,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
		}
		else {
			$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
		}
	}
	else {
		$intfCheck = Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($intfCheck)
	{
		$response = sendRequest2LM "delvxlan" @{iface=$InterfaceID} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		if ($response -eq "ok")
		{
			"VXLAN removed"
			if (-not ([String]::IsNullOrEmpty($SubjectCN)))
			{
				if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
				{
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation 
				}
				else {
					Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN 
				}
			}
			else {
				Get-Interface -InterfaceID $InterfaceID -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
			}
		}
		else {
			Write-Error $response.Error
		}
	}
}
New-Alias -Name Remove-VxLan -value Remove-NetworkVxLAN -Description "Alias for Remove-VxLan command"
Export-ModuleMember -function Remove-NetworkVxLAN -Alias Remove-VxLan

#endregion - Interfaces

#region - DNS Configuration

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-DNSConfiguration
Function Get-NetworkDNSConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$haModeRequest = @{}
	$haModeRequest.Add("Param", "Hamode")
	$response = sendRequest2LM "get" $haModeRequest $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	$LmHaStatus = ($response | Convert-XmlToPSObject).hamode
	Write-Verbose "LmHaStatus [$LmHaStatus]"
	if ($LmHaStatus -eq 0) {
		$parameters = @("Hostname", "NamServer", "SearchList")
	}
	else {
		$parameters = @("Hostname","HA1Hostname","HA2Hostname","NamServer","SearchList")
	}

	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
		if ($response.FirstChild)
		{
			$settings.add($param, $response.firstchild.InnerText)
		}
		else {
			$settings.add($param, $response)
		}
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-DNSConfiguration -value Get-NetworkDNSConfiguration -Description "Alias for Get-DNSConfiguration command"
Export-ModuleMember -function Get-NetworkDNSConfiguration -Alias Get-DNSConfiguration

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-DNSConfiguration
Function Set-NetworkDNSConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$Hostname,
		[string]$HA1Hostname,
		[string]$HA2Hostname,
		[string]$NameServer,
		[string]$Searchlist,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command set -parameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
			}
		}
		else {
			$response = Send-LBMessage -Command set -parameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-DNSConfiguration -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-DNSConfiguration -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-DNSConfiguration -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-DNSConfiguration -value Set-NetworkDNSConfiguration -Description "Alias for Set-DNSConfiguration command"
Export-ModuleMember -function Set-NetworkDNSConfiguration -Alias Set-DNSConfiguration

Function Update-LmDnsCache
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	try {
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command resolvenow -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command resolvenow -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command resolvenow -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "Command successfully executed"
		$tempApiRetObj.Data = $null

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Update-LmDnsCache

#endregion - DNS Configuration

#region - Hosts Management

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Get-Hosts
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command gethosts -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command gethosts -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command gethosts -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($response.hasChildNodes)
	{
		$response.HostsEntry | Convert-XmlToPSObject
	}
}
Export-ModuleMember -function Get-Hosts

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Add-HostsEntry
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$HostIP,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[string]$HostFQDN,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$response = sendRequest2LM "addhostsentry" @{hostip=$HostIP;hostfqdn=$HostFQDN} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	$response
}
Export-ModuleMember -function Add-HostsEntry

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Remove-HostsEntry
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$HostIP,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$response = sendRequest2LM "delhostsentry" @{hostip=$HostIP} $LoadBalancer $LmPort $Credential $SubjectCN $CertificateStoreLocation
	$response
}
Export-ModuleMember -function Remove-HostsEntry
#endregion - Hosts Management

#region - Route Management

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-Route
Function Get-NetworkRoute
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command showroute -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command showroute -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command showroute -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}

	if ($response.hasChildNodes)
	{
		$response.Route | Convert-XmlToPSObject
	}
}
New-Alias -Name Get-Route -value Get-NetworkRoute -Description "Alias for Get-Route command"
Export-ModuleMember -function Get-NetworkRoute -Alias Get-Route

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function New-Route
Function New-NetworkRoute
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Destination,

		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[Alias("Mask","SubnetMask")]
		[Int16]$CIDR,

		[Parameter(Mandatory=$true,Position=2)]
		[ValidateNotNullOrEmpty()]
		[string]$Gateway,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$DestIP = "$Destination/$CIDR"
	$response = sendRequest2LM "addroute" @{dest=$DestIP;gateway=$Gateway} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	if ($response -ne "ok") {
		Write-Verbose "ERROR: the cmd has failed"
		setKempAPIReturnObject 400 "Command failure. Error: $response" $null
		return
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-Route -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-Route -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-Route -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name New-Route -value New-NetworkRoute -Description "Alias for New-Route command"
Export-ModuleMember -function New-NetworkRoute -Alias New-Route

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-Route
Function Remove-NetworkRoute
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Destination,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$response = sendRequest2LM "delroute" @{dest=$Destination} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-Route -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-Route -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-Route -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Remove-Route -value Remove-NetworkRoute -Description "Alias for Remove-Route command"
Export-ModuleMember -function Remove-NetworkRoute -Alias Remove-Route

#endregion - Route Management

#region - Packet Filter Options

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Get-PacketFilterOption
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateSet("enable","drop","ifblock")]
		[String]$Option,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if ($Option -eq "enable")
	{
		$ParameterName = "isenabled"
	}
	else {
		$ParameterName = "is$Option"
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Send-LBMessage -Command aclcontrol -ParameterName $ParameterName -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Send-LBMessage -Command aclcontrol -ParameterName $ParameterName -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Send-LBMessage -Command aclcontrol -ParameterName $ParameterName -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
}
Export-ModuleMember -function Get-PacketFilterOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Set-PacketFilterOption
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateSet("enable","drop","ifblock")]
		[String]$Option,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[bool]$Value,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = @{}
	$propertyTable.Add($Option, $Value -as [int])

	$response =	sendRequest2LM "aclcontrol" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	if ($response.code -eq "fail")
	{
		$response.Error
	}
	else {
		$response.cmdexitstatus
	}
}
Export-ModuleMember -function Set-PacketFilterOption

#endregion - Packet Filter Options

#region - Global Black and White Lists

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Get-GlobalPacketFilterACL
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateSet("black","white")]
		[String]$Type,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = @{}
	$propertyTable.Add("list", $Type)

	try {
		$response = sendRequest2LM "aclcontrol" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$exceptionMsg = $($_.Exception.Message)
		setKempAPIReturnObject 401 "$exceptionMsg" $null
		return
	}

	if ($response.code -eq "fail") {
		$errorString = $response.Error
		setKempAPIReturnObject 401 "$errorString" $null
		return
	}

	if ($response) {
		try {
			$x = $response | Get-Member -MemberType Method "Contains"
		}
		catch {
			$exceptionMsg = $($_.Exception.Message)
			setKempAPIReturnObject 401 "$exceptionMsg" $null
			return
		}
	}
	else {
		setKempAPIReturnObject 401 "System error: this error can be caused using a wrong Virtual service/port." $null
		return
	}

	if ($x.Name) {
		if ($response -is [array]) {
			$r_response = $response[0]
		}
		else {
			$r_response = $response
		}

		if ($r_response.Contains("Unauthorized") -or
				$r_response.Contains("Invalid address format") -or
				$r_response.Contains("Unable to connect") -or
				$r_response.Contains("Can't find certificate") -or
				$r_response.Contains("The remote name could") -or
				$r_response.Contains("unknown host")) {
			$errorString = $r_response
		}
		else {
			$errorString = "Unknow error"
		}
		setKempAPIReturnObject 401 "$errorString" $null
		return
	}

	$outputParams = [ordered]@{}

	if ($Type -eq "black") {
		$list = $response.Blacklist
		if ($list) {
			$aa = $list | Convert-XmlToPSObject
		}
	}
	else {
		$list = $response.Whitelist
		if ($list) {
			$aa = $list | Convert-XmlToPSObject
		}
	}

	if ($list) {
		$ips = @()
		$aa.PSObject.Properties | Sort-Object | foreach-object {
			$tmp = [ordered]@{}
			$name  = $_.Name 
			$value = $_.value
			$values = $value -Split ','
			$x = $tmp.Add("Name", $name)
			$x = $tmp.Add("Addr", $values[0])
			$x = $tmp.Add("Comment", $values[1])
			$obj = New-Object -TypeName PSObject -Prop $tmp
			$ips += $obj
		}

		if ($Type -eq "black") {
			$outputParams.Add("Blacklist", $ips)
		}
		else {
			$outputParams.Add("Whitelist", $ips)
		}
	}

	if (-not $list) {
		$outputParams = $null
	}
	$apiRetObject = New-Object -TypeName PSObject -Prop $outputParams

	setKempAPIReturnObject 200 "Command successfully executed" $apiRetObject
}
Export-ModuleMember -function Get-GlobalPacketFilterACL

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Add-GlobalPacketFilterAddr
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateSet("black","white")]
		[String]$Type,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$Address,
		[String]$Comment,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = @{}
	$propertyTable.Add("add", $Type)
	$propertyTable.Add("addr", $Address)
	$propertyTable.Add("comment", [System.Uri]::EscapeDataString("""$Comment"""))

	try {
		$answer = sendRequest2LM "aclcontrol" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$exceptionMsg = $($_.Exception.Message)
		setKempAPIReturnObject 401 "$exceptionMsg" $null
		return
	}

	if ($answer -eq "ok") {
		setKempAPIReturnObject 200 "Command successfully executed" $null
	}
	else {
		if ($answer -is [array]) {
			$len = $answer.Length
			if ($len -eq 2) {
				setKempAPIReturnObject 401 "$($answer[0])" $null
				return
			}
			elseif ($len -eq 3) {
				setKempAPIReturnObject 401 "$($answer[2])" $null
				return
			}
			else {
				setKempAPIReturnObject 401 "Unknown error." $null
				return
			}
		}
		elseif ($answer -is [string]) {
			if ($answer.Contains("violation")) {
				setKempAPIReturnObject 401 "command too long" $null
			}
			else {
				setKempAPIReturnObject 401 "$answer" $null
			}
			return
		}
		else {
			setKempAPIReturnObject 401 "Unknown error." $null
			return
		}
	}
}
Export-ModuleMember -function Add-GlobalPacketFilterAddr

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Remove-GlobalPacketFilterAddr
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateSet("black","white")]
		[String]$Type,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$Address,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = @{}
	$propertyTable.Add("del", $Type)
	$propertyTable.Add("addr", $Address)

	sendRequest2LM "aclcontrol" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
Export-ModuleMember -function Remove-GlobalPacketFilterAddr

#endregion - Global Black and White Lists

#region - System Administration

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
Function Update-License
{
	[CmdletBinding()]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateScript({Test-Path $_})]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$response = Send-LBMessage -Command license -File $Path -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	$response
}
Export-ModuleMember -function Update-License

Function HandleOfflineLicAnswer($LmResponse)
{
	$errObj = [ordered]@{}
	$errObj.PSTypeName = "KempAPI"

	if ($LmResponse -is [array]) {
		$errMsg = $LmResponse[1]
		$etype = $errMsg.GetType().Name
		if ($etype.Contains("Xml") -eq $true) {
			Write-Verbose "[array] xml case"
			$errObj.Add("ReturnCode", $errMsg.stat)
			$errObj.Add("Response", $errMsg.Error)
			$errObj.Add("Data", $null)
		}
		else {
			Write-Verbose "[array] string case"
			if ($errMsg.Contains("Unauthorized") -eq $true) {
				$errObj.Add("ReturnCode", 401)
				$errObj.Add("Response", $errMsg)
				$errObj.Add("Data", $null)
			}
			elseif ($errMsg.Contains("The remote name could not be resolved") -eq $true) {
				$errObj.Add("ReturnCode", 400)
				$errObj.Add("Response", $errMsg)
				$errObj.Add("Data", $null)
			}
			elseif ($errMsg.Contains("Unable to connect to the remote server") -eq $true) {
				$errObj.Add("ReturnCode", 400)
				$errObj.Add("Response", $errMsg)
				$errObj.Add("Data", $null)
			}
			else {
				$errObj.Add("ReturnCode", 400)
				$errObj.Add("Response", "Unknown error")
				$errObj.Add("Data", $null)
			}
		}
	}
	else {
		Write-Verbose "string case"

		$errMsg = $LmResponse
		if ($errMsg.Contains("The remote name could not be resolved") -eq $true) {
			$errObj.Add("ReturnCode", 400)
			$errObj.Add("Response", $errMsg)
			$errObj.Add("Data", $null)
		}
		elseif ($errMsg.Contains("ok") -eq $true) {
			$errObj.Add("ReturnCode", 200)
			$errObj.Add("Response", "Command successfully executed")
			$errObj.Add("Data", $null)
		}
		else {
			$errObj.Add("ReturnCode", 400)
			$errObj.Add("Response", "Unknown error")
			$errObj.Add("Data", $null)
		}
	}
	New-Object -TypeName PSObject -Prop $errObj
}

Function Request-LicenseOffline
{
	[CmdletBinding()]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateScript({Test-Path $_})]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	try {
		$response = Send-LBMessage -Command license -File $Path -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort

		Write-Verbose "response [$response]"

		HandleOfflineLicAnswer $response
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Request-LicenseOffline

Function Update-LicenseOffline
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateScript({Test-Path $_})]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3, 65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	try {
		$params = @{
			Command = "license"
			File = $Path
			LoadBalancer = $LoadBalancer
			LBPort = $LBPort
		}

		if (-not ([String]::IsNullOrEmpty($SubjectCN))) {
			$params.Add("SubjectCN", $SubjectCN)
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation))) {
				$params.Add("CertificateStoreLocation", $CertificateStoreLocation)
			}
		}
		else {
			$params.Add("Credential", $Credential)
		}
		$response = Send-LBMessage @params

		Write-Verbose "response [$response]"

		HandleOfflineLicAnswer $response
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Update-LicenseOffline

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-AccessKey
Function Get-LicenseAccessKey
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command accesskey -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command accesskey -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command accesskey -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
	$response.AccessKey
}
New-Alias -Name Get-AccessKey -value Get-LicenseAccessKey -Description "Alias for Get-AccessKey command"
Export-ModuleMember -function Get-LicenseAccessKey -Alias Get-AccessKey

Function Get-LicenseType
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true)]
		[string]$KempId = $KempId,

		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true)]
		[string]$Password = $Password,

		[ValidateNotNullOrEmpty()]
		[string]$OrderId,

		[ValidateNotNullOrEmpty()]
		[string]$http_proxy
	)

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	try {
		$response = sendRequest2LM "alsilicensetypes" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$response = $_.Exception.Message
	}
	if ($response) {
		if ($response.Response.stat -eq 200) {
			$data = $response.Response.Success
			if ($data.Contains("License type information not available")) {
				setKempAPIReturnObject 401 "ERROR: License type information not available. Please check your kempid and/or password." $null
				return
			}
			if ($data -ne "[]") {
				$TempLicObject = $data | ConvertFrom-Json
				$LicObject = [ordered]@{}
				$LicObject.PSTypeName = "LicenseData"
				$licNumber = $TempLicObject.categories.licenseTypes.Length
				if (([String]::IsNullOrEmpty($licNumber))) {
					$licNumber = 1
				}
				$LicObject.Add("OrderID", $OrderId)
				$LicObject.Add("AvailableLicenses", $licNumber)
				$LicObject.Add("Licenses", $TempLicObject.categories.licenseTypes)
				$licData = New-Object -TypeName PSObject -Prop $LicObject
				setKempAPIReturnObject 200 "Command successfully executed." $licData
			}
			else {
				$emptyLicObject = [ordered]@{}
				$emptyLicObject.PSTypeName = "LicenseData"
				$emptyLicObject.Add("OrderID", $OrderId)
				$emptyLicObject.Add("AvailableLicenses", 0)
				$emptyLicObject.Add("Licenses", $null)
				$licData = New-Object -TypeName PSObject -Prop $emptyLicObject
				setKempAPIReturnObject 200 "Command successfully executed." $licData
			}
		}
		else {
			if ($response.Response) {
				$errorString = $response.Response
				setKempAPIReturnObject 401 "$errorString" $null
			}
			else {
				if ($response -is [array]) {
					$tmp = $response[1].Error
					if (-not ([String]::IsNullOrEmpty($tmp))) {
						$errorCode = $response[1].stat
						$errorString = $tmp
					}
					else {
						$errorCode = 401
						$errorString = $response[0]
					}
				}
				else {
					$errorCode = 401
					$errorString = $response
				}
				setKempAPIReturnObject $errorCode "$errorString" $null
			}
		}
	}
	else {
		setKempAPIReturnObject 401 "ERROR the answer is NULL" $null
	}
}
Export-ModuleMember -function Get-LicenseType

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Restart-LoadBalancer
Function Restart-Lm
{
	[cmdletbinding(SupportsShouldProcess=$true,DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[switch]$Shutdown,
		[switch]$Force
	)

	$Command = $null
	if ($Shutdown)
	{
		$Command = "shutdown"
	}
	else {
		$Command = "reboot"
	}

	if (($Force) -or ($PsCmdlet.ShouldProcess($Command, "Shutdown $LoadBalancer")))
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command $Command -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command $Command -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command $Command -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
		$response
	}
}
New-Alias -Name Restart-LoadBalancer -value Restart-Lm -Description "Alias for Restart-LoadBalancer command"
Export-ModuleMember -function Restart-Lm -Alias Restart-LoadBalancer

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Install-Patch
Function Install-LmPatch
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[Parameter(Mandatory=$true)]
		[ValidateScript({Test-Path $_})]
		[string]$Path
	)
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command installpatch -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command installpatch -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command installpatch -File $Path -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
	$response
}
New-Alias -Name Install-Patch -value Install-LmPatch -Description "Alias for Install-Patch command"
Export-ModuleMember -function Install-LmPatch -Alias Install-Patch

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Restore-Patch
Function Uninstall-LmPatch
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command restorepatch -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
		else {
		}
	}
	else {
		$response = Send-LBMessage -command restorepatch -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
	$response
	if ($response.ToString().ToLower() -eq "ok")
	{
		Write-Warning "You must restart the LoadMaster for this process to be completed."
	}
}
New-Alias -Name Restore-Patch -value Uninstall-LmPatch -Description "Alias for Restore-Patch command"
Export-ModuleMember -function Uninstall-LmPatch -Alias Restore-Patch

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Backup-LoadBalancer
Function Backup-LmConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[Parameter(Mandatory=$true)]
		[string]$Path
	)
	$params = @{}
	if (-not ($Path))
	{
		$Path = "$($Env:SystemRoot)\Temp\LMBackup_$(Get-Date -format yyyy-MM-dd_HH-mm-ss)"
	}

	Write-Verbose "Path = $Path"

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command backup -File $Path -Output -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $LBPort
		}
		else {
			$response = Send-LBMessage -command backup -File $Path -Output -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $LBPort
		}
	}
	else {
		$response = Send-LBMessage -command backup -File $Path -Output -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
	$response
}
New-Alias -Name Backup-LoadBalancer -value Backup-LmConfiguration -Description "Alias for Backup-LoadBalancer command"
Export-ModuleMember -function Backup-LmConfiguration -Alias Backup-LoadBalancer

Function Set-LmBackupSecureIdentity
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateScript({Test-Path $_})]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3, 65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$params = @{}

	$currFolder = Split-Path -Path $Path
	if (-not (Test-Path -Path $currFolder)) {
		Throw "ERROR: the folder $currFolder does not exist"
	}

	$fileToUpload = $currFolder + "\IdentityFile.txt"
	$tmpFile = $currFolder + "\IdentityFile_temp.txt"

	if (Test-Path -Path $fileToUpload) {
		Remove-Item $fileToUpload
	}

	if (Test-Path -Path $tmpFile) {
		Remove-Item $tmpFile
	}

	try {
		$privKeyFileContent = Get-Content -Raw $Path
		$privKeyFileContentBytes = [System.Text.Encoding]::UTF8.GetBytes($privKeyFileContent)
		$privKeyFileContentEncoded = [System.Convert]::ToBase64String($privKeyFileContentBytes)
		$secureString = "param=backupident&value=" + $privKeyFileContentEncoded
		Set-Content -Value $secureString -Path $tmpFile 
		$txt = (Get-Content -Raw $tmpFile) -replace "`r`n",""
		[io.file]::WriteAllText($fileToUpload, $txt)
	}
	catch {
		$errMsg = $_.Exception.Message
		setKempAPIReturnObject 400 "$errMsg" $null
		return
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command set -ParameterValuePair $params -File $fileToUpload -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $LBPort
		}
		else {
			$response = Send-LBMessage -Command set -ParameterValuePair $params -File $fileToUpload -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $LBPort
		}
	}
	else {
		$response = Send-LBMessage -Command set -ParameterValuePair $params -File $fileToUpload -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}

	if (-not ($PSBoundParameters['Debug'])) {
		Remove-Item $tmpFile
		Remove-Item $fileToUpload
	}

	if ($response -eq "ok") {
		setKempAPIReturnObject 200 "Command successfully executed" $null
	}
	else {
		setKempAPIReturnObject 422 "$response" $null
	}
}
Export-ModuleMember -function Set-LmBackupSecureIdentity

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Restore-LoadBalancer
Function Restore-LmConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[Parameter(Mandatory=$true)]
		[ValidateScript({Test-Path $_})]
		[string]$Path,

		[Parameter(Mandatory=$true)]
		[ValidateSet("Base","VS","Geo")]
		[string]$Type
	)
	$params = @{}
	$TypeInt = $RestoreType[$Type]
	Write-Verbose "Type = $TypeInt"
	$params.Add("Type", $TypeInt)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command restore -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $LBPort
		}
		else {
			$response = Send-LBMessage -Command restore -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $LBPort
		}
	}
	else {
		$response = Send-LBMessage -Command restore -ParameterValuePair $params -File $Path -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
	$response
}
New-Alias -Name Restore-LoadBalancer -value Restore-LmConfiguration -Description "Alias for Restore-LoadBalancer command"
Export-ModuleMember -function Restore-LmConfiguration -Alias Restore-LoadBalancer

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-BackupOption
Function Get-LmBackupConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$parameters = @("backupday","backupenable","backuphost","backuphour","backupminute",
		"backuppassword","backuppath","backupuser")

	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $LBPort
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $LBPort
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
		$settings.add($param, $response.firstchild.InnerText)
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-BackupOption -value Get-LmBackupConfiguration -Description "Alias for Get-BackupOption command"
Export-ModuleMember -function Get-LmBackupConfiguration -Alias Get-BackupOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-BackupOption
Function Set-LmBackupConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[bool]$BackupEnable,
		[string]$BackupPath,
		[string]$BackupUser,
		[string]$BackupPassword,
		[string]$BackupHost,
		[ValidateRange(0,7)]
		[Int16]$BackupDay,
		[ValidateRange(0,23)]
		[Int16]$BackupHour,
		[ValidateRange(0,59)]
		[Int16]$BackupMinute,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	
	foreach ($entry in $propertyTable.Keys)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $LBPort
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $LBPort
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		{
			Get-BackupOption -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $LBPort
		}
		else {
			Get-BackupOption -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $LBPort
		}
	}
	else {
		Get-BackupOption -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
}
New-Alias -Name Set-BackupOption -value Set-LmBackupConfiguration -Description "Alias for Set-BackupOption command"
Export-ModuleMember -function Set-LmBackupConfiguration -Alias Set-BackupOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-DateTimeOption
Function Get-LmDateTimeConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	# Don't get NTPKeySecret - no valid data is returned
	$parameters = @("NTPHost","Time","BootTime","ActiveTime","TimeZone","NTPKeyId")
	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
		$settings.add($param, $response.firstchild.InnerText)
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-DateTimeOption -value Get-LmDateTimeConfiguration -Description "Alias for Get-DateTimeOption command"
Export-ModuleMember -function Get-LmDateTimeConfiguration -Alias Get-DateTimeOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-DateTimeOption
Function Set-LmDateTimeConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$NTPHost,
		[Int32]$Time,
		[string]$TimeZone,
		[ValidateRange(1,100)]
		[int]$NTPKeyId,
		[string]$NTPKeySecret,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command set -parameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -parameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -parameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		{
			Get-DateTimeOption -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-DateTimeOption -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-DateTimeOption -LoadBalancer $LoadBalancer -Credential $Credential
	}
}
New-Alias -Name Set-DateTimeOption -value Set-LmDateTimeConfiguration -Description "Alias for Set-DateTimeOption command"
Export-ModuleMember -function Set-LmDateTimeConfiguration -Alias Set-DateTimeOption

Function Get-PreviousVersion
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	try {
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -Command getpreviousversion -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command getpreviousversion -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command getpreviousversion -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "Command successfully executed"
		
		$prevVersion = @{}
		$prevVersion.PSTypeName = "PreviousVersion"
		$prevVersion.PreviousVersion = $response.PreviousVersion
		$previousVersion = New-Object -TypeName PSObject -Prop $prevVersion
		
		$tempApiRetObj.Data = $previousVersion

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Get-PreviousVersion

#endregion - System Administration

#region - Logging Options

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-DebugOption
Function Get-LmDebugConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$parameters = @("IRQBalance","LineAREspLogs","NetConsole","NetConsoleInterface")
	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
		$settings.add($param, $response.firstchild.InnerText)
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-DebugOption -value Get-LmDebugConfiguration -Description "Alias for Get-DebugOption command"
Export-ModuleMember -function Get-LmDebugConfiguration -Alias Get-DebugOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-DebugOption
Function Set-LmDebugConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[bool]$IRQBalance,
		[bool]$LineAREspLogs,
		[string]$NetConsole,
		[Int16]$NetConsoleInterface,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (isLoginParam $entry)
		{
			continue
		}
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-DebugOption -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-DebugOption -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-DebugOption -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-DebugOption -value Set-LmDebugConfiguration -Description "Alias for Set-DebugOption command"
Export-ModuleMember -function Set-LmDebugConfiguration -Alias Set-DebugOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-SyslogOption
Function Get-LogSyslogConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$parameters = @("SyslogCritical","SyslogEmergency","SyslogError","SyslogInfo","SyslogNotice","SyslogWarn")

	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		$settings.add($param, $response.firstchild.InnerText)
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-SyslogOption -value Get-LogSyslogConfiguration -Description "Alias for Get-SyslogOption command"
Export-ModuleMember -function Get-LogSyslogConfiguration -Alias Get-SyslogOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-SyslogOption
Function Set-LogSyslogConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$SyslogCritical,
		[string]$SyslogEmergency,
		[string]$SyslogError,
		[string]$SyslogInfo,
		[string]$SyslogNotice,
		[string]$SyslogWarn,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (isLoginParam $entry)
		{
			continue
		}
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		$response
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-SyslogOption -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-SyslogOption -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-SyslogOption -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-SyslogOption -value Set-LogSyslogConfiguration -Description "Alias for Set-SyslogOption command"
Export-ModuleMember -function Set-LogSyslogConfiguration -Alias Set-SyslogOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-SNMPOption
Function Get-NetworkSNMPConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	# if snmp is NOT enabled, return only SNMP disable as in the WUI
	$haModeRequest = @{}
	$haModeRequest.Add("Param", "Snmpenable")
	try {
		$response = sendRequest2LM "get" $haModeRequest $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$response_ex = $_.Exception.Message
	}

	if ($response_ex) {
		# Exception
		Write-Verbose "Exception [$response_ex]"
		setKempAPIReturnObject 401 "$response_ex" $null
		return
	}

	if ($response.hasChildNodes) {
    $snmpStatus = ($response | Convert-XmlToPSObject).snmpenable
    Write-Verbose "snmpStatus [$snmpStatus]"

		if ($SnmpStatus -eq "no") {
			$parameters = [ordered]@{}
			$parameters.Add("SNMPEnable", "no")

			$data = New-Object -TypeName PSObject -Property $parameters
			setKempAPIReturnObject 200 "Command successfully executed" $data
			return
		}

		if ($SnmpStatus -eq "yes") {

			$parameters = @("SNMPCommunity","SNMPv3user","SNMPv3userpasswd","snmpAuthProt", "snmpPrivProt", "SNMPContact","SNMPEnable", "SNMPv3enable","SNMPTrapEnable","SNMPv1Sink","SNMPv2Sink", "SNMPHaTrap", "SNMPClient", "SNMPLocation")

			$settings = @{}
			foreach ($param in $parameters) {
				if (-not ([String]::IsNullOrEmpty($SubjectCN))) {
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation))) {
						$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
					}
					else {
						$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
					}
				}
				else {
					$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
				}
				$settings.add($param, $response.firstchild.InnerText)
				Start-Sleep -m 500
			}
			$data = New-Object -TypeName PSObject -Property $settings
			setKempAPIReturnObject 200 "Command successfully executed" $data
			return
		}
	}
	# if we get here means that we have an ERROR
	setKempAPIReturnObject 401 "$response" $null
}
New-Alias -Name Get-SNMPOption -value Get-NetworkSNMPConfiguration -Description "Alias for Get-SNMPOption command"
Export-ModuleMember -function Get-NetworkSNMPConfiguration -Alias Get-SNMPOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-SNMPOption
Function Set-NetworkSNMPConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$SNMPCommunity,
		[string]$SNMPv3user,
		[string]$SNMPv3userpasswd,
		[string]$snmpAuthProt,
		[string]$snmpPrivProt,
		[string]$SNMPContact,
		[bool]$SNMPEnable,
		[bool]$SNMPv3enable,
		[bool]$SNMPTrapEnable,
		[string]$SNMPv1Sink,
		[string]$SNMPv2Sink,
		[bool]$SNMPHaTrap,
		[string]$SNMPClient,
		[string]$SNMPLocation,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-SNMPOption -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-SNMPOption -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-SNMPOption -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-SNMPOption -value Set-NetworkSNMPConfiguration -Description "Alias for Set-SNMPOption command"
Export-ModuleMember -function Set-NetworkSNMPConfiguration -Alias Set-SNMPOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-EmailOption
Function Get-LogEmailConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$parameters = @("EmailCritical","EmailDomain","EmailEmergency","EmailEnable","EmailError","EmailInfo",
		"EmailNotice","EmailPassword","EmailPort","EmailServer","EmailSSLMode","EmailUser","EmailWarn")

	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		$settings.add($param, $response.firstchild.InnerText)
		Start-Sleep -m 500
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-EmailOption -value Get-LogEmailConfiguration -Description "Alias for Get-EmailOption command"
Export-ModuleMember -function Get-LogEmailConfiguration -Alias Get-EmailOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-EmailOption
Function Set-LogEmailConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$EmailUser,
		[string]$EmailDomain,
		[string]$EmailPassword,
		[ValidateRange(3,65530)]
		[string]$EmailPort,
		[string]$EmailServer,
		[bool]$EmailEnable,
		[ValidateRange(0,3)]
		[Int16]$EmailSSLMode,
		[string]$EmailInfo,
		[string]$EmailNotice,
		[string]$EmailWarn,
		[string]$EmailError,
		[string]$EmailEmergency,
		[string]$EmailCritical,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-EmailOption -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-EmailOption -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-EmailOption -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-EmailOption -value Set-LogEmailConfiguration -Description "Alias for Set-EmailOption command"
Export-ModuleMember -function Set-LogEmailConfiguration -Alias Set-EmailOption

#endregion - Logging Options

#region - Miscellaneous Options

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-WUISetting
Function Get-SecWebUIConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$parameters = @("hoverhelp","motd","sessioncontrol","sessionidletime","sessionmaxfailattempts","wuidisplaylines")
	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		$settings.add($param, $response.firstchild.InnerText)
	}
	New-Object -TypeName PSObject -Property $settings 
}
New-Alias -Name Get-WUISetting -value Get-SecWebUIConfiguration -Description "Alias for Get-WUISetting command"
Export-ModuleMember -function Get-SecWebUIConfiguration -Alias Get-WUISetting

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-WUISetting
Function Set-SecWebUIConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[bool]$HoverHelp,
		[string]$Motd,
		[bool]$SessionControl,
		[ValidateRange(60,86400)]
		[Int32]$SessionIdleTime,
		[ValidateRange(1,999)]
		[Int16]$SessionMaxFailAttempts,
		[ValidateRange(10,100)]
		[Int16]$WUIDisplayLines,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.keys)
	{
		if (isLoginParam $entry)
		{
			continue
		}
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		{
			Get-WUISetting -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-WUISetting -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-WUISetting -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-WUISetting -value Set-SecWebUIConfiguration -Description "Alias for Set-WUISetting command"
Export-ModuleMember -function Set-SecWebUIConfiguration -Alias Set-WUISetting

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-RemoteAccess
Function Get-SecRemoteAccess
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$parameters = @("admingw","enableapi","geoclients","geopartners", "geosshport","sshaccess","sshiface","sshport","wuiaccess","wuiiface","wuiport", "geosshiface")
	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		$settings.add($param, $response.firstchild.InnerText)
		Start-Sleep -m 500
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-RemoteAccess -value Get-SecRemoteAccess -Description "Alias for Get-RemoteAccess command"
Export-ModuleMember -function Get-SecRemoteAccess -Alias Get-RemoteAccess

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-RemoteAccess
Function Set-SecRemoteAccess
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$AdminGW,
		[bool]$EnableAPI,
		[ValidateRange(3,65530)]
		[Int64]$GeoSSHPort,
		[bool]$SSHAccess,
		[string]$SSHIface,
		[ValidateRange(3,65530)]
		[Int64]$SSHPort,
		[bool]$WUIAccess,
		[int32]$WUIIface,
		[ValidateRange(3,65530)]
		[Int64]$WUIPort,
		[String[]]$GeoClients,
		[String[]]$GeoPartners,
		[Int32]$GeoInterfaceId,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (isLoginParam $entry)
		{
			continue
		}
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-RemoteAccess -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-RemoteAccess -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-RemoteAccess -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-RemoteAccess -value Set-SecRemoteAccess -Description "Alias for Set-RemoteAccess command"
Export-ModuleMember -function Set-SecRemoteAccess -Alias Set-RemoteAccess

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-WUIAuth
Function Get-SecWebUIAuth
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$parameters = @("ldapserver","ldapbackupserver","ldapsecurity","ldaprevalidateinterval","radiusserver","radiusport","radiussecret",
		"radiusrevalidateinterval","radiusbackupserver","radiusbackupport","radiusbackupsecret","sessionlocalauth","sessionauthmode")
	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		$settings.add($param, $response.firstchild.InnerText)
		Start-Sleep -m 500
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-WUIAuth -value Get-SecWebUIAuth -Description "Alias for Get-WUIAuth command"
Export-ModuleMember -function Get-SecWebUIAuth -Alias Get-WUIAuth

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-WUIAuth
Function Set-SecWebUIAuth
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[string]$LDAPServer,
		[string]$LDAPBackupServer,
		[ValidateRange(0,2)]
		[Int16]$LDAPSecurity,
		[ValidateRange(10,86400)]
		[int16]$LDAPRevalidateInterval,
		[string]$RADIUSServer,
		[string]$RADIUSBackupServer,
		[ValidateRange(3,65530)]
		[Int64]$RADIUSPort,
		[ValidateRange(3,65530)]
		[Int64]$RADIUSBackupPort,
		[string]$RADIUSSecret,
		[string]$RADIUSBackupSecret,
		[bool]$SessionLocalAuth,
		[ValidateSet(7,22,23,262,263,278,279,772,773,774,775,788,789,790,791)]
		[Int16]$SessionAuthMode,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (isLoginParam $entry)
		{
			continue
		}
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		$response
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		{
			Get-WUIAuth -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-WUIAuth -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-WUIAuth -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-WUIAuth -value Set-SecWebUIAuth -Description "Alias for Set-WUIAuth command"
Export-ModuleMember -function Set-SecWebUIAuth -Alias Set-WUIAuth

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-L7Configuration
Function Get-AdcL7Configuration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$parameters = @("addcookieport","addvia","alwayspersist","closeonerror","dropatdrainend","droponfail","expect100","rfcconform",
		"rsarelocal","localbind","transparent","slowstart","addforwardheader", "allowemptyposts", "authtimeout", "clienttokentimeout")

	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $LBPort
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $LBPort
			}
		}
		else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
		$settings.add($param, $response.firstchild.InnerText)
		Start-Sleep -m 500
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-L7Configuration -value Get-AdcL7Configuration -Description "Alias for Get-L7Configuration command"
Export-ModuleMember -function Get-AdcL7Configuration -Alias Get-L7Configuration

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-L7Configuration
Function Set-AdcL7Configuration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[bool]$AddCookiePort,
		[bool]$AddVia,
		[bool]$AlwaysPersist,
		[bool]$CloseOnError,
		[bool]$DropAtDrainEnd,
		[ValidateRange(30,300)]
		[Int16]$AuthTimeout,
		[ValidateRange(60,300)]
		[Int16]$ClientTokenTimeout,
		[bool]$DropOnFail,
		[ValidateRange(0,2)]
		[int16]$Expect100,
		[bool]$RFConform,
		[bool]$RSAreLocal,
		[bool]$LocalBind,
		[bool]$Transparent,
		[bool]$AllowEmptyPosts,
		[ValidateRange(0,600)]
		[Int16]$SlowStart,
		[ValidateRange(0,2)]
		[Int16]$AddForwardHeader,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (isLoginParam $entry)
		{
			continue
		}
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation -LBPort $LBPort
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -LBPort $LBPort
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}

	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-L7Configuration -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-L7Configuration -LoadBalancer $LoadBalancer -SubjectCN $SubjectCN
		}
	}
	else {
		Get-L7Configuration -LoadBalancer $LoadBalancer -Credential $Credential
	}
}
New-Alias -Name Set-L7Configuration -value Set-AdcL7Configuration -Description "Alias for Set-L7Configuration command"
Export-ModuleMember -function Set-AdcL7Configuration -Alias Set-L7Configuration

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-NetworkOptions
Function Get-NetworkConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$parameters = @("snat","allowupload","conntimeout","keepalive","multigw","nonlocalrs","onlydefaultroutes","resetclose",
		"subnetorigin","subnetoriginating","tcptimestamp","routefilter", "dhkeysize", "http_proxy")

	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		$settings.add($param, $response.firstchild.InnerText)
		Start-Sleep -m 500
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-NetworkOptions -value Get-NetworkConfiguration -Description "Alias for Get-NetworkOptions command"
Export-ModuleMember -function Get-NetworkConfiguration -Alias Get-NetworkOptions

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-NetworkOptions
Function Set-NetworkConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[bool]$SNAT,
		[bool]$AllowUpload,
		[ValidateRange(0,86400)]
		[Int64]$ConnTimeout,
		[bool]$KeepAlive,
		[bool]$MultiGW,
		[bool]$NonLocalRS,
		[bool]$OnlyDefaultRoutes,
		[bool]$ResetClose,
		[bool]$SubnetOrigin,
		[bool]$SubnetOriginating,
		[bool]$TCPTimeStamp,
		[bool]$RouteFilter,
		[ValidateRange(512,4096)]
		[Int32]$DHKeySize,
		[string]$Http_Proxy,

    [ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (isLoginParam $entry)
		{
			continue
		}
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-NetworkOptions -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-NetworkOptions -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-NetworkOptions -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-NetworkOptions -value Set-NetworkConfiguration -Description "Alias for Set-NetworkOptions command"
Export-ModuleMember -function Set-NetworkConfiguration -Alias Set-NetworkOptions

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Update-IDSRules
Function Update-WafIDSRules
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateScript({Test-Path -Path $_})]
		[ValidateNotNullOrEmpty()]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command updatedetect -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command updatedetect -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command updatedetect -File $Path -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}

	if ($response.code -eq "fail")
	{
		$response.Error
	}
	else {
		$response
	}
}
New-Alias -Name Update-IDSRules -value Update-WafIDSRules -Description "Alias for Update-IDSRules command"
Export-ModuleMember -function Update-WafIDSRules -Alias Update-IDSRules

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Add-NoCacheExtension
Function Add-AdcHttpCacheException
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0,ValueFromPipeline=$true)]
		[ValidatePattern({^\.})]
		[String[]]$Extension,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	PROCESS
	{
		foreach ($ext in $Extension)
		{
			$response = sendRequest2LM "addnocache" @{param=$ext} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		}
		$response
	}
}
New-Alias -Name Add-NoCacheExtension -value Add-AdcHttpCacheException -Description "Alias for Add-NoCacheExtension command"
Export-ModuleMember -function Add-AdcHttpCacheException -Alias Add-NoCacheExtension

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-NoCacheExtension
Function Remove-AdcHttpCacheException
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0,ValueFromPipeline=$true)]
		[ValidatePattern({^\.})]
		[String[]]$Extension,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	PROCESS
	{
		foreach ($ext in $Extension)
		{
			$response = sendRequest2LM "delnocache" @{param=$ext} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		}
		$response
	}
}
New-Alias -Name Remove-NoCacheExtension -value Remove-AdcHttpCacheException -Description "Alias for Remove-NoCacheExtension command"
Export-ModuleMember -function Remove-AdcHttpCacheException -Alias Remove-NoCacheExtension

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Add-NoCompressExtension
Function Add-AdcHttpCompressionException
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0,ValueFromPipeline=$true)]
		[ValidatePattern({^\.})]
		[String[]]$Extension,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	PROCESS
	{
		foreach ($ext in $Extension)
		{
			$response = sendRequest2LM "addnocompress" @{param=$ext} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		}
		$response
	}
}
New-Alias -Name Add-NoCompressExtension -value Add-AdcHttpCompressionException -Description "Alias for Add-NoCompressExtension command"
Export-ModuleMember -function Add-AdcHttpCompressionException -Alias Add-NoCompressExtension

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Remove-NoCompressExtension
Function Remove-AdcHttpCompressionException
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0,ValueFromPipeline=$true)]
		[ValidatePattern({^\.})]
		[String[]]$Extension,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	PROCESS
	{
		foreach ($ext in $Extension)
		{
			$response = sendRequest2LM "delnocompress" @{param=$ext} $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		}
		$response
	}
}
New-Alias -Name Remove-NoCompressExtension -value Remove-AdcHttpCompressionException -Description "Alias for Remove-NoCompressExtension command"
Export-ModuleMember -function Remove-AdcHttpCompressionException -Alias Remove-NoCompressExtension

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-AFEConfiguration
Function Get-LmAFEConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$parameters = @("Cachesize","HostCache","Paranoia","LimitInput")
	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
		$settings.add($param, $response.firstchild.InnerText)
	}
	New-Object -TypeName PSObject -Property $settings
}
New-Alias -Name Get-AFEConfiguration -value Get-LmAFEConfiguration -Description "Alias for Get-AFEConfiguration command"
Export-ModuleMember -function Get-LmAFEConfiguration -Alias Get-AFEConfiguration

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-AFEConfiguration
Function Set-LmAFEConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateRange(1,250)]
		[Int16]$CacheSize,
		[bool]$HostCache,
		[ValidateRange(0,3)]
		[Int16]$Paranoia,
		[ValidateRange(0,100000)]
		[Int64]$LimitInput,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	
	foreach ($entry in $propertyTable.Keys)
	{
		if (isLoginParam $entry)
		{
			continue
		}
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
		}
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-AFEConfiguration -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-AFEConfiguration -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-AFEConfiguration -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-AFEConfiguration -value Set-LmAFEConfiguration -Description "Alias for Set-AFEConfiguration command"
Export-ModuleMember -function Set-LmAFEConfiguration -Alias Set-AFEConfiguration

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Get-HAOption
Function Get-ClusterHAConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN))) {
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation))) {
			$hamode = Get-ClusterHAMode -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$hamode = Get-ClusterHAMode -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$hamode = Get-ClusterHAMode -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if ($hamode.ReturnCode -ne 200) {
		# ERROR
		$errorString = $hamode.Response
		setKempAPIReturnObject 401 "$errorString" $null
		return
	}

	if ($hamode.ReturnCode -eq 200) {
		# Testing the HA mode
		if ($($hamode.Data).HaMode -eq 0 -or $($hamode.Data).HaMode -ge 3) {
			$errorString = $hamode.Response
			setKempAPIReturnObject 401 "The machine is not in HA mode." $null
			return
		}
	}

	$parameters = @("HAIf","HAInitial","HAPrefered","HAStyle","HATimeout","HAVhid","HAWait","MCast","Vmac","TCPFailover","CookieUpdate","FinalPersist")
	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		if ($response.ToString().Contains("non HA mode"))
		{
			"This unit is not in HA mode."
			break
		}
		$settings.add($param, $response.firstchild.InnerText)
	}
	$tmp = New-Object -TypeName PSObject -Property $settings
	setKempAPIReturnObject 200 "Command successfully executed." $tmp
}
New-Alias -Name Get-HAOption -value Get-ClusterHAConfiguration -Description "Alias for Get-HAOption command"
Export-ModuleMember -function Get-ClusterHAConfiguration -Alias Get-HAOption

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-HAOption
Function Set-ClusterHAConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int16]$HAIf,
		[bool]$HAInitial,
		[ValidateRange(0,2)]
		[Int16]$HAPrefered,
		[bool]$HAStyle,
		[ValidateRange(1,5)]
		[Int16]$HATimeout,
		[ValidateRange(0,255)]
		[Int16]$HAVhid,
		[ValidateRange(0,200)]
		[Int16]$HAWait,
		[Int16]$MCast,
		[bool]$Vmac,
		[bool]$TCPFailover,
		[bool]$CookieUpdate,
		[ValidateScript({($_ -eq 0) -or (($_ -ge 60) -and ($_ -le 86400))})]
		[Int32]$FinalPersist,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	foreach ($entry in $propertyTable.Keys)
	{
		if (isLoginParam $entry)
		{
			continue
		}
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName $entry -ParameterValue $propertyTable[$entry] -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-HAOption  -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-HAOption  -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-HAOption  -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-HAOption -value Set-ClusterHAConfiguration -Description "Alias for Set-HAOption command"
Export-ModuleMember -function Set-ClusterHAConfiguration -Alias Set-HAOption

# Internal use only
Function ConvertHaModeStringToInt($HaMode)
{
	switch($HaMode)
	{
		"SingleMode"
		{
			$HaModeIntValue = 0
			break
		}
		"First"
		{
			$HaModeIntValue = 1
			break
		}
		"Second"
		{
			$HaModeIntValue = 2
			break
		}
		default
		{
			$HaModeIntValue = -1
			break
		}
	}
	return $HaModeIntValue
}

Function GetHaModeString($HaMode)
{
	switch($HaMode)
	{
		0 { $HaModeStringValue = "No HA mode" }
		1 { $HaModeStringValue = "HA First" }
		2 { $HaModeStringValue = "HA Second" }
		3 { $HaModeStringValue = "Cloud HA" }
		4 { $HaModeStringValue = "N+M Cluster" }

		default { $HaModeStringValue = "HA mode UNKNOWN" }
	}
	return $HaModeStringValue
}

Function Get-ClusterHAMode
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$propertyTable.Add("param", "hamode")

	try {
		$HaMode_R = sendRequest2LM "get" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$HaMode_E = $_.Exception.Message
	}

	if ($HaMode_R.hasChildNodes) {
		$HaMode = $HaMode_R | Convert-XmlToPSObject
		Write-Verbose "answer [$HaMode]"

		$HaModeValue = $HaMode.hamode
		if ($HaModeValue -ge 0 -and $HaModeValue -le 4) {
			# OK
			$HaModeDescription = GetHaModeString $HaModeValue
			$tmp = [ordered]@{}
			$tmp.Add("HaMode", $HaModeValue)
			$tmp.Add("HaDescription", $HaModeDescription)
			$dataObj = New-Object -TypeName PSObject -Prop $tmp
			setKempAPIReturnObject 200 "Command successfully executed" $dataObj
		}
		else {
			# ERROR
			Write-Verbose "ERROR: invalid HA mode value [$HaModeValue]"
			setKempAPIReturnObject 401 "ERROR: invalid HA mode value" $null
		}
	}
	else {
		# ERROR
		if ($HaMode_E) {
			Write-Verbose "ERROR [$HaMode_E]"
			setKempAPIReturnObject 401 "$HaMode_E" $null
		}
		else {
			Write-Verbose "ERROR [$HaMode_R]"
			setKempAPIReturnObject 401 "$HaMode_R" $null
		}
	}
}
Export-ModuleMember -function Get-ClusterHAMode

Function Set-ClusterHAMode
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateSet("SingleMode", "First", "Second")]
		[string]$HaMode,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$intHaMode = ConvertHaModeStringToInt $HaMode
	if ($intHaMode -lt 0) {
		setKempAPIReturnObject 401 "System error: not allowed HA mode value [$intHaMode]" $null
		return
	}
	$propertyTable.Remove("HaMode")
	$propertyTable.Add("param", "hamode")
	$propertyTable.Add("value", $intHaMode -as [int])

	try {
		$sHaMode_R = sendRequest2LM "set" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$sHaMode_R = $_.Exception.Message
	}
	if ($sHaMode_R -ne "ok") {
		Write-Verbose "answer [$sHaMode_R]"
		setKempAPIReturnObject 401 "$sHaMode_R" $null
		return
	}
	setKempAPIReturnObject 200 "Command successfully executed" $null
}
Export-ModuleMember -function Set-ClusterHAMode

#Function Get-MultiWui
Function Get-SecMultipleWebUIAccess
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	param(
		[validatenotnullorempty()]
		[string]$LoadBalancer = $loadbalanceraddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$parameters = @("multihomedwui")
	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBmessage -command get -parametername $param -loadbalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBmessage -command get -parametername $param -loadbalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBmessage -command get -parametername $param -loadbalancer $LoadBalancer -LBPort $LBPort -credential $credential
		}
		$settings.add($param, $response.firstchild.innertext)
	}
	new-object -typename psobject -property $settings
}
New-Alias -Name Get-MultiWui -value Get-SecMultipleWebUIAccess -Description "Alias for Get-MultiWui command"
Export-ModuleMember -function Get-SecMultipleWebUIAccess -Alias Get-MultiWui

#Function Set-MultiWui
Function Set-SecMultipleWebUIAccess
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param( 
		[Parameter(Mandatory=$true,Position=0)]
		[string]$multi,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command set -ParameterName multihomedwui -ParameterValue $multi -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName multihomedwui -ParameterValue $multi -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command set -ParameterName multihomedwui -ParameterValue $multi -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-MultiWui  -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-MultiWui  -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Get-MultiWui  -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-MultiWui -value Set-SecMultipleWebUIAccess -Description "Alias for Set-MultiWui command"
Export-ModuleMember -function Set-SecMultipleWebUIAccess -Alias Set-MultiWui

#Function Get-LogSplitInterval
Function Get-AdcL7LogInsightSplitConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	param(
		[validatenotnullorempty()]
		[string]$LoadBalancer = $loadbalanceraddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$parameters = @("logsplitinterval")
	$settings = @{}
	foreach ($param in $parameters)
	{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$response = Send-LBMessage -command get -parametername $param -loadbalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -command get -parametername $param -loadbalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -command get -parametername $param -loadbalancer $LoadBalancer -LBPort $LBPort -credential $Credential
		}
		$settings.add($param, $response.firstchild.innertext)
	}
	new-object -typename psobject -property $settings
}
New-Alias -Name Get-LogSplitInterval -value Get-AdcL7LogInsightSplitConfiguration -Description "Alias for Get-LogSplitInterval command"
Export-ModuleMember -function Get-AdcL7LogInsightSplitConfiguration -Alias Get-LogSplitInterval

#Function Set-LogSplitInterval
Function Set-AdcL7LogInsightSplitConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param( 
		[Parameter(Mandatory=$true,Position=0)]
		[string]$logsplit,	

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command set -ParameterName logsplitinterval -ParameterValue $logsplit -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command set -ParameterName logsplitinterval -ParameterValue $logsplit -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command set -ParameterName logsplitinterval -ParameterValue $logsplit -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			Get-LogSplitInterval -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Get-LogSplitInterval -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
	Get-LogSplitInterval -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name Set-LogSplitInterval -value Set-AdcL7LogInsightSplitConfiguration -Description "Alias for Set-LogSplitInterval command"
Export-ModuleMember -function Set-AdcL7LogInsightSplitConfiguration -Alias Set-LogSplitInterval

#Function Set-AzureHAMode
Function Set-ClusterAzureHAMode
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateSet("master", "slave", "single")]
		[string]$HAMode,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "azurehamode" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Set-AzureHAMode -value Set-ClusterAzureHAMode -Description "Alias for Set-AzureHAMode command"
Export-ModuleMember -function Set-ClusterAzureHAMode -Alias Set-AzureHAMode

#Function Get-AzureHAOption
Function Get-ClusterAzureHAConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command getazurehaparams -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command getazurehaparams -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command getazurehaparams -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	$response | Convert-XmlToPSObject
}
New-Alias -Name Get-AzureHAOption -value Get-ClusterAzureHAConfiguration -Description "Alias for Get-AzureHAOption command"
Export-ModuleMember -function Get-ClusterAzureHAConfiguration -Alias Get-AzureHAOption

#Function Set-AzureHAOption
Function Set-ClusterAzureHAConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$Partner,
		[ValidateNotNullOrEmpty()]
		[string]$Hcp,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (!$Partner -and !$Hcp) {
		Throw "One or both parameters must be provided."
	}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "azurehaparam" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Set-AzureHAOption -value Set-ClusterAzureHAConfiguration -Description "Alias for Set-AzureHAOption command"
Export-ModuleMember -function Set-ClusterAzureHAConfiguration -Alias Set-AzureHAOption

#Function Set-AwsHAMode
Function Set-ClusterAwsHAMode
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateSet("master", "slave", "single")]
		[string]$HAMode,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "awshamode" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Set-AwsHAMode -value Set-ClusterAwsHAMode -Description "Alias for Set-AwsHAMode command"
Export-ModuleMember -function Set-ClusterAwsHAMode -Alias Set-AwsHAMode

#Function Get-AwsHAOption
Function Get-ClusterAwsHaConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command getawshaparams -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command getawshaparams -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command getawshaparams -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	$response | Convert-XmlToPSObject
}
New-Alias -Name Get-AwsHAOption -value Get-ClusterAwsHaConfiguration -Description "Alias for Get-AwsHAOption command"
Export-ModuleMember -function Get-ClusterAwsHaConfiguration -Alias Get-AwsHAOption

#Function Set-AwsHAOption
Function Set-ClusterAwsHAConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$Partner,
		[ValidateNotNullOrEmpty()]
		[string]$Hcp,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (!$Partner -and !$Hcp) {
		Throw "One or both parameters must be provided."
	}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "awshaparam" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name Set-AwsHAOption -value Set-ClusterAwsHAConfiguration -Description "Alias for Set-AwsHAOption command"
Export-ModuleMember -function Set-ClusterAwsHAConfiguration -Alias Set-AwsHAOption

# Internal use only
Function GetLmInterfacesDetails($LoadBalancer, $LBPort, $Credential, $SubjectCN, $CertLoc)
{
	try {
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertLoc))) {
				$networkData = Get-NetworkInterface -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertLoc
			}
			else {
				$networkData = Get-NetworkInterface -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$networkData = Get-NetworkInterface -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
	}
	catch {
		return
	}

	if ($networkData.ReturnCode) {
		if ($networkData.ReturnCode -ge 400) {
			$networkData = $null
		}
	}

	if ($networkData) {
		$networkData
	}
	return
}

# Internal use only
Function GetSecRemoteAccessData($LMIP, $LBPort, $Credential, $SubjectCN, $CertLoc)
{
	$parameters = @("wuiiface", "wuiport", "admingw")
	$settings = @{}

	try {
		Foreach ($param in $parameters)
		{
			if (-not ([String]::IsNullOrEmpty($SubjectCN))) {
				if (-not ([String]::IsNullOrEmpty($CertLoc))) {
					$res = Send-LBMessage -command get -parametername $param -LoadBalancer $LMIP -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertLoc
				}
				else {
					$res = Send-LBMessage -command get -parametername $param -LoadBalancer $LMIP -LBPort $LBPort -SubjectCN $SubjectCN
				}
			}
			else {
				$res = Send-LBMessage -command get -parametername $param -LoadBalancer $LMIP -LBPort $LBPort -Credential $Credential
			}
			$settings.add($param, $res.firstchild.InnerText)
			Start-Sleep -m 500
		}
		New-Object -TypeName PSObject -Property $settings
	}
	catch {
		$response = $_.Exception.Message
		setKempAPIReturnObject 401 "$response" $null
		return
	}
}

# Internal use only
Function checkInterface($interfaceDetails, $WuiIface)
{
	if ($interfaceDetails) {
		if ($interfaceDetails -is [array]) {
			$netArrayLen = $interfaceDetails.Length
			$i = 0
			For($i = 0; $i -lt $netArrayLen; $i++) {
				$id = $interfaceDetails[$i].Id
				if ($id -eq $WuiIface) {
					return $true
				}
			}
		}
		else {
			$id = $interfaceDetails.Id
			if ($id -eq $WuiIface) {
				return $true
			}
		}
	}
	return $false
}

# Internal use only
Function GetCurrentIntfId($interfaceDetails, $LoadBalancer)
{
	if ($interfaceDetails) {
		if ($interfaceDetails -is [array]) {
			$netArrayLen = $interfaceDetails.Length
			$i = 0
			For($i = 0; $i -lt $netArrayLen; $i++) {
				$lmIP = $interfaceDetails[$i].IPAddress 
				$id = $interfaceDetails[$i].Id
				if ($lmIP.Contains($LoadBalancer)) {
					return $id
				}
			}
		}
		else {
			$lmIP = $interfaceDetails.IPAddress 
			$id = $interfaceDetails.Id
			if ($lmIP.Contains($LoadBalancer)) {
				return $id
			}
		}
	}
	return "-1"
}

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function Set-AdminAccess
Function Set-SecAdminAccess
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateRange(0,99)]
		[Int32]$WuiIface = -1,

		[Parameter(Mandatory=$true)]
		[ValidateRange(3,65530)]
		[int32]$WuiPort,

		# TODO to add a validation
		#[ValidateScript({$_ -match [IPAddress]$_ })]
		[string]$WuiDefaultGateway,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	if ($WuiIface -lt 0) {
		Throw "ERROR: the WuiIface parameter is mandatory"
		return
	}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if ($WuiIface) {
		$propertyTable.Remove("WuiIface")
		$propertyTable.Add("wuiiface", $WuiIface)
	}

	if ($WuiPort) {
		$propertyTable.Remove("WuiPort")
		$propertyTable.Add("wuiport", $WuiPort)
	}

	if ($WuiDefaultGateway) {
		$propertyTable.Remove("WuiDefaultGateway")
		$propertyTable.Add("wuidefaultgateway", $WuiDefaultGateway)
	}

	$interfaceDetails = GetLmInterfacesDetails $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	if (([String]::IsNullOrEmpty($interfaceDetails))) {
		setKempAPIReturnObject 401 "Connection error. Please check your credentials and/or connection parameters." $null
		return
	}

	$intfCheck = checkInterface $interfaceDetails $WuiIface
	if ($intfCheck -eq $false) {
		setKempAPIReturnObject 401 "Out of range interface id value [$WuiIface]. Please check your input." $null
		return
	}
	Write-Verbose "Interface id [$WuiIface] is OK"
	$currIntfId = GetCurrentIntfId $interfaceDetails $LoadBalancer
	Write-Verbose "Current Intf id is [$currIntfId]"

	if ($currIntfId -eq $WuiIface) {
		Write-Verbose "Interface id has not changed"
		$reconnectIP = $LoadBalancer
		$reconnectPort = $LBPort
		$newIntfIdx = $currIntfId
	}
	else {
		Write-Verbose "Interface id has changed"
		$tmpReconnectIP = $interfaceDetails[$WuiIface].IPAddress
		$sIdx = $tmpReconnectIP.IndexOf("/")
		$reconnectIP = $tmpReconnectIP.Substring(0, $sIdx)
		$newIntfIdx = $WuiIface
	}

	if ($LBPort -eq $WuiPort) {
		$reconnectPort = $LBPort
	}
	else {
		$reconnectPort = $WuiPort
	}

	try {
		$response = sendRequest2LM "setadminaccess" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$response = $_.Exception.Message
		setKempAPIReturnObject 401 "$response" $null
		return
	}

	$wait = $true
	$counter = 1
	while ($wait -eq $true) {
		Write-Verbose "Waiting to reconnect to the LM . . .($counter)"
		Start-Sleep -s 10

		if (-not (Test-ServerConnection -ComputerName $reconnectIP -Port $reconnectPort)) {
			$counter += 1
			if ($counter -eq 10) {
				setKempAPIReturnObject 401 "ERROR: Unable to re-connect to the LM. Reboot the LM." $null
				return
			}
		}
		else {
			$secAdminData = GetSecRemoteAccessData $reconnectIP $reconnectPort $Credential $SubjectCN $CertificateStoreLocation
			$wait = $false
		}
	}
	setKempAPIReturnObject 200 "Command successfully executed." $secAdminData
	return
}
New-Alias -Name Set-AdminAccess -value Set-SecAdminAccess -Description "Alias for Set-AdminAccess command"
Export-ModuleMember -function Set-SecAdminAccess -Alias Set-AdminAccess

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function AfeClientLimitAdd
Function Add-LmIPConnectionLimit
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[String]$L7addr,
		[Parameter(Mandatory=$true)]
		[int32]$L7limit,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "afeclientlimitadd" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	$response
}
New-Alias -Name AfeClientLimitAdd -value Add-LmIPConnectionLimit -Description "Alias for AfeClientLimitAdd command"
Export-ModuleMember -function Add-LmIPConnectionLimit -Alias AfeClientLimitAdd

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function AfeClientLimitDelete
Function Remove-LmIPConnectionLimit
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[String]$L7addr,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "afeclientlimitdel" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	$response
}
New-Alias -Name AfeClientLimitDelete -value Remove-LmIPConnectionLimit -Description "Alias for AfeClientLimitDelete command"
Export-ModuleMember -function Remove-LmIPConnectionLimit -Alias AfeClientLimitDelete

#.ExternalHelp Kemp.LoadBalancer.Powershell-Help.xml
#Function AfeClientLimitList
Function Get-LmIPConnectionLimit
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$response = sendRequest2LM "afeclientlimitlist" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	$response.ClientLimit | Convert-XmlToPSObject
}
New-Alias -Name AfeClientLimitList -value Get-LmIPConnectionLimit -Description "Alias for AfeClientLimitList command"
Export-ModuleMember -function Get-LmIPConnectionLimit -Alias AfeClientLimitList

#endregion - Miscellaneous Options

#region - GeoIPBlacklist/Whitelist

Function setKempAPIReturnObject($retCode, $apiResponse, $data)
{
	$tempApiRetObj = [ordered]@{}
	$tempApiRetObj.PSTypeName = "KempAPI"
	$tempApiRetObj.ReturnCode = $retCode
	$tempApiRetObj.Response = $apiResponse
	$tempApiRetObj.Data = $data

	$kempApiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj

	$kempApiRetObject
}

Function setGetGeoAclBlacklistConfigurationAnswer($autoupdate, $lastupdate, $autoinstall, $installtime, $lastinstall)
{
	$tempGeoAcl = @{}
	$tempGeoAcl.PSTypeName = "GeoAcl"
	$tempGeoAcl.DatabaseAutoUpdate = $autoupdate
	$tempGeoAcl.DatabaseLastUpdate = $lastupdate
	$tempGeoAcl.DatabaseAutoInstall = $autoinstall
	$tempGeoAcl.DatabaseInstallTimeHour = $installtime
	$tempGeoAcl.DatabaseLastInstall = $lastinstall

	$geoAclAnswer = New-Object -TypeName PSObject -Prop $tempGeoAcl

	$geoAclAnswer
}

Function getDateFromString($dateString)
{
	if ($dateString -ne "Never")
	{
		$date_s = [Datetime]::ParseExact($dateString,"d MMM yyyy HH:mm:ss", $null)
	}
	else {
		$date_s = $null
	}
	$date_s
}

Function handleGeoIPBlacklistAnswer()
{
	$tempApiRetObj = @{}
	$tempApiRetObj.PSTypeName = "KempAPI"
	$tempApiRetObj.ReturnCode = 200
	$tempApiRetObj.Response = "Command successfully executed"
	$tempApiRetObj.Data = $null

	$kempApiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj

	$kempApiRetObject
}

# (1) Function Get-GeoIPBlacklistDatabaseConfiguration
Function Get-GeoIPBlacklistDatabaseConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$ErrorActionPreference = "Stop"
	try {
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -Command geoacl/getsettings -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command geoacl/getsettings -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command geoacl/getsettings -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
		if ($response -is [string] -and $response.contains("401"))
		{
			Throw "ERROR: $response"
		}

		[string]$r_string = $response | Convert-XmlToPSObject

		$eqp = $r_string.IndexOf("=")
		$lbp = $r_string.IndexOf("}")

		$r_string = $r_string.substring($eqp + 1, $lbp - $eqp - 1)

		$p1 = $r_string.IndexOf(",")					# auto update
		$p2 = $r_string.IndexOf(",", $p1 + 1)	# last update date
		$p3 = $r_string.IndexOf(",", $p2 + 1)	# auto install
		$p4 = $r_string.IndexOf(",", $p3 + 1)	# default auto install hour

		$autoupdate  = $r_string.substring(0, $p1)
		$autoinstall = $r_string.substring($p2 + 1, $p3 - $p2 - 1)
		$installtime = $r_string.substring($p3 + 1, $p4 - $p3 - 1)

		$lastupdate_s = $r_string.substring($p1 + 1, $p2 - $p1 - 1)
		$lastupdate   = getDateFromString $lastupdate_s

		$lastinstall_s = $r_string.substring($p4 + 1)
		$lastinstall   = getDateFromString $lastinstall_s

		$geoACL = setGetGeoAclBlacklistConfigurationAnswer $autoupdate $lastupdate $autoinstall $installtime $lastinstall

		$retCode = 200
		$apiResponse = "Command successfully executed"

		$kempAPI = setKempAPIReturnObject $retCode $apiResponse $geoACL

		$kempAPI
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Get-GeoIPBlacklistDatabaseConfiguration

# (2) Function Set-GeoIPBlacklistDatabaseConfiguration
Function Set-GeoIPBlacklistDatabaseConfiguration
{
	Param(
		[Parameter(ParameterSetName="Update", Mandatory=$True)]
			[switch]$DatabaseAutoUpdate,

		[Parameter(ParameterSetName="Install")]
			[switch]$DatabaseAutoInstall,

		[Parameter(ParameterSetName="Install")]
		[ValidateRange(0,23)]
		[ValidateNotNullOrEmpty()]
		[Int32]$DatabaseInstallTimeHour,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred,

		[ValidateNotNullOrEmpty()]
		[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[ValidateNotNullOrEmpty()]
		[String]$SubjectCN = $script:SubjectCN
	)

	$ErrorActionPreference = "Stop"
	if ( (-not ([String]::IsNullOrEmpty($SubjectCN))) -and (-not ([String]::IsNullOrEmpty($Credential))) )
	{
		Throw "ERROR: Can only set one login method, certificate or credential, not both."
	}

	$tempPT = Convert-BoundParameters -hashtable $psboundparameters
	$propertyTable = $tempPT.GetEnumerator() | sort-object -Property Name
	try {
		ForEach ($h in $propertyTable) {
			switch($($h.Name))
			{
				"DatabaseAutoUpdate"
				{
					$cmd2do = "geoacl/setautoupdate"
					if($($h.Value) -eq $true) {
						$dbUpdateParam = @{"enable"=1}
					}
					else {
						$dbUpdateParam = @{"enable"=0}
					}
					break
				}
				"DatabaseAutoInstall"
				{
					$cmd2do = "geoacl/setautoinstall"
					if($($h.Value) -eq $true) {
						$dbUpdateParam = @{"enable"=1}
					}
					else {
						$dbUpdateParam = @{"enable"=0}
					}
					break
				}
				"DatabaseInstallTimeHour"
				{
					$cmd2do = "geoacl/setinstalltime"
					$dbUpdateParam = @{"hour"=$($h.Value)}
					break
				}
			}
			if($cmd2do -eq $null) {
				continue
			}
			$response = sendRequest2LM $cmd2do $dbUpdateParam $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
			if ($response -ne "ok")
			{
				Throw "ERROR: $response"
			}
			$cmd2do = $null
		}
		$kempAPI = handleGeoIPBlacklistAnswer
		$kempAPI
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Set-GeoIPBlacklistDatabaseConfiguration

# (3) Function Update-GeoIPBlacklistDatabase
Function Update-GeoIPBlacklistDatabase
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$ErrorActionPreference = "Stop"
	try {
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -Command geoacl/updatenow -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command geoacl/updatenow -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command geoacl/updatenow -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}

		$kempAPI = handleGeoIPBlacklistAnswer
		$kempAPI
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Update-GeoIPBlacklistDatabase

# (4) Function Install-GeoIPBlacklistDatabase
Function Install-GeoIPBlacklistDatabase
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$ErrorActionPreference = "Stop"
	try {
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -Command geoacl/installnow -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command geoacl/installnow -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command geoacl/installnow -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}

		$kempAPI = handleGeoIPBlacklistAnswer
		$kempAPI
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Install-GeoIPBlacklistDatabase

# (5) Function Export-GeoIPBlacklistDatabase
Function Export-GeoIPBlacklistDatabase
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$filename,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$ErrorActionPreference = "Stop"
	$rpath = split-path "$filename"
	if (-NOT (Test-Path $rpath -PathType 'Container'))
	{
		Throw "$($rpath) is not a valid folder"
	}
	$fname = split-path "$filename" -leaf
	if ([string]::IsNullOrEmpty($fname))
	{
		Throw "file name is NULL"
	}
	if ((Test-Path $fname -PathType 'Container'))
	{
		Throw "file name is a folder"
	}
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command geoacl/downloadlist -File $filename -Output -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command geoacl/downloadlist -File $filename -Output -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command geoacl/downloadlist -File $filename -Output -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	$kempAPI = handleGeoIPBlacklistAnswer
	$kempAPI
}
Export-ModuleMember -function Export-GeoIPBlacklistDatabase

# (6) Function Export-GeoIPBlacklistDatabaseChanges
Function Export-GeoIPBlacklistDatabaseChanges
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$filename,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$ErrorActionPreference = "Stop"
	$rpath = split-path "$filename"
	if (-NOT (Test-Path $rpath -PathType 'Container'))
	{
		Throw "$($rpath) is not a valid folder"
	}
	$fname = split-path "$filename" -leaf
	if ([string]::IsNullOrEmpty($fname))
	{
		Throw "file name is NULL"
	}
	if ((Test-Path $fname -PathType 'Container'))
	{
		Throw "file name is a folder"
	}
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command geoacl/downloadchanges -File $filename -Output -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command geoacl/downloadchanges -File $filename -Output -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command geoacl/downloadchanges -File $filename -Output -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	$kempAPI = handleGeoIPBlacklistAnswer
	$kempAPI
}
Export-ModuleMember -function Export-GeoIPBlacklistDatabaseChanges

# (7) Function Add-GeoIPWhitelist
Function Add-GeoIPWhitelist
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Addr,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$ErrorActionPreference = "Stop"
	try {
		$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
		$response = sendRequest2LM "geoacl/addcustom" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		$kempAPI = handleGeoIPBlacklistAnswer
		$kempAPI
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Add-GeoIPWhitelist

# (8) Function Remove-GeoIPWhitelist
Function Remove-GeoIPWhitelist
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Addr,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$ErrorActionPreference = "Stop"
	try {
		$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
		$response = sendRequest2LM "geoacl/removecustom" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		$kempAPI = handleGeoIPBlacklistAnswer
		$kempAPI
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Remove-GeoIPWhitelist

# (9) Function Export-GeoIPWhitelistDatabase
Function Export-GeoIPWhitelistDatabase
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$filename,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$ErrorActionPreference = "Stop"
	$rpath = split-path "$filename"
	if (-NOT (Test-Path $rpath -PathType 'Container'))
	{
		Throw "$($rpath) is not a valid folder"
	}
	$fname = split-path "$filename" -leaf
	if ([string]::IsNullOrEmpty($fname))
	{
		Throw "file name is NULL"
	}
	if ((Test-Path $fname -PathType 'Container'))
	{
		Throw "file name is a folder"
	}
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command geoacl/listcustom -Output -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command geoacl/listcustom -Output -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command geoacl/listcustom -Output -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}

	[string]$r_string = $response | Convert-XmlToPSObject
	$eqp = $r_string.IndexOf("=")
	$lbp = $r_string.IndexOf("}")
	$r_string = $r_string.substring($eqp + 1, $lbp - $eqp - 1)

	$today = Get-Date
	Out-File $filename -NoClobber
	Add-Content $filename "----------------------------------------------"
	Add-Content $filename " Whitelisted IPs ($today)"
	Add-Content $filename "----------------------------------------------"
	$r_string.Split(',') | ForEach {
		Add-Content $filename "$_"
	}
	$kempAPI = handleGeoIPBlacklistAnswer
	$kempAPI
}
Export-ModuleMember -function Export-GeoIPWhitelistDatabase

#endregion - GeoIPBlacklist/Whitelist

#region - WAF

#Function GetWafSettings
Function Get-WafRulesAutoUpdateConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$response = sendRequest2LM "getwafsettings" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	if ($response -is [string] -and ($response.contains("401") -or $response.Contains("Unable to connect")))
	{
		Throw "ERROR: $response"
	}

	[string]$r_string = $response | Convert-XmlToPSObject

	$eqp = $r_string.IndexOf("=")
	$lbp = $r_string.IndexOf("}")

	$r_string = $r_string.substring($eqp + 1, $lbp - $eqp - 1)
	$r_string = $r_string -replace "`n|`r",""

	$p1 = $r_string.IndexOf(",")					# auto update
	$p2 = $r_string.IndexOf(",", $p1 + 1)	# auto install
	$p3 = $r_string.IndexOf(",", $p2 + 1)	# default auto install hour
	$p4 = $r_string.IndexOf(",", $p3 + 1)	# last update date

	$autoupdate   = $r_string.substring(0, $p1)
	$autoinstall  = $r_string.substring($p1 + 1, $p2 - $p1 - 1)
	$installtime  = $r_string.substring($p2 + 1, $p3 - $p2 - 1)
	$lastupdate_tmp = $r_string.substring($p3 + 1)

	$idx1 = $lastupdate_tmp.IndexOf("-")
	$idx2 = $lastupdate_tmp.IndexOf("-", $idx1 + 1)
	$dd = $lastupdate_tmp.Substring($idx1 + 1, $idx2 - $idx1 - 1)
	$mm = $lastupdate_tmp.Substring($idx2 + 1)
	$yy = $lastupdate_tmp.Substring(0, $idx1)
	$lastupdate_s = "$yy/$mm/$dd"
	$lastupdate = [Datetime]::ParseExact($lastupdate_s, "yyyy/MM/d", $null)

	$tempData = @{}
	$tempData.PSTypeName = "Waf"
	$tempData.DatabaseAutoUpdate = $autoupdate
	$tempData.DatabaseLastUpdate = $lastupdate
	$tempData.DatabaseAutoInstall = $autoinstall
	$tempData.DatabaseInstallTimeHour = $installtime

	$Data = New-Object -TypeName PSObject -Prop $tempData

	$answer = setKempAPIReturnObject 200 "Command successfully executed" $Data
	$answer
}
New-Alias -Name GetWafSettings -value Get-WafRulesAutoUpdateConfiguration -Description "Alias for GetWafSettings command"
Export-ModuleMember -function Get-WafRulesAutoUpdateConfiguration -Alias GetWafSettings

Function Set-WafRulesAutoUpdateConfiguration
{
	Param(
		[Parameter(ParameterSetName="AutoUpdate", Mandatory=$True)]
			[switch]$DatabaseAutoUpdate,

		[Parameter(ParameterSetName="AutoInstall")]
			[switch]$DatabaseAutoInstall,

		[Parameter(ParameterSetName="AutoInstall")]
			[ValidateRange(0,23)]
			[ValidateNotNullOrEmpty()]
			[Int32]$DatabaseInstallTimeHour,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred,

		[ValidateNotNullOrEmpty()]
		[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[ValidateNotNullOrEmpty()]
		[String]$SubjectCN = $script:SubjectCN
	)

	if ( (-not ([String]::IsNullOrEmpty($SubjectCN))) -and (-not ([String]::IsNullOrEmpty($Credential))) )
	{
		Throw "ERROR: Can only set one login method, certificate or credential, not both."
	}

	$ErrorActionPreference = "Stop"
	$tempPT = Convert-BoundParameters -hashtable $psboundparameters
	$propertyTable = $tempPT.GetEnumerator() | sort-object -Property Name
	try {
		ForEach ($h in $propertyTable) {
			switch($($h.Name))
			{
				"DatabaseAutoUpdate"
				{
					$cmd2do = "setwafautoupdate"
					if($($h.Value) -eq $true) {
						$dbUpdateParam = @{"Enable"=1}
					}
					else {
						$dbUpdateParam = @{"Enable"=0}
					}
					break
				}
				"DatabaseAutoInstall"
				{
					$cmd2do = "enablewafautoinstall"
					if($($h.Value) -eq $true) {
						$dbUpdateParam = @{"Enable"=1}
					}
					else {
						$dbUpdateParam = @{"Enable"=0}
					}
					break
				}
				"DatabaseInstallTimeHour"
				{
					$cmd2do = "setwafinstalltime"
					$dbUpdateParam = @{"Hour"=$($h.Value)}
					break
				}
			}
			if($cmd2do -eq $null) {
				continue
			}
			$response = sendRequest2LM $cmd2do $dbUpdateParam $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
			if ($response -ne "ok")
			{
				Throw "$response"
			}
			$cmd2do = $null
		}
		$answer = setKempAPIReturnObject 200 "Command successfully executed" $null
		$answer
	}
	catch [Exception] {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Set-WafRulesAutoUpdateConfiguration

# ----------- WAF LEGACY START ---------------------------
Function SetWafAutoUpdate
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[bool]$Enable,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setwafautoupdate" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
Export-ModuleMember -function SetWafAutoUpdate
# ----------- WAF LEGACY END -----------------------------

#Function DownloadWafRules
Function Update-WafRulesDatabase
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	try {
		sendRequest2LM "downloadwafrules" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		$retCode = 200
		$apiResponse = "Command successfully executed"

		$kempAPI = setKempAPIReturnObject $retCode $apiResponse $null

		$kempAPI
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
New-Alias -Name DownloadWafRules -value Update-WafRulesDatabase -Description "Alias for DownloadWafRules command"
Export-ModuleMember -function Update-WafRulesDatabase -Alias DownloadWafRules

# ----------- WAF LEGACY START ---------------------------
Function EnableWafAutoInstall
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Enable,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "enablewafautoinstall" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name SetWafAutoInstall -value EnableWafAutoInstall -Description "Alias for SetWafAutoInstall command"
Export-ModuleMember -function EnableWafAutoInstall -Alias SetWafAutoInstall

Function SetWafInstallTime
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateRange(0,23)]
		[Int32]$Hour,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setwafinstalltime" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
Export-ModuleMember -function SetWafInstallTime
# ----------- WAF LEGACY END -----------------------------


# ----------- GEO CUSTOM LOCATIONS START -----------------------------
#Function AddCustomLocation
Function Add-GeoCustomLocation
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Location,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "addcustomlocation" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name AddCustomLocation -value Add-GeoCustomLocation -Description "Alias for AddCustomLocation command"
Export-ModuleMember -function Add-GeoCustomLocation -Alias AddCustomLocation

#Function DeleteCustomLocation
Function Remove-GeoCustomLocation
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$clName,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "deletecustomlocation" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DeleteCustomLocation -value Remove-GeoCustomLocation -Description "Alias for DeleteCustomLocation command"
Export-ModuleMember -function Remove-GeoCustomLocation -Alias DeleteCustomLocation

#Function ListCustomLocation
Function Get-GeoCustomLocation
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "listcustomlocation" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.location | Convert-XmlToPSObject
}
New-Alias -Name ListCustomLocation -value Get-GeoCustomLocation -Description "Alias for ListCustomLocation command"
Export-ModuleMember -function Get-GeoCustomLocation -Alias ListCustomLocation

#Function EditCustomLocation
Function Set-GeoCustomLocation
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$clOldName,

		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$clNewName,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "editcustomlocation" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name EditCustomLocation -value Set-GeoCustomLocation -Description "Alias for EditCustomLocation command"
Export-ModuleMember -function Set-GeoCustomLocation -Alias EditCustomLocation
# ----------- GEO CUSTOM LOCATIONS END -----------------------------

#Function AddWafCustomRule
Function Add-WafCustomRuleSet
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Filename,
		[Parameter(Mandatory=$true)]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command addwafcustomrule -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN -SubjectCN $CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command addwafcustomrule -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command addwafcustomrule -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
	$response
}
New-Alias -Name AddWafCustomRule -value Add-WafCustomRuleSet -Description "Alias for AddWafCustomRule command"
Export-ModuleMember -function Add-WafCustomRuleSet -Alias AddWafCustomRule

#Function DelWafCustomRule
Function Uninstall-WafCustomRuleSet
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Filename,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "delwafcustomrule" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DelWafCustomRule -value Uninstall-WafCustomRuleSet -Description "Alias for DelWafCustomRule command"
Export-ModuleMember -function Uninstall-WafCustomRuleSet -Alias DelWafCustomRule

#Function DownloadWafCustomRule
Function Export-WafCustomRule
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[Parameter(Mandatory=$true)]
		[string]$Path,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[String]$Filename
	)
	$params = @{}
	if (-not ($Path))
	{
		$Path = "$($Env:SystemRoot)\Temp\LMBackup_$(Get-Date -format yyyy-MM-dd_HH-mm-ss)"
	}
	Write-Verbose "Path = $Path"

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command downloadwafcustomrule -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command downloadwafcustomrule -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command downloadwafcustomrule -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}

	$response
}
New-Alias -Name DownloadWafCustomRule -value Export-WafCustomRule -Description "Alias for DownloadWafCustomRule command"
Export-ModuleMember -function Export-WafCustomRule -Alias DownloadWafCustomRule

#Function AddWafCustomData
Function Add-WafCustomRuleData
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Filename,
		[Parameter(Mandatory=$true)]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command addwafcustomdata -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command addwafcustomdata -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command addwafcustomdata -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}

	$response
}
New-Alias -Name AddWafCustomData -value Add-WafCustomRuleData -Description "Alias for AddWafCustomData command"
Export-ModuleMember -function Add-WafCustomRuleData -Alias AddWafCustomData

#Function DelWafCustomData
Function Uninstall-WafCustomRuleData
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Filename,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "delwafcustomdata" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DelWafCustomData -value Uninstall-WafCustomRuleData -Description "Alias for DelWafCustomData command"
Export-ModuleMember -function Uninstall-WafCustomRuleData -Alias DelWafCustomData

#Function DownloadWafCustomData
Function Export-WafCustomRuleData
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[Parameter(Mandatory=$true)]
		[string]$Path,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[String]$Filename
	)
	$params = @{}
	if (-not ($Path))
	{
		$Path = "$($Env:SystemRoot)\Temp\LMBackup_$(Get-Date -format yyyy-MM-dd_HH-mm-ss)"
	}
	Write-Verbose "Path = $Path"

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command downloadwafcustomdata -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command downloadwafcustomdata -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command downloadwafcustomdata -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}

	$response
}
New-Alias -Name DownloadWafCustomData -value Export-WafCustomRuleData -Description "Alias for DownloadWafCustomData command"
Export-ModuleMember -function Export-WafCustomRuleData -Alias DownloadWafCustomData

#Function ManInstallWafRules
Function Install-WafRulesDatabase
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	try {
		sendRequest2LM "maninstallwafrules" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		$retCode = 200
		$apiResponse = "Command successfully executed"
 
		$kempAPI = setKempAPIReturnObject $retCode $apiResponse $null
 
		$kempAPI
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
New-Alias -Name ManInstallWafRules -value Install-WafRulesDatabase -Description "Alias for ManInstallWafRules command"
Export-ModuleMember -function Install-WafRulesDatabase -Alias ManInstallWafRules

#Function ListWafAuditFiles
Function Get-WafAuditFiles
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "listwafauditfiles" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.WAFAuditFiles | Convert-XmlToPSObject
}
New-Alias -Name ListWafAuditFiles -value Get-WafAuditFiles -Description "Alias for ListWafAuditFiles command"
Export-ModuleMember -function Get-WafAuditFiles -Alias ListWafAuditFiles

#Function DownloadWafAuditLog
Function Export-WafAuditLog
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[Parameter(Mandatory=$true)]
		[string]$Path,

		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$File,
		[string]$Filter
	)
	$params = @{}
	if (-not ($Path))
	{
		$Path = "$($Env:SystemRoot)\Temp\wafaudit.log"
	}
	Write-Verbose "Path = $Path"

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command downloadwafauditlog -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command downloadwafauditlog -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command downloadwafauditlog -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
	
	$response
}
New-Alias -Name DownloadWafAuditLog -value Export-WafAuditLog -Description "Alias for DownloadWafAuditLog command"
Export-ModuleMember -function Export-WafAuditLog -Alias DownloadWafAuditLog

#Function ListWafRules
Function Get-WafRules
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "listwafrules" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.Rules | Convert-XmlToPSObject
}
New-Alias -Name ListWafRules -value Get-WafRules -Description "Alias for ListWafRules command"
Export-ModuleMember -function Get-WafRules -Alias ListWafRules

#Function GetWafChangeLog
Function Export-WafChangeLog
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[Parameter(Mandatory=$true)]
		[string]$Path
	)
	$params = @{}
	if (-not ($Path))
	{
		$Path = "$($Env:SystemRoot)\Temp\WAF_changes.log"
	}
	Write-Verbose "Path = $Path"
	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -command getwafchangelog -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -command getwafchangelog -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -command getwafchangelog -File $Path -Output -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}

	$response
}
New-Alias -Name GetWafChangeLog -value Export-WafChangeLog -Description "Alias for GetWafChangeLog command"
Export-ModuleMember -function Export-WafChangeLog -Alias GetWafChangeLog

#Function VSAddWafRule
Function Add-AdcWafRule
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$VS,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Port,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Prot,
		[string]$Rule,
		[string]$Disablerules,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "vsaddwafrule" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name VSAddWafRule -value Add-AdcWafRule -Description "Alias for VSAddWafRule command"
Export-ModuleMember -function Add-AdcWafRule -Alias VSAddWafRule

#Function VSRemoveWafRule
Function Remove-AdcWafRule
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$VS,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Port,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Prot,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Rule,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "vsremovewafrule" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name VSRemoveWafRule -value Remove-AdcWafRule -Description "Alias for VSRemoveWafRule command"
Export-ModuleMember -function Remove-AdcWafRule -Alias VSRemoveWafRule

#Function VSListWafRuleIds
Function Get-WafVSRules
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$VS,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Port,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Prot,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[string]$Rule,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "vslistwafruleids" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$rulename = $Rule.Split("/")[1]

	[array]$xmla = $response.$rulename

	New-Variable -Name "xmltag" -Value $xmla[0]

	If($xmltag -eq $rulename){ $response.$rulename | Convert-XmlToPSObject }
	Else{ $xmltag }
}
New-Alias -Name VSListWafRuleIds -value Get-WafVSRules -Description "Alias for VSListWafRuleIds command"
Export-ModuleMember -function Get-WafVSRules -Alias VSListWafRuleIds

Function Set-WafLogFormat
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$LogFormat,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setwaflogformat" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
Export-ModuleMember -function Set-WafLogFormat

#Function EnableWafRemoteLogging
Function Enable-WafRemoteLogging
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$RemoteURI,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$Username,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[string]$Passwd,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "enablewafremotelogging" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name EnableWafRemoteLogging -value Enable-WafRemoteLogging -Description "Alias for EnableWafRemoteLogging command"
Export-ModuleMember -function Enable-WafRemoteLogging -Alias EnableWafRemoteLogging
	
#Function DisableWafRemoteLogging
Function Disable-WafRemoteLogging
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "disablewafremotelogging" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DisableWafRemoteLogging -value Disable-WafRemoteLogging -Description "Alias for DisableWafRemoteLogging command"
Export-ModuleMember -function Disable-WafRemoteLogging -Alias DisableWafRemoteLogging

#endregion - WAF

#region - GEO

#FQDN

#Function AddFQDN
Function Add-GeoFQDN
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "addfqdn" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name AddFQDN -value Add-GeoFQDN -Description "Alias for AddFQDN command"
Export-ModuleMember -function Add-GeoFQDN -Alias AddFQDN

#Function DeleteFQDN
Function Remove-GeoFQDN
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "delfqdn" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DeleteFQDN -value Remove-GeoFQDN -Description "Alias for DeleteFQDN command"
Export-ModuleMember -function Remove-GeoFQDN -Alias DeleteFQDN

# Internal use only
Function SetSelectionCriteria($propertyTable, $SelectionCriteria)
{
	switch ($SelectionCriteria)
	{
		"RoundRobin" {$ss = "rr"}
		"WeightedRoundRobin" {$ss = "wrr"}
		"FixedWeighting" {$ss = "fw"}
		"RealServerLoad" {$ss = "rsr"}
		"Proximity" {$ss = "prx"}
		"LocationBased" {$ss = "lb"}
		"All" {$ss = "all"}
	}

	$propertyTable.Remove("SelectionCriteria")
	$propertyTable.Add("SelectionCriteria", $ss)
}

# Internal use only
Function SetFQDNData($props)
{
	$tmpObj = [ordered]@{}
	$tmpObj.PSTypeName = "FQDNData"
	ForEach($item in $props) {
		switch ($item.Name)
		{
			{ ($_ -in "FullyQualifiedDomainName") } { $tmpObj.Add("FQDN", $($item.Value)) }
			{ ($_ -in "failover") } { $tmpObj.Add("Failover", $($item.Value)) }
			{ ($_ -in "publicRequestValue") } { $tmpObj.Add("PublicRequestValue", $($item.Value)) }
			{ ($_ -in "privateRequestValue") } { $tmpObj.Add("PrivateRequestValue", $($item.Value)) }
			{ ($_ -in "FailTime") } { $tmpObj.Add("SiteFailureDelay", $($item.Value)) }
			{ ($_ -in "SelectionCriteria") } {
				$name = ""
				switch ($($item.Value))
				{
					"rr" { $name = "RoundRobin" }
					"wrr" { $name = "WeightedRoundRobin" }
					"fw" { $name = "FixedWeighting" }
					"rsr" { $name = "RealServerLoad" }
					"prx" { $name = "Proximity" }
					"lb" { $name = "LocationBased" }
					"all" { $name = "All" }
				}
				$tmpObj.Add("SelectionCriteria", $name)
																			 }
			default { $tmpObj.Add($($item.Name), $($item.Value)) }
		}
	}
	$tmpObj
}

# Internal use only
Function handleSetFqdnCorrectAnswer($response)
{
	$r_response = $response.fqdn | Convert-XmlToPSObject
	$props = $($r_response.psobject).Properties
	if ($props) {
		$tmpObj = SetFQDNData $props
		$data = New-Object -TypeName PSObject -Prop $tmpObj
		$answer = setKempAPIReturnObject 200 "Command successfully executed." $data
	}
	else {
		$answer = setKempAPIReturnObject 401 "System error." $null
	}
	$answer
	return
}

# Internal use only
Function handleSetFqdnErrorAnswer($lmResponse)
{
	$retCode = 401
	if ($lmResponse -is [array]) {

		$lmaType = $($lmResponse[1].GetType()).Name

		if ($lmaType -eq "XmlElement") {
			$r_lmResponse = $($lmResponse[1]).Error
			$retCode = $($lmResponse[1]).Stat
		}
		else {
			$r_lmResponse = $lmResponse[0]
		}
	}
	else {
		$r_lmResponse = $lmResponse
	}

	Write-Verbose "r_lmResponse [$r_lmResponse]"

	if ($r_lmResponse.Contains("Unauthorized") -or
	    $r_lmResponse.Contains("Can't find certificate") -or
	    $r_lmResponse.Contains("The remote name could not be resolved")) {
		$errorString = $r_lmResponse
	}
	elseif ($r_lmResponse.Contains("Invalid address format")) {
		$errorString = "Invalid address format"
	}
	elseif ($r_lmResponse.Contains("Unable to connect")) {
		$errorString = "Unable to connect to the remote server"
	}
	else {
		$errorString = $r_lmResponse
	}
	setKempAPIReturnObject $retCode "$errorString" $null
	return
}

#Function ModifyFQDN
Function Set-GeoFQDN
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[ValidateSet("RoundRobin", "WeightedRoundRobin", "FixedWeighting", "RealServerLoad", "Proximity", "LocationBased")]
		[String]$SelectionCriteria,

		[ValidateRange(0,1440)]
		[Int32]$SiteFailureDelay = -1,

		[ValidateSet("auto", "manual")]
		[String]$SiteRecoveryMode,

		[ValidateRange(0,3)]
		[Int16]$PublicRequestValue,

		[ValidateRange(0,3)]
		[Int16]$PrivateRequestValue,

		[String]$Failover,

		[ValidateRange(0,1)]
		[Int16]$LocalSettings,

		[ValidateRange(1,86400)]
		[Int32]$localttl,

		[ValidateRange(0,86400)]
		[Int32]$localsticky,

		[bool]$UnanimousChecks,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if ($SiteFailureDelay -ge 0) {
		$propertyTable.Remove("SiteFailureDelay")
		$propertyTable.Add("FailTime", $SiteFailureDelay)
	}

	if ($SelectionCriteria) {
		SetSelectionCriteria $propertyTable $SelectionCriteria
	}

	try {
		$response = sendRequest2LM "modfqdn" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$response = $_.Exception.Message
		setKempAPIReturnObject 401 "$response" $null
		return
	}

	if ($response.fqdn) {
		$fqdnData = handleSetFqdnCorrectAnswer $response
		$fqdnData
	}
	else {
		handleSetFqdnErrorAnswer $response
	}
	return
}
New-Alias -Name ModifyFQDN -value Set-GeoFQDN -Description "Alias for ModifyFQDN command"
Export-ModuleMember -function Set-GeoFQDN -Alias ModifyFQDN

#Function ListFQDNs
Function Get-GeoFQDN
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if ($FQDN -eq "") {
		$response = sendRequest2LM "listfqdns" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	else {
		$response = sendRequest2LM "showfqdn" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	$response.fqdn | Convert-XmlToPSObject
}
New-Alias -Name ListFQDNs -value Get-GeoFQDN -Description "Alias for ListFQDNs command"
Export-ModuleMember -function Get-GeoFQDN -Alias ListFQDNs

#Function ShowFQDN
#
# LEGACY: it will removed. Use Get-GeoFQDN instead
#
Function Get-GeoSingleFQDN
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "showfqdn" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.fqdn | Convert-XmlToPSObject
}
New-Alias -Name ShowFQDN -value Get-GeoSingleFQDN -Description "Alias for ShowFQDN command"
Export-ModuleMember -function Get-GeoSingleFQDN -Alias ShowFQDN

#Function AddMap
Function Add-GeoFQDNSiteAddress
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[string]$Cluster,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "addmap" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name AddMap -value Add-GeoFQDNSiteAddress -Description "Alias for AddMap command"
Export-ModuleMember -function Add-GeoFQDNSiteAddress -Alias AddMap

#Function DeleteMap
Function Remove-GeoFQDNSiteAddress
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "delmap" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DeleteMap -value Remove-GeoFQDNSiteAddress -Description "Alias for DeleteMap command"
Export-ModuleMember -function Remove-GeoFQDNSiteAddress -Alias DeleteMap

#Function ModifyMap
Function Set-GeoFQDNSiteAddress
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,
		[String]$Checker,
		[Int32]$Weight,
		[String]$Enable,
		[String]$Cluster,
		[String]$Mapaddress,
		[String]$Mapport,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "modmap" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.fqdn.map | Convert-XMLToPSObject
}
New-Alias -Name ModifyMap -value Set-GeoFQDNSiteAddress -Description "Alias for ModifyMap command"
Export-ModuleMember -function Set-GeoFQDNSiteAddress -Alias ModifyMap

#Function ChangeCheckerAddr
Function Set-GeoFQDNSiteCheckerAddress
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[Parameter(Mandatory=$true,Position=2)]
		[ValidateNotNullOrEmpty()]
		[String]$CheckerIP,

		[Parameter(Mandatory=$true,Position=3)]
		[ValidateNotNullOrEmpty()]
		[String]$Port,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "changecheckeraddr" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name ChangeCheckerAddr -value Set-GeoFQDNSiteCheckerAddress -Description "Alias for ChangeCheckerAddr command"
Export-ModuleMember -function Set-GeoFQDNSiteCheckerAddress -Alias ChangeCheckerAddr

#Function AddCountry
Function Add-GeoCountry
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[ValidateNotNullOrEmpty()]
		[String]$CountryCode,

		[ValidateNotNullOrEmpty()]
		[String]$IsContinent,

		[ValidateNotNullOrEmpty()]
		[String]$CustomLocation,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (!$CountryCode -and !$CustomLocation) {
		Throw "A country code or custom location must be provided."
	}

	if ($CountryCode -and !$IsContinent) {
		Throw "Please indicate if country code refers to a continent."
	}

	if (!$CountryCode -and $IsContinent) {
		Throw "IsContinent parameter requires a country code."
	}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "addcountry" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name AddCountry -value Add-GeoCountry -Description "Alias for AddCountry command"
Export-ModuleMember -function Add-GeoCountry -Alias AddCountry

#Function RemoveCountry
Function Remove-GeoCountry
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[ValidateNotNullOrEmpty()]
		[String]$CountryCode,

		[ValidateNotNullOrEmpty()]
		[String]$IsContinent,

		[ValidateNotNullOrEmpty()]
		[String]$CustomLocation,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	if (!$CountryCode -and !$CustomLocation) {
		Throw "A country code or custom location must be provided."
	}

	if ($CountryCode -and !$IsContinent) {
		Throw "Please indicate if country code refers to a continent."
	}

	if (!$CountryCode -and $IsContinent) {
		Throw "IsContinent parameter requires a country code."
	}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "removecountry" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name RemoveCountry -value Remove-GeoCountry -Description "Alias for RemoveCountry command"
Export-ModuleMember -function Remove-GeoCountry -Alias RemoveCountry

#Function ChangeMapLocation
Function Set-GeoFQDNSiteCoordinates
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$FQDN,

		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[Parameter(Mandatory=$true,Position=2)]
		[Int32]$Lat,

		[Parameter(Mandatory=$true,Position=3)]
		[Int32]$Long,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "changemaploc" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name ChangeMapLocation -value Set-GeoFQDNSiteCoordinates -Description "Alias for ChangeMapLocation command"
Export-ModuleMember -function Set-GeoFQDNSiteCoordinates -Alias ChangeMapLocation

#Cluster

#Function ListClusters
Function Get-GeoCluster
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if ($ip -eq "")
	{
		$response = sendRequest2LM "listclusters" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	else {
		$response = sendRequest2LM "showcluster" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	$response.cluster | Convert-XmlToPSObject
}
New-Alias -Name ListClusters -value Get-GeoCluster -Description "Alias for ListClusters command"
New-Alias -Name ShowCluster -value Get-GeoCluster -Description "Alias for ShowCluster command"
Export-ModuleMember -function Get-GeoCluster -Alias ListClusters, ShowCluster

#Function AddCluster
Function Add-GeoCluster
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "addcluster" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name AddCluster -value Add-GeoCluster -Description "Alias for AddCluster command"
Export-ModuleMember -function Add-GeoCluster -Alias AddCluster

#Function DeleteCluster
Function Remove-GeoCluster
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "delcluster" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DeleteCluster -value Remove-GeoCluster -Description "Alias for DeleteCluster command"
Export-ModuleMember -function Remove-GeoCluster -Alias DeleteCluster

#Function ModifyCluster
Function Set-GeoCluster
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[String]$Type,

		[String]$Name,

		[String]$Checker,

		[Int32]$CheckerPort,

		[String]$Enable,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "modcluster" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.cluster | Convert-XMLToPSObject
}
New-Alias -Name ModifyCluster -value Set-GeoCluster -Description "Alias for ModifyCluster command"
Export-ModuleMember -function Set-GeoCluster -Alias ModifyCluster

#Function ClusterChangeLocation
Function Set-GeoClusterCoordinates
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[Parameter(Mandatory=$true,Position=1)]
		[Int32]$LatSecs,

		[Parameter(Mandatory=$true,Position=2)]
		[Int32]$LongSecs,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "clustchangeloc" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name ClusterChangeLocation -value Set-GeoClusterCoordinates -Description "Alias for ClusterChangeLocation command"
Export-ModuleMember -function Set-GeoClusterCoordinates -Alias ClusterChangeLocation

#Misc params

#Function ListMiscParameters
Function Get-LmMiscParameter
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$vals = @{}

	$response = sendRequest2LM "listparams" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$vals.add("Zone", $response.soa.Zone)
	$vals.add("SourceOfAuthority", $response.soa.SourceOfAuthority)
	$vals.add("NameSrv", $response.soa.NameSrv)
	$vals.add("SOAEmail", $response.soa.SOAEmail)
	$vals.add("TTL", $response.soa.TTL)
	$vals.add("Persist", $response.soa.persist)
	$vals.add("CheckInterval", $response.check.CheckInterval)
	$vals.add("ConnTimeout", $response.check.ConnTimeout)
	$vals.add("RetryAttempts", $response.check.RetryAttempts)

	New-Object -TypeName PSObject -Property $vals
}
New-Alias -Name ListMiscParameters -value Get-LmMiscParameter -Description "Alias for ListMiscParameters command"
Export-ModuleMember -function Get-LmMiscParameter -Alias ListMiscParameters

#Function ModifyMiscParameters
Function Set-LmMiscParameter
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[String]$SourceOfAuthority,
		[String]$NameSrv,
		[String]$SOAEmail,
		[String]$TTL,
		[String]$Persist,
		[String]$CheckInterval,
		[String]$ConnTimeout,
		[String]$RetryAttempts,
		[String]$Zone,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$vals = @{}

	$response = sendRequest2LM "modparams" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$vals.add("Zone", $response.soa.Zone)
	$vals.add("SourceOfAuthority", $response.soa.SourceOfAuthority)
	$vals.add("NameSrv", $response.soa.NameSrv)
	$vals.add("SOAEmail", $response.soa.SOAEmail)
	$vals.add("TTL", $response.soa.TTL)
	$vals.add("Persist", $response.soa.persist)
	$vals.add("CheckInterval", $response.check.CheckInterval)
	$vals.add("ConnTimeout", $response.check.ConnTimeout)
	$vals.add("RetryAttempts", $response.check.RetryAttempts)

	New-Object -TypeName PSObject -Property $vals
}
New-Alias -Name ModifyMiscParameters -value Set-LmMiscParameter -Description "Alias for ModifyMiscParameters command"
Export-ModuleMember -function Set-LmMiscParameter -Alias ModifyMiscParameters

#Function LocationDataUpdate
Function Update-GeoDatabase
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "locdataupdate" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation $Path
	$response
}
New-Alias -Name LocationDataUpdate -value Update-GeoDatabase -Description "Alias for LocationDataUpdate command"
Export-ModuleMember -function Update-GeoDatabase -Alias LocationDataUpdate

#IP Range

#Function ListIPs
Function Get-GeoIpRange
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if ($IP -eq "")
	{
		$response = sendRequest2LM "listips" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	else {
		$response = sendRequest2LM "showip" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	$response.IPAddress | Convert-XmlToPSObject
}
New-Alias -Name ListIPs -value Get-GeoIpRange -Description "Alias for ListIPs command"
New-Alias -Name ShowIP -value Get-GeoIpRange -Description "Alias for ShowIP command"
Export-ModuleMember -function Get-GeoIpRange -Alias ListIPs, ShowIP

#Function AddIP
Function New-GeoIPRange
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "addip" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name AddIP -value New-GeoIPRange -Description "Alias for AddIP command"
Export-ModuleMember -function New-GeoIPRange -Alias AddIP

#Function DeleteIP
Function Remove-GeoIPRange
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "delip" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DeleteIP -value Remove-GeoIPRange -Description "Alias for DeleteIP command"
Export-ModuleMember -function Remove-GeoIPRange -Alias DeleteIP

#Function ModifyIPLocation
Function Set-GeoIPRangeCoordinates
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[Parameter(Mandatory=$true,Position=1)]
		[Int32]$Lat,

		[Parameter(Mandatory=$true,Position=2)]
		[Int32]$Long,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "modiploc" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name ModifyIPLocation -value Set-GeoIPRangeCoordinates -Description "Alias for ModifyIPLocation command"
Export-ModuleMember -function Set-GeoIPRangeCoordinates -Alias ModifyIPLocation

#Function DeleteIPLocation
Function Remove-GeoIPRangeCoordinates
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "deliploc" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DeleteIPLocation -value Remove-GeoIPRangeCoordinates -Description "Alias for DeleteIPLocation command"
Export-ModuleMember -function Remove-GeoIPRangeCoordinates -Alias DeleteIPLocation

#Function AddIPCountry
Function Set-GeoIPRangeCountry
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[Parameter(Mandatory=$true,Position=0)]
		[String]$CountryCode,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "addipcountry" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name AddIPCountry -value Set-GeoIPRangeCountry -Description "Alias for AddIPCountry command"
Export-ModuleMember -function Set-GeoIPRangeCountry -Alias AddIPCountry

#Function AddIPCountryCustom
Function Set-GeoIPRangeCustomLocation
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[Parameter(Mandatory=$true,Position=0)]
		[String]$CustomLoc,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "addipcountry" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name AddIPCountryCustom -value Set-GeoIPRangeCustomLocation -Description "Alias for AddIPCountryCustom command"
Export-ModuleMember -function Set-GeoIPRangeCustomLocation -Alias AddIPCountryCustom

#Function RemoveIPCountry
Function Remove-GeoIPRangeCountry
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IP,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "removeipcountry" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name RemoveIPCountry -value Remove-GeoIPRangeCountry -Description "Alias for RemoveIPCountry command"
New-Alias -Name RemoveIPCountryCustom -value Remove-GeoIPRangeCountry -Description "Alias for RemoveIPCountryCustom command"
New-Alias -Name Remove-GeoIPRangeCustomLocation -value Remove-GeoIPRangeCountry -Description "Alias for Remove-GeoIPRangeCustomLocation command"
Export-ModuleMember -function Remove-GeoIPRangeCountry -Alias RemoveIPCountry, RemoveIPCountryCustom, Remove-GeoIPRangeCustomLocation 

#Function EnableGEO
Function Enable-LmGeoPack
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "enablegeo" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name EnableGEO -value Enable-LmGeoPack -Description "Alias for EnableGEO command"
Export-ModuleMember -function Enable-LmGeoPack -Alias EnableGEO

#Function DisableGEO
Function Disable-LmGeoPack
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,
		
		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "disablegeo" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DisableGEO -value Disable-LmGeoPack -Description "Alias for DisableGEO command"
Export-ModuleMember -function Disable-LmGeoPack -Alias DisableGEO

#Function IsGEOEnabled
Function Test-LmGeoEnabled
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "isgeoenabled" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name IsGEOEnabled -value Test-LmGeoEnabled -Description "Alias for IsGEOEnabled command"
Export-ModuleMember -function Test-LmGeoEnabled -Alias IsGEOEnabled

Function Get-GeoPartnerStatus
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = Convert-BoundParameters -hashtable $psboundparameters

	try {
		$answer = sendRequest2LM "getgeopartnerstatus" $params $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		if ($answer -is [array]) {
			$response = $answer[0]
			if ($response.Contains("Unauthorized")) {
				setKempAPIReturnObject 401 "$response" $null
				return
			}
			if ($response.Contains("Unable to connect")) {
				setKempAPIReturnObject 400 "$response" $null
				return
			}
			setKempAPIReturnObject 400 "Unknown error (1)" $null
			return
		}
		else {
			$partners = $answer.GeoPartners.Partner

			$outputParams = [ordered]@{}

			if ($partners) {
				$pa = @()
				foreach($p in $partners) {
					$address = $p.Address
					$status  = $p.Status

					$tht = [ordered]@{}
					$tht.Add("Address", $address) | Out-null
					$tht.Add("Status", $status) | Out-null

					$thtObject = New-Object -TypeName PSObject -Prop $tht

					$pa += $thtObject
					$i += 1
				}
				$outputParams.Add("DefinedGeoPartners", $i) | Out-null
				$outputParams.Add("GeoPartners", $pa) | Out-null
				$apiRetObject = New-Object -TypeName PSObject -Prop $outputParams
			}
			else {
				$outputParams.Add("DefinedGeoPartners", 0) | Out-null
				$outputParams.Add("GeoPartners", $null) | Out-null
				$apiRetObject = New-Object -TypeName PSObject -Prop $outputParams
			}

			setKempAPIReturnObject 200 "Command successfully executed" $apiRetObject
			return
		}
	}
	catch {
		$tmp = $_.Exception.Message
		if ($errorString -is [array]) {
			$errorString = $tmp[0]
		}
		else {
			$errorString = $tmp
		}
		if ($errorString.Contains("Unauthorized")) {
			$errorCode = 401
		}
		else {
			$errorCode = 400
		}
		setKempAPIReturnObject $errorCode "$errorString" $null
	}
}
Export-ModuleMember -function Get-GeoPartnerStatus

# DNSSEC

Function Set-GeoDNSSEC
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[bool]$Enable,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$params = Convert-BoundParameters -hashtable $psboundparameters

	try {
		$response = sendRequest2LM "geosetdnssec" $params $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		if ($response -eq "ok") {
			setKempAPIReturnObject 200 "Command successfully executed" $null
		}
		else {
			if ($response -is [array]) {
				if ($response[1] -is [System.Xml.XmlElement]) {
					$errCode = $response[1].stat
					$errStr = $response[1].Error
				}
				else {
					$errStr = $response[0]
					if ($errStr.Contains("Unauthorized")) {
						$errCode = 401
					}
					else {
						$errCode = 400
					}
				}
			}
			else {
				$errCode = 400
				$errStr = $response
			}
			setKempAPIReturnObject $errCode "$errStr" $null
		}

	}
	catch {
		$errStr = $_.Exception.Message
		setKempAPIReturnObject 400 "$errStr" $null
	}
}
Export-ModuleMember -function Set-GeoDNSSEC

Function New-GeoDNSSECKeySigningKey
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateSet("RSASHA256","RSASHA512", "NSEC3RSASHA1")]
		[String]$SigningAlgorithm,

		[ValidateSet("1024","2048", "4096")]
		[int]$SigningKeySize,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$params = Convert-BoundParameters -hashtable $psboundparameters

	if ($SigningAlgorithm) {
		$params.Remove("SigningAlgorithm")
		$params.Add("algorithm", $SigningAlgorithm)
	}

	if ($SigningKeySize) {
		$params.Remove("SigningKeySize")
		$params.Add("keysize", $SigningKeySize)
	}

	try {
		$response = sendRequest2LM "geogenerateksk" $params $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		if ($response -eq "ok") {
			setKempAPIReturnObject 200 "Command successfully executed" $null
		}
		else {
			if ($response -is [array]) {
				if ($response[1] -is [System.Xml.XmlElement]) {
					$errCode = $response[1].stat
					$errStr = $response[1].Error
				}
				else {
					$errStr = $response[0]
					if ($errStr.Contains("Unauthorized")) {
						$errCode = 401
					}
					else {
						$errCode = 400
					}
				}
			}
			else {
				$errCode = 400
				$errStr = $response
			}
			setKempAPIReturnObject $errCode "$errStr" $null
		}

	}
	catch {
		$errStr = $_.Exception.Message
		setKempAPIReturnObject 400 "$errStr" $null
	}
}
Export-ModuleMember -function New-GeoDNSSECKeySigningKey

Function Remove-GeoDNSSECKeySigningKey
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$params = Convert-BoundParameters -hashtable $psboundparameters

	try {
		$response = sendRequest2LM "geodeleteksk" $params $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		if ($response -eq "ok") {
			setKempAPIReturnObject 200 "Command successfully executed" $null
		}
		else {
			if ($response -is [array]) {
				$errStr = $response[0]
			}
			else {
				$errStr = $response
			}
			if ($errStr.Contains("Unauthorized")) {
				$errCode = 401
			}
			else {
				$errCode = 400
			}
			setKempAPIReturnObject $errCode "$errStr" $null
		}

	}
	catch {
		$errStr = $_.Exception.Message
		setKempAPIReturnObject 400 "$errStr" $null
	}
}
Export-ModuleMember -function Remove-GeoDNSSECKeySigningKey

Function Get-GeoDNSSECConfiguration
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$params = Convert-BoundParameters -hashtable $psboundparameters

	try {
		$response = sendRequest2LM "geoshowdnssec" $params $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

		if ($response -is [System.Xml.XmlElement]) {
			$data = $response | Convert-XmlToPSObject
			setKempAPIReturnObject 200 "Command successfully executed" $data
		}
		else {
			if ($response -is [array]) {
				$errStr = $response[0]
			}
			else {
				$errStr = $response
			}
			if ($errStr.Contains("Unauthorized")) {
				$errCode = 401
			}
			else {
				$errCode = 400
			}
			setKempAPIReturnObject $errCode "$errStr" $null
		}

	}
	catch {
		$errStr = $_.Exception.Message
		setKempAPIReturnObject 400 "$errStr" $null
	}
}
Export-ModuleMember -function Get-GeoDNSSECConfiguration

#endregion - GEO

#region - HSM

#Function HSMShow
Function Get-TlsHSM
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "showhsm" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.HSM | Convert-XmlToPSObject
}
New-Alias -Name HSMShow -value Get-TlsHSM -Description "Alias for HSMShow command"
Export-ModuleMember -function Get-TlsHSM -Alias HSMShow

#Function HSMConfigure
Function Set-TlsHSM
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[String]$Sethsm,
		[String]$Safeaddr,
		[String]$Clpass,
		[String]$Enable,
		[String]$Cavhsmaddr,
		[String]$Cavhsmpasswd,
		[String]$Cavhsmuser,
		[String]$Cavhsmenable,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "hsmconfig" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.HSM | Convert-XmlToPSObject
}
New-Alias -Name HSMConfigure -value Set-TlsHSM -Description "Alias for HSMConfigure command"
Export-ModuleMember -function Set-TlsHSM -Alias HSMConfigure

#Function HSMUploadCACert
Function Set-TlsHSMCACert
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command hsmuploadca -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command hsmuploadca -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command hsmuploadca -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	$response
}
New-Alias -Name HSMUploadCACert -value Set-TlsHSMCACert -Description "Alias for HSMUploadCACert command"
Export-ModuleMember -function Set-TlsHSMCACert -Alias HSMUploadCACert

#Function HSMGenerateClientCert
Function New-TlsHSMClientCert
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[Parameter(Mandatory=$true)]
		[string]$Path,

		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[String]$Clcertname
	)
	$params = @{}
	if (-not ($Path))
	{
		$Path = "$($Env:SystemRoot)\Temp\LMBackup_$(Get-Date -format yyyy-MM-dd_HH-mm-ss)"
	}
	else {
		if($Path.EndsWith("\\") -or $Path.EndsWith("/"))
		{
			$Path = $Path + $Clcertname + ".pem"
		}
		else {
			$Path = $Path + "\\" + $Clcertname + ".pem"
		}
	}
	Write-Verbose "Path = $Path"

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command hsmgenclientcert -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command hsmgenclientcert -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command hsmgenclientcert -ParameterValuePair $propertyTable -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
	$response
}
New-Alias -Name HSMGenerateClientCert -value New-TlsHSMClientCert -Description "Alias for HSMGenerateClientCert command"
Export-ModuleMember -function New-TlsHSMClientCert -Alias HSMGenerateClientCert

#endregion - HSM

#region - User management

#Function UserSetSystemPassword
Function Set-SecSystemUserPassword
{	
	[CmdletBinding()]
	Param(
		[Parameter(Mandatory=$true)]
		[String]$CurrPassword,
		[Parameter(Mandatory=$true)]
		[String]$Password,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "usersetsyspassword" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name UserSetSystemPassword -value Set-SecSystemUserPassword -Description "Alias for UserSetSystemPassword command"
Export-ModuleMember -function Set-SecSystemUserPassword -Alias UserSetSystemPassword

#Function UserAddLocal
Function New-SecUser
{	
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[String]$User,
		[Parameter(Mandatory=$true)]
		[String]$Password,
		[Parameter(Mandatory=$true)]
		[String]$Radius,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "useraddlocal" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name UserAddLocal -value New-SecUser -Description "Alias for UserAddLocal command"
Export-ModuleMember -function New-SecUser -Alias UserAddLocal

#Function UserSetPermissions
Function Set-SecUserPermission
{	
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[String]$User,
		[Parameter(Mandatory=$true)]
		[String]$Perms,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "usersetperms" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name UserSetPermissions -value Set-SecUserPermission -Description "Alias for UserSetPermissions command"
Export-ModuleMember -function Set-SecUserPermission -Alias UserSetPermissions

#Function UserChangeLocalPassword
Function Set-SecUserPassword
{	
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[String]$User,
		[Parameter(Mandatory=$true)]
		[String]$Password,
		[Parameter(Mandatory=$true)]
		[String]$Radius,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "userchangelocpass" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name UserChangeLocalPassword -value Set-SecUserPassword -Description "Alias for UserChangeLocalPassword command"
Export-ModuleMember -function Set-SecUserPassword -Alias UserChangeLocalPassword

#Function UserDeleteLocal
Function Remove-SecUser
{	
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[String]$User,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "userdellocal" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name UserDeleteLocal -value Remove-SecUser -Description "Alias for UserDeleteLocal command"
Export-ModuleMember -function Remove-SecUser -Alias UserDeleteLocal

#Function UserShow
Function Get-SecSingleUser
{	
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[String]$User,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "usershow" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	if ($response.User) {
		$response.User | Convert-XmlToPSObject
	}
	else {
		$response
	}
}
New-Alias -Name UserShow -value Get-SecSingleUser -Description "Alias for UserShow command"
Export-ModuleMember -function Get-SecSingleUser -Alias UserShow

#Function UserList
Function Get-SecUser
{	
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$User,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (([String]::IsNullOrEmpty($User)))
	{
		$response = sendRequest2LM "userlist" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		$response.User | Convert-XmlToPSObject
	}
	else {
		$response = sendRequest2LM "usershow" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		if ($response.User) {
			$response.User | Convert-XmlToPSObject
		}
		else {
			$response
		}
	}
}
New-Alias -Name UserList -value Get-SecUser -Description "Alias for UserList command"
Export-ModuleMember -function Get-SecUser -Alias UserList

#Function UserGenerateCert
Function New-SecUserCertificate
{	
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[String]$User,
		[ValidateNotNullOrEmpty()]
		[String]$Passphrase,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}
	$params.Add("user", $User)

	if ($Passphrase) {
		$params.Add("passphrase", [System.Web.HttpUtility]::UrlEncode($Passphrase))
	}
	sendRequest2LM "usernewcert" $params $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name UserGenerateCert -value New-SecUserCertificate -Description "Alias for UserGenerateCert command"
Export-ModuleMember -function New-SecUserCertificate -Alias UserGenerateCert

#Function UserDownloadCert
Function Export-SecUserCertificate
{	
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[String]$User,
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[String]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}
	$params.Add("user", $User)

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		{
			Send-LBMessage -Command userdownloadcert -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			Send-LBMessage -Command userdownloadcert -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		Send-LBMessage -Command userdownloadcert -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	}
}
New-Alias -Name UserDownloadCert -value Export-SecUserCertificate -Description "Alias for UserDownloadCert command"
Export-ModuleMember -function Export-SecUserCertificate -Alias UserDownloadCert

#Function UserDeleteCert
Function Remove-SecUserCertificate
{	
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[String]$User,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "userdelcert" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name UserDeleteCert -value Remove-SecUserCertificate -Description "Alias for UserDeleteCert command"
Export-ModuleMember -function Remove-SecUserCertificate -Alias UserDeleteCert

#endregion - User management

Function AlsiLicense
{
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred,

		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true,Position=0)]
		[string]$KempId = $KempId,
		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true,Position=1)]
		[string]$Password = $Password,
		[string]$http_proxy
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	Send-LBMessage -Command alsilicense -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
}
Export-ModuleMember -function AlsiLicense

Function Request-LicenseOnline
{
	[CmdletBinding()]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred,

		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true,Position=0)]
		[string]$KempId = $KempId,

		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true,Position=1)]
		[string]$Password = $Password,

		[ValidateNotNullOrEmpty()]
		[string]$OrderId,

		[ValidateNotNullOrEmpty()]
		[string]$LicenseTypeId,

		[string]$http_proxy
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if ($OrderId) {
		$propertyTable.Remove("OrderId")
		$propertyTable.Add("orderid", $OrderId)
	}

	if ($LicenseTypeId) {
		$propertyTable.Remove("LicenseTypeId")
		$propertyTable.Add("licensetypeid", $LicenseTypeId)
	}

	try {
		$answer = Send-LBMessage -Command alsilicense2 -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		if ($answer -ne "ok") {
			if ($answer -is [array]) {
				$response = $answer[0]
			}
			else {
				$response = $answer
			}

			if ($response.Contains("Unable to connect")) {
				setKempAPIReturnObject 401 "Unable to connect to the remote server." $null
				return
			}
			elseif ($response.Contains("Unauthorized")) {
				setKempAPIReturnObject 401 "Unauthorized." $null
				return
			}
			else {
				setKempAPIReturnObject 401 "Unknown error." $null
				return
			}
		}

		$tempApiRetObj = [ordered]@{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "Command successfully executed"
		$tempApiRetObj.Data = $null

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	catch {
		$errorString = $_.Exception.Message
		if ($errorString.Contains("invalid username")) {
			$errCode= 422
		}
		else {
			$errCode= 401
		}
		setKempAPIReturnObject $errCode "$errorString" $null
	}
}
Export-ModuleMember -function Request-LicenseOnline

Function Update-LicenseOnline
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true,Position=0)]
		[string]$KempId = $KempId,

		[ValidateNotNullOrEmpty()]
		[Parameter(Mandatory=$true,Position=1)]

		[ValidateNotNullOrEmpty()]
		[string]$Password = $Password,

		[ValidateNotNullOrEmpty()]
		[string]$OrderId,

		#[ValidateNotNullOrEmpty()]
		#[string]$LicenseTypeId,

		[string]$http_proxy
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if ($OrderId) {
		$propertyTable.Remove("OrderId")
		$propertyTable.Add("orderid", $OrderId)
	}

	try {
		$answer = sendRequest2LM "alsilicense2" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		if ($answer -ne "ok") {
			if ($answer -is [array]) {
				$response = $answer[0]
			}
			else {
				$response = $answer
			}

			if ($response.Contains("Unable to connect")) {
				setKempAPIReturnObject 401 "Unable to connect to the remote server." $null
				return
			}
			elseif ($response.Contains("Unauthorized")) {
				setKempAPIReturnObject 401 "Unauthorized." $null
				return
			}
			else {
				setKempAPIReturnObject 401 "Unknown error." $null
				return
			}
		}

		$tempApiRetObj = [ordered]@{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "Command successfully executed"
		$tempApiRetObj.Data = $null

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	catch {
		$errorString = $_.Exception.Message
		if ($errorString.Contains("invalid username")) {
			$errCode= 422
		}
		else {
			$errCode= 401
		}
		setKempAPIReturnObject $errCode "$errorString" $null
	}
}
Export-ModuleMember -function Update-LicenseOnline

#region - Addons

#Function UploadAddon
Function Install-LmAddon
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$params = @{}

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if (-not ([String]::IsNullOrEmpty($SubjectCN)))
 	{
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
	 	{
			$response = Send-LBMessage -Command addaddon -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
		}
		else {
			$response = Send-LBMessage -Command addaddon -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
		}
	}
	else {
		$response = Send-LBMessage -Command addaddon -ParameterValuePair $propertyTable -File $Path -LoadBalancer $LoadBalancer -Credential $Credential -LBPort $LBPort
	}
	$response
}
New-Alias -Name UploadAddon -value Install-LmAddon -Description "Alias for UploadAddon command"
Export-ModuleMember -function Install-LmAddon -Alias UploadAddon

#Function DeleteAddon
Function Remove-LmAddon
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "deladdon" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name DeleteAddon -value Remove-LmAddon -Description "Alias for DeleteAddon command"
Export-ModuleMember -function Remove-LmAddon -Alias DeleteAddon

#Function ListAddons
Function Get-LmAddOn
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "listaddon" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.Packages.Package | Convert-XmlToPSObject
}
New-Alias -Name ListAddons -value Get-LmAddOn -Description "Alias for ListAddons command"
Export-ModuleMember -function Get-LmAddOn -Alias ListAddons

#endregion - Addons

#region - IPsec

#Function CreateVpnConnection
Function New-VpnConnection
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "createvpncon" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name CreateVpnConnection -value New-VpnConnection -Description "Alias for CreateVpnConnection command"
Export-ModuleMember -function New-VpnConnection -Alias CreateVpnConnection

#Function DeleteVpnConnection
Function Remove-VpnConnection
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "deletevpncon" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name DeleteVpnConnection -value Remove-VpnConnection -Description "Alias for DeleteVpnConnection command"
Export-ModuleMember -function Remove-VpnConnection -Alias DeleteVpnConnection

#Function SetVpnAddrs
Function Set-VpnAddrs
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$LocalIp,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$LocalSubnets,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$RemoteIp,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$RemoteSubnets,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setvpnaddr" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name SetVpnAddrs -value Set-VpnAddrs -Description "Alias for SetVpnAddrs command"
Export-ModuleMember -function Set-VpnAddrs -Alias SetVpnAddrs

#Function SetVpnLocalIp
Function Set-VpnLocalIp
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$LocalIp,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setvpnlocalip" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name SetVpnLocalIp -value Set-VpnLocalIp -Description "Alias for SetVpnLocalIp command"
Export-ModuleMember -function Set-VpnLocalIp -Alias SetVpnLocalIp

#Function SetVpnLocalSubnets
Function Set-VpnLocalSubnet
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$LocalSubnets,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setvpnlocalsubnets" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name SetVpnLocalSubnets -value Set-VpnLocalSubnet -Description "Alias for SetVpnLocalSubnets command"
Export-ModuleMember -function Set-VpnLocalSubnet -Alias SetVpnLocalSubnets

#Function SetVpnRemoteIp
Function Set-VpnRemoteIp
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$RemoteIp,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setvpnremoteip" $propertyTable $LoadBalancer $LmPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name SetVpnRemoteIp -value Set-VpnRemoteIp -Description "Alias for SetVpnRemoteIp command"
Export-ModuleMember -function Set-VpnRemoteIp -Alias SetVpnRemoteIp

#Function SetVpnRemoteSubnets
Function Set-VpnRemoteSubnet
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$RemoteSubnets,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setvpnremotesubnets" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name SetVpnRemoteSubnets -value Set-VpnRemoteSubnet -Description "Alias for SetVpnRemoteSubnets command"
Export-ModuleMember -function Set-VpnRemoteSubnet -Alias SetVpnRemoteSubnets

#Function SetVpnSecret
Function Set-VpnSecret
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$LocalId,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$RemoteId,
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Key,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setvpnsecret" $propertyTable $LoadBalancer $LbPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name SetVpnSecret -value Set-VpnSecret -Description "Alias for SetVpnSecret command"
Export-ModuleMember -function Set-VpnSecret -Alias SetVpnSecret

#Function StartVpnConnection
Function Start-VpnConnection
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "startvpncon" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name StartVpnConnection -value Start-VpnConnection -Description "Alias for StartVpnConnection command"
Export-ModuleMember -function Start-VpnConnection -Alias StartVpnConnection

#Function StopVpnConnection
Function Stop-VpnConnection
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "stopvpncon" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name StopVpnConnection -value Stop-VpnConnection -Description "Alias for StopVpnConnection command"
Export-ModuleMember -function Stop-VpnConnection -Alias StopVpnConnection

#Function StartIkeDaemon
Function Start-VpnIkeDaemon
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "startikedaemon" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name StartIkeDaemon -value Start-VpnIkeDaemon -Description "Alias for StartIkeDaemon command"
Export-ModuleMember -function Start-VpnIkeDaemon -Alias StartIkeDaemon

#Function StopIkeDaemon
Function Stop-VpnIkeDaemon
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "stopikedaemon" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name StopIkeDaemon -value Stop-VpnIkeDaemon -Description "Alias for StopIkeDaemon command"
Export-ModuleMember -function Stop-VpnIkeDaemon -Alias StopIkeDaemon

#Function StatusIkeDaemon
Function Get-VpnIkeDaemonStatus
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "statusikedaemon" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name StatusIkeDaemon -value Get-VpnIkeDaemonStatus -Description "Alias for StatusIkeDaemon command"
Export-ModuleMember -function Get-VpnIkeDaemonStatus -Alias StatusIkeDaemon

#Function ListVpns
Function Get-VpnConnection
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	if ($name -eq "")
	{
		$response = sendRequest2LM "listvpns" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		$response.VPN | Convert-XmlToPSObject
	}
	else {
		sendRequest2LM "getvpnstatus" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
}
New-Alias -Name ListVpns -value Get-VpnConnection -Description "Alias for ListVpns command"
Export-ModuleMember -function Get-VpnConnection -Alias ListVpns

# LEGACY: see the above command Get-VpnConnection
Function GetVpnStatus
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "getvpnstatus" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
Export-ModuleMember -function GetVpnStatus

#Function SetVpnPfsEnable
Function Set-VpnPfsEnable
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setvpnpfsenable" $propertyTable $LoadBalancer $BmPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name SetVpnPfsEnable -value Set-VpnPfsEnable -Description "Alias for SetVpnPfsEnable command"
Export-ModuleMember -function Set-VpnPfsEnable -Alias SetVpnPfsEnable

#Function SetVpnPfsDisable
Function Set-VpnPfsDisable
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	sendRequest2LM "setvpnpfsdisable" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
}
New-Alias -Name SetVpnPfsDisable -value Set-VpnPfsDisable -Description "Alias for SetVpnPfsDisable command"
Export-ModuleMember -function Set-VpnPfsDisable -Alias SetVpnPfsDisable

#endregion - IPsec

#region - Clusters

#Function NMClusterStatus
Function Get-ClusterStatus
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "cluster/status" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.childnodes | Convert-XmlToPSObject
}
New-Alias -Name NMClusterStatus -value Get-ClusterStatus -Description "Alias for NMClusterStatus command"
Export-ModuleMember -function Get-ClusterStatus -Alias NMClusterStatus

#Function NMClusterCreate
Function New-Cluster
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$SharedAddress,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "cluster/create" $propertyTable $LoadBalancer $LmPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name NMClusterCreate -value New-Cluster -Description "Alias for NMClusterCreate command"
Export-ModuleMember -function New-Cluster -Alias NMClusterCreate

#Function NMAddNode
Function Add-ClusterNode
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Address,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "cluster/addnode" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name NMAddNode -value Add-ClusterNode -Description "Alias for NMAddNode command"
Export-ModuleMember -function Add-ClusterNode -Alias NMAddNode

#Function NMJoinCluster
Function Join-Cluster
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "cluster/joincluster" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name NMJoinCluster -value Join-Cluster -Description "Alias for NMJoinCluster command"
Export-ModuleMember -function Join-Cluster -Alias NMJoinCluster

#Function NMEnableNode
Function Enable-ClusterNode
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[Int32]$NodeId,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "cluster/enablenode" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name NMEnableNode -value Enable-ClusterNode -Description "Alias for NMEnableNode command"
Export-ModuleMember -function Enable-ClusterNode -Alias NMEnableNode

#Function NMDisableNode
Function Disable-ClusterNode
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[Int32]$NodeId,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "cluster/disablenode" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name NMDisableNode -value Disable-ClusterNode -Description "Alias for NMDisableNode command"
Export-ModuleMember -function Disable-ClusterNode -Alias NMDisableNode

#Function NMDeleteNode
Function Remove-ClusterNode
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[Int32]$NodeId,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "cluster/deletenode" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response
}
New-Alias -Name NMDeleteNode -value Remove-ClusterNode -Description "Alias for NMDeleteNode command"
Export-ModuleMember -function Remove-ClusterNode -Alias NMDeleteNode

#endregion - Clusters

#region - EULA

Function ReadEULA
{
	[CmdletBinding()]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	$response = Send-LBMessage -Command readeula -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	$response
}
Export-ModuleMember -function ReadEULA

Function Read-LicenseEULA
{
	[CmdletBinding()]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	try {
		$response =	Send-LBMessage -Command readeula -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential

		$mlidx = $response.IndexOf("`<Magic`>") + 7
		$muidx = $response.IndexOf("`<`/Magic`>")
		$mstr  = $response.SubString($mlidx, $muidx - $mlidx)

		$elidx = $response.IndexOf("`<Eula`>") + 6
		$euidx = $response.IndexOf("`<`/Eula`>")
		$eula  = $response.SubString($elidx, $euidx - $elidx)

		$tempEulaAnswer = @{}
		$tempEulaAnswer.PSTypeName = "Eula"
		$tempEulaAnswer.Eula = $eula
		$tempEulaAnswer.MagicString = $mstr
		$eulaAnswer = New-Object -TypeName PSObject -Prop $tempEulaAnswer

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "Command successfully executed"
		$tempApiRetObj.Data = $eulaAnswer

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	catch {
		Write-Output "Caught an exception" -ForegroundColor Red
		Write-Output "Exception Type: $($_.Exception.GetType().FullName)" -ForegroundColor Red
		Write-Output "Exception Message: $($_.Exception.Message)" -ForegroundColor Red
	}
}
Export-ModuleMember -function Read-LicenseEULA

Function AcceptEULA
{
	[CmdletBinding()]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Type,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$Magic,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	$response =	Send-LBMessage -Command accepteula -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	$response
}
Export-ModuleMember -function AcceptEULA

Function Confirm-LicenseEULA
{
	[CmdletBinding()]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Type,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$Magic,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,
		
		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	try {
		$response =	Send-LBMessage -Command accepteula -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential

		$mlidx = $response.IndexOf("`<Magic`>") + 7
		$muidx = $response.IndexOf("`<`/Magic`>")
		$mstr  = $response.SubString($mlidx, $muidx - $mlidx)

		$elidx = $response.IndexOf("`<Eula`>") + 6
		$euidx = $response.IndexOf("`<`/Eula`>")
		$eula2 = $response.SubString($elidx, $euidx - $elidx)

		$tempEula2Answer = @{}
		$tempEula2Answer.PSTypeName = "Eula2"
		$tempEula2Answer.Eula2 = $eula2
		$tempEula2Answer.MagicString = $mstr
		$eulaAnswer2 = New-Object -TypeName PSObject -Prop $tempEula2Answer

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "Command successfully executed"
		$tempApiRetObj.Data = $eulaAnswer2

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	catch {
		Write-Output "Caught an exception"
		Write-Output "Exception Type: $($_.Exception.GetType().FullName)"
		write-Output "Exception Message: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Confirm-LicenseEULA

Function AcceptEULA2
{
	[CmdletBinding()]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Magic,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$Accept,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,
		
		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	$response =	Send-LBMessage -Command accepteula2 -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	$response
}
Export-ModuleMember -function AcceptEULA2

Function Confirm-LicenseEULA2
{
	[CmdletBinding()]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Magic,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$Accept,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	try {
		$response =	Send-LBMessage -Command accepteula2 -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "Command successfully executed"
		$tempApiRetObj.Data = $null

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	catch {
		Write-Output "Caught an exception" -ForegroundColor Red
		Write-Output "Exception Type: $($_.Exception.GetType().FullName)" -ForegroundColor Red
		Write-Output "Exception Message: $($_.Exception.Message)" -ForegroundColor Red
	}
}
Export-ModuleMember -function Confirm-LicenseEULA2

Function SetInitialPasswd
{
	[CmdletBinding()]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Passwd,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response =	Send-LBMessage -Command set_initial_passwd -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
	$response
}
Export-ModuleMember -function SetInitialPasswd

Function Set-LicenseInitialPassword
{
	[CmdletBinding()]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Passwd,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	try {
		$response =	Send-LBMessage -Command set_initial_passwd2 -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "Command successfully executed"
		$tempApiRetObj.Data = $null

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	catch {
		Write-Output "Caught an exception" -ForegroundColor Red
		Write-Output "Exception Type: $($_.Exception.GetType().FullName)" -ForegroundColor Red
		Write-Output "Exception Message: $($_.Exception.Message)" -ForegroundColor Red
	}
}
Export-ModuleMember -function Set-LicenseInitialPassword

#endregion - EULA

#region - SDN

#Function AddSDNController
Function Add-SdnController
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$IPV4,
		[Parameter(Mandatory=$true,Position=1)]
		[ValidateNotNullOrEmpty()]
		[Int32]$Port,
		[Int32]$Clid,
		[Boolean]$Https,
		[String]$User,
		[String]$Password,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "addsdncontroller" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.childnodes.childnodes.childnodes | Convert-XmlToPSObject
}
New-Alias -Name AddSDNController -value Add-SdnController -Description "Alias for AddSDNController command"
Export-ModuleMember -function Add-SdnController -Alias AddSDNController

#Function DeleteSDNController
Function Remove-SdnController
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Int32]$Clid,
		[Int32]$Cid,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "delsdncontroller" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response | Convert-XmlToPSObject
}
New-Alias -Name DeleteSDNController -value Remove-SdnController -Description "Alias for DeleteSDNController command"
Export-ModuleMember -function Remove-SdnController -Alias DeleteSDNController

#Function ModifySDNController
Function Set-SdnController
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[Int32]$Cid,
		[Int32]$Clid,
		[String]$IPV4,
		[Int32]$Port,
		[Boolean]$Https,
		[String]$User,
		[String]$Password,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "modsdncontroller" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.childnodes.childnodes.childnodes | Convert-XmlToPSObject
}
New-Alias -Name ModifySDNController -value Set-SdnController -Description "Alias for ModifySDNController command"
Export-ModuleMember -function Set-SdnController -Alias ModifySDNController

#Function GetSDNController
Function Get-SdnController
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "getsdncontroller" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	$response.childnodes.childnodes.childnodes | Convert-XmlToPSObject
}
New-Alias -Name GetSDNController -value Get-SdnController -Description "Alias for GetSDNController command"
Export-ModuleMember -function Get-SdnController -Alias GetSDNController

#endregion - SDN

#region - Logging and Debug

#Function FlushSsoCache
Function Clear-SSOCache
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$response = sendRequest2LM "logging/ssoflush" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation

	if ($response.code -eq "fail")
	{
		Write-Error $response.Error
	}
	else {
		$response
	}
}
New-Alias -Name FlushSsoCache -value Clear-SSOCache -Description "Alias for FlushSsoCache command"
Export-ModuleMember -function Clear-SSOCache -Alias FlushSsoCache

#endregion

#Function DoTcpDump
Function Trace-TcpTraffic
{
	[cmdletbinding(SupportsShouldProcess=$true, DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[ValidateNotNullOrEmpty()]
		[int]$MaxPackets,		
		[ValidateNotNullOrEmpty()]
		[int]$MaxTime,
		[ValidateNotNullOrEmpty()]
		[string]$Interface,
		[ValidateNotNullOrEmpty()]
		[string]$Port,
		[ValidateNotNullOrEmpty()]
		[string]$Address,
		[ValidateNotNullOrEmpty()]
		[string]$TcpOptions,

		[string]$Path,

		[switch]$Force
	)

	PROCESS
	{
		$params = @{}
		if($MaxPackets){
			$params.Add("maxpackets", $MaxPackets)
		}
		if($MaxTime){
			$params.Add("maxtime", $MaxTime)
		}
		if($Interface){
			$params.Add("interface", $Interface)
		}
		if($Port){
			$params.Add("port", $Port)
		}
		if($Address){
			$params.Add("address", $Address)
		}
		if($TcpOptions){
			$params.Add("tcpoptions", $TcpOptions)
		}

		if (-not ($Path))
		{
			$defaultFolder = "$($Env:SystemRoot)\Temp"
			if (Test-Path -Path $defaultFolder) {
				$Path = "$defaultFolder\tcpdump_$(Get-Date -format yyyy-MM-dd_HH-mm-ss).pcap"
			}
			else {
				Throw "ERROR: the default folder $($Env:SystemRoot)\Temp does not exist"
			}
		}
		else {
			$folder = Split-Path -Path $path
			if (-not (Test-Path -Path $folder)) {
				Throw "ERROR: the folder $folder does not exist"
			}
			Write-Verbose "folder [$folder]"
		}
		Write-Verbose "File = $Path"

		try {
			if (($Force) -or ($PsCmdlet.ShouldProcess($Path, "Output File Overwrite")))
			{
				if (-not ([String]::IsNullOrEmpty($SubjectCN)))
				{
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
					{
						$response = Send-LBMessage -command tcpdump -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
					}
					else {
						$response = Send-LBMessage -command tcpdump -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
					}
				}
				else {
					$response = Send-LBMessage -command tcpdump -ParameterValuePair $params -File $Path -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
				}
			}
		}
		catch {
			$response = $_.Exception.Message
		}
		Write-Verbose "response = $response"
		if ($response -eq "OK") {

			$tmpObj = [ordered]@{}
			$tmpObj.PSTypeName = "TcpDumpOutputFile"
			$tmpObj.OutputFile = $Path
			$TcpDumpObject = New-Object -TypeName PSObject -Prop $tmpObj

			$answer = setKempAPIReturnObject 200 "cmd successfully executed" $TcpDumpObject
		}
		else {
			if ($response.Contains("Unauthorized")) {
				$answer = setKempAPIReturnObject 401 $response $null
			}
			else {
				$answer = setKempAPIReturnObject 422 $response $null
			}
		}
		$answer
	}
}
New-Alias -Name DoTcpDump -value Trace-TcpTraffic -Description "Alias for DoTcpDump command"
Export-ModuleMember -function Trace-TcpTraffic -Alias DoTcpDump

#endregion

# region ASL

Function get_stringbyname([String]$strToSearch, [String]$inputStr)
{
	$tmp = $inputStr
	$len = $strToSearch.Length

	$idx = $tmp.IndexOf($strToSearch) + 1
	if ($idx -eq 0)
	{
		$idx = $len
	}
	else {
		if ($idx -gt 0)
		{
			$tmp = $tmp.SubString($idx)
			$idx = $len
		}
		else {
			throw "Internal error."
		}
	}
	$icomma = $tmp.IndexOf(";")
	if ($icomma -gt 0) {
		$response = $tmp.SubString($idx, $icomma - $idx)
	}
	else {
		$icomma = $tmp.IndexOf("=") + 1
		$response = $tmp.SubString($icomma)
	}
	$response
}

Function Get-AslLicenseType
{
	[CmdletBinding()]
	Param(
		[ValidateNotNullOrEmpty()]
		[String]$aslipaddr,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(1, 65535)]
		[int]$aslport,

		[ValidateNotNullOrEmpty()]
		[String]$aslname,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3, 65530)]
		[int]$LBPort = $LBAccessPort
	)

	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	try {
		# need credentials
		$pw = ConvertTo-SecureString "xxxx" -AsPlainText -Force
		$Credential = New-Object System.Management.Automation.PSCredential("user", $pw)
		$response = sendRequest2LM "aslgetlicensetypes" $propertyTable $LoadBalancer $LBPort $Credential
	}
	catch {
		$response = $_.Exception.Message
	}
	if ($response) {
		if ($response.Response.stat -eq 200 -or $response.Response.stat -eq "ok") {
			$data = $response.Response.Success
			if ($data.Contains("License type information not available")) {
				setKempAPIReturnObject 401 "ERROR: License type information not available. Please check your kempid and/or password." $null
				return
			}
			if ($data -ne "[]") {
				$TempLicObject = $data | ConvertFrom-Json
				$LicObject = [ordered]@{}
				$LicObject.PSTypeName = "LicenseData"
				$licNumber = $TempLicObject.categories.licenseTypes.Length
				if (([String]::IsNullOrEmpty($licNumber))) {
					$licNumber = 1
				}
				$LicObject.Add("OrderID", $OrderId)
				$LicObject.Add("AvailableLicenses", $licNumber)
				$LicObject.Add("Licenses", $TempLicObject.categories.licenseTypes)
				$licData = New-Object -TypeName PSObject -Prop $LicObject
				setKempAPIReturnObject 200 "Command successfully executed." $licData
			}
			else {
				$emptyLicObject = [ordered]@{}
				$emptyLicObject.PSTypeName = "LicenseData"
				$emptyLicObject.Add("OrderID", $OrderId)
				$emptyLicObject.Add("AvailableLicenses", 0)
				$emptyLicObject.Add("Licenses", $null)
				$licData = New-Object -TypeName PSObject -Prop $emptyLicObject
				setKempAPIReturnObject 200 "Command successfully executed." $licData
			}
		}
		else {
			if ($response.Response) {
				$errorString = $response.Response
				setKempAPIReturnObject 401 "$errorString" $null
			}
			else {
				if ($response -is [array]) {
					$tmp = $response[1].Error
					if (-not ([String]::IsNullOrEmpty($tmp))) {
						$errorCode = $response[1].stat
						$errorString = $tmp
					}
					else {
						$errorCode = 401
						$errorString = $response[0]
					}
				}
				else {
					$errorCode = 401
					$errorString = $response
				}
				setKempAPIReturnObject $errorCode "$errorString" $null
			}
		}
	}
	else {
		setKempAPIReturnObject 401 "ERROR the answer is NULL" $null
	}
}
Export-ModuleMember -function Get-AslLicenseType

Function Request-LicenseOnPremise
{
	[CmdletBinding()]
	Param(
		[Parameter(Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$aslipaddr,
		[Parameter(Position=1)]
		[ValidateNotNullOrEmpty()]
		[String]$aslname,
		[Parameter(Position=2)]
		[ValidateNotNullOrEmpty()]
		[ValidateRange(1,65535)]
		[int]$aslport,
		[ValidateNotNullOrEmpty()]
		[string]$LicenseTypeId,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[ValidateNotNullOrEmpty()]
		[System.Management.Automation.Credential()]$Credential = $script:cred
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters
	$propertyTable.Remove("Credential")

	if ($LicenseTypeId) {
		$propertyTable.Remove("LicenseTypeId")
		$propertyTable.Add("lic_type_id", $LicenseTypeId)
	}

	try {
		$response = Send-LBMessage -Command aslactivate -ParameterValuePair $propertyTable -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential

		[string]$str_response = $response | Convert-XmlToPSObject
		Write-Verbose "str_response [$str_response]"

		$eqp = $str_response.IndexOf("{")
		$lbp = $str_response.IndexOf("}")
		$str_response = $str_response.substring($eqp + 1, $lbp - $eqp - 1)
		Write-Verbose "str_response [$str_response]"

		$ipStr   = get_stringbyname "aslipaddr" $str_response
		$portStr = get_stringbyname "aslport" $str_response
		$nameStr = get_stringbyname "aslname" $str_response

		$tempData = @{}
		$tempData.PSTypeName = "Asl"
		$tempData.AslIpAddress = $ipStr
		$tempData.AslName = $nameStr
		$tempData.AslPort = $portStr

		$AslData = New-Object -TypeName PSObject -Prop $tempData

		$tempApiRetObj = @{}
		$tempApiRetObj.PSTypeName = "KempAPI"
		$tempApiRetObj.ReturnCode = 200
		$tempApiRetObj.Response = "Command successfully executed"
		$tempApiRetObj.Data = $AslData

		$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
		$apiRetObject
	}
	catch {
		Throw "ERROR $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Request-LicenseOnPremise

Function Stop-AslInstance
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$ErrorActionPreference = "Stop"
	try{
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
	 	{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
		 	{
				$response = Send-LBMessage -Command killaslinstance -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$response = Send-LBMessage -Command killaslinstance -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$response = Send-LBMessage -Command killaslinstance -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}

		if ($response -eq "kill_asl_instance: success.")
		{
			$tempApiRetObj = @{}
			$tempApiRetObj.PSTypeName = "KempAPI"
			$tempApiRetObj.ReturnCode = 200
			$tempApiRetObj.Response = "Command successfully executed"
			$tempApiRetObj.Data = $null

			$apiRetObject = New-Object -TypeName PSObject -Prop $tempApiRetObj
			$apiRetObject
		}
		else {
			Throw "ERROR: command failed."
		}
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Stop-AslInstance

# end ASL region

#endregion Originating script: 'c:\devhome\powershell\KempTechPowershellModule\Kemp.LoadBalancer.Powershell.ps1'

#region - LDAP endpoint

Function Get-LdapEndpoint
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	try {
		if ($Name -eq "")
		{
			$response = sendRequest2LM "showldaplist" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
			$response.Endpoint | Convert-XmlToPSObject
		} else {
			sendRequest2LM "showldapendpoint" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		}
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Get-LdapEndpoint

Function Add-LdapEndpoint
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,
		[String]$Server,
		[ValidateRange(0,2)]
		[Int16]$LdapType,
		[ValidateRange(10,86400)]
		[Int16]$VInterval,
		[String]$AdminUser,
		[String]$AdminPass,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	try {
		sendRequest2LM "addldapendpoint" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		setKempAPIReturnObject 200 "Command successfully executed" $Name
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Add-LdapEndpoint

Function Set-LdapEndpoint
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,
		[String]$Server,
		[ValidateRange(0,2)]
		[Int16]$LdapType,
		[ValidateRange(10,86400)]
		[Int16]$VInterval,
		[String]$AdminUser,
		[String]$AdminPass,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	try {
		sendRequest2LM "modifyldapendpoint" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		setKempAPIReturnObject 200 "Command successfully executed" $Name
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Set-LdapEndpoint

Function Remove-LdapEndpoint
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true,Position=0)]
		[ValidateNotNullOrEmpty()]
		[String]$Name,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	try {
		sendRequest2LM "deleteldapendpoint" $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
		setKempAPIReturnObject 200 "Command successfully executed" $Name
	}
	catch {
		Throw "ERROR: $($_.Exception.Message)"
	}
}
Export-ModuleMember -function Remove-LdapEndpoint

#endregion

#region - diagnostic

Function CheckCmdletLoginParameters($Credential, $SubjectCN, $CertificateStoreLocation)
{
	if (-not ([String]::IsNullOrEmpty($Credential))) {
		Write-Verbose "Credential case: [$($Credential.username), ********]"
		$plen  = $Credential.Password.Length
		if ($plen -gt 0) {
			Write-Verbose "Password OK"
		}
		else {
			Throw "ERROR: NULL Password"
		}
	}
	elseif (-not ([String]::IsNullOrEmpty($SubjectCN))) {
		if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation))) {
			Write-Verbose "SubjectCN + Certificate store location case [$SubjectCN, $CertificateStoreLocation]"
		}
		else {
			Write-Verbose "SubjectCN case [$SubjectCN]"
		}
	}
	else {
		Throw "ERROR: missing login parameters Credential/Certificate."
	}
}

Function CheckPingInputParameters($Address2Ping, $PingClass, $Interface, $propsTable)
{
	if (([String]::IsNullOrEmpty($Address2Ping))) {
		Throw "ERROR: [Address to ping is NULL]"
	}
	$propsTable.Remove("Address")
	$propsTable.Add("addr", $Address)

	if (([String]::IsNullOrEmpty($PingClass))) {
		Throw "ERROR: [Ping class is NULL]"
	}

	if ($PingClass -ne "ping" -and $PingClass -ne "ping6") {
		Throw "ERROR: [Not allowed Ping Class ($PingClass)]"
	}
	if ($PingClass -ne "") {
		$propsTable.Remove("PingClass")
	}

	if ($Interface) {
		$propsTable.Remove("Interface")
		$propsTable.Add("intf", $Interface)
	}

	$propsTable
}

Function setPingOKAnswer($rawOKAnswer)
{
	$StatsPos1 = $rawOKAnswer.IndexOf("ping statistics") + "ping statistics --- ".Length
	$StatsPos2 = $rawOKAnswer.IndexOf("received") + "received".Length
	$trPcksString = $rawOKAnswer.Substring($StatsPos1, $StatsPos2 - $StatsPos1)
	$cPos = $trPcksString.IndexOf(",")
	$tPackets = $trPcksString.Substring(0, $cPos)
	$rPackets = $trPcksString.Substring($cPos + 2)

	$PLPos = $rawOKAnswer.IndexOf("packet loss") + "packet loss".Length
	$lossPackets = $rawOKAnswer.Substring($StatsPos2 + 2, ($PLPos - $StatsPos2 - 2))

	$timePos = $rawOKAnswer.IndexOf("time ")
	$rttPos = $rawOKAnswer.IndexOf("rtt min/avg/max/mdev")
	$timeStr = $rawOKAnswer.Substring($timePos, ($rttPos - $timePos))
	$rtt = $rawOKAnswer.Substring($rttPos)

	$endPingResult = $rawOKAnswer.IndexOf(" --- ")
	$PingResult = $rawOKAnswer.Substring(0, $endPingResult)

	$pingData = [ordered]@{}

	$pingData.Add("PacketsTransmitted", $tPackets)
	$pingData.Add("PacketsReceived", $rPackets)
	$pingData.Add("PacketsLoss", $lossPackets)
	$pingData.Add("Time", $timeStr)
	$pingData.Add("Rtt", $rtt)
	$pingData.Add("PingResult", $PingResult)

	$PingObject = New-Object -TypeName PSObject -Prop $pingData

	$kempAPI = setKempAPIReturnObject 200 "Ping successfully executed" $PingObject

	$kempAPI
}

Function setPingNOTOKAnswer($rawNOTOKAnswer)
{
	$StatsPos1 = $rawNOTOKAnswer.IndexOf("ping statistics") + "ping statistics --- ".Length
	$StatsPos2 = $rawNOTOKAnswer.IndexOf("received") + "received".Length
	$trPcksString = $rawNOTOKAnswer.Substring($StatsPos1, $StatsPos2 - $StatsPos1)
	$cPos = $trPcksString.IndexOf(",")
	$tPackets = $trPcksString.Substring(0, $cPos)
	$rPackets = $trPcksString.Substring($cPos + 2)

	$errStringCheck = $rawNOTOKAnswer.IndexOf("errors")
	if ($errStringCheck -gt 0) {
		$ErrorPos = $rawNOTOKAnswer.IndexOf("errors") + "errors,".Length
		$Errors = $rawNOTOKAnswer.Substring($StatsPos2 + 2, ($ErrorPos - $StatsPos2 - 3))
	}
	else {
		$Errors = ""
	}

	$PLPos = $rawNOTOKAnswer.IndexOf("packet loss") + "packet loss".Length
	if ($errStringCheck -gt 0) {
		$lossPackets = $rawNOTOKAnswer.Substring($ErrorPos + 1, ($PLPos - $ErrorPos - 1))
	}
	else {
		$lossPackets = $rawNOTOKAnswer.Substring($StatsPos2 + 2, ($PLPos - $StatsPos2 - 2))
	}

	$endPingResult = $rawNOTOKAnswer.IndexOf(" --- ")
	$PingResult = $rawNOTOKAnswer.Substring(0, $endPingResult)

	$pingData = [ordered]@{}

	$pingData.Add("PacketsTransmitted", $tPackets)
	$pingData.Add("PacketsReceived", $rPackets)
	$pingData.Add("Errors", $Errors)
	$pingData.Add("PacketsLoss", $lossPackets)
	$pingData.Add("PingResult", $PingResult)

	$PingObject = New-Object -TypeName PSObject -Prop $pingData

	$kempAPI = setKempAPIReturnObject 422 "Ping failed" $PingObject

	$kempAPI
}

Function SetErrorAnswer($ErrorDescription, $code)
{
	$PingObject = $null

	if ($code) {
		$kempAPI = setKempAPIReturnObject $code $ErrorDescription $PingObject
	}
	else {
		$kempAPI = setKempAPIReturnObject 422 $ErrorDescription $PingObject
	}

	$kempAPI
}

Function setPingAnswer($RawAnswer)
{
	if (-not ([String]::IsNullOrEmpty($RawAnswer))) {

		$answer = [string]$RawAnswer

		if ($answer.Contains("Response Command Failed")) {
			Write-Verbose "[setPingAnswer] Command Failed case [$answer]"
			setPingNOTOKAnswer $answer
		}
		else {
			Write-Verbose "[setPingAnswer] Command Success case [$answer]"
			setPingOKAnswer $answer
		}

	}
	else {
		Throw "ERROR: Ping Answer is NULL"
	}
}

Function checkDiagnosticCmdletsErrors($answer)
{
	$errString = ""

	if (-not ([String]::IsNullOrEmpty($answer)))
	{
		if (-not ($answer -is [System.Xml.XmlElement])) {
			$answer = $answer | out-string
		}
		else {
			$at = $answer.GetType()
			Write-Verbose "answer type [$at]"
		}

		if ($answer -is [String]) {

			if ($answer.Contains("Unauthorized")) {
				$errString = SetErrorAnswer $answer 401
			}

			if ($answer.Contains("Invalid address format")) {
				$errString = SetErrorAnswer "Invalid address format"
			}

			if ($answer.Contains("Unable to connect")) {
				$errString = SetErrorAnswer $answer
			}

			if ($answer.Contains("Can't find certificate")) {
				$errString = SetErrorAnswer $answer
			}

			if ($answer.Contains("unknown host")) {
				$errString = SetErrorAnswer $answer
			}

			if ($answer.Contains("protocol violation")) {
				$errString = SetErrorAnswer $answer
			}

			if ($answer.Contains("Invalid argument")) {

				if ($answer.Contains("#")) {
					# Traceroute
					$pos = $answer.IndexOf("#")
					$tmp = $answer.Substring($pos + 1)
					$pos = $tmp.IndexOf("#")
					$tr  = $tmp.Substring(0, $pos)
					$trErr = $tmp.Substring($pos + 1)
					$trErr = $trErr -replace '#', ''

					$errString = setKempAPIReturnObject 422 "Traceroute failure: $trErr" $null
				}
				else {
					$errString = SetErrorAnswer $answer
				}
			}
		}
	}
	$errString
}

Function Ping-Host
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[String]$Address,

		[ValidateSet("ping","ping6")]
		[string]$PingClass = "ping",

		[string]$Interface = $null,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	CheckCmdletLoginParameters $Credential $SubjectCN $CertificateStoreLocation

	$params = CheckPingInputParameters $Address $PingClass $Interface $propertyTable

	$cmd = "logging/$PingClass"

	try {
		$response = sendRequest2LM $cmd $params $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$response = $_.Exception.Message
	}

	$check = checkDiagnosticCmdletsErrors $response
	if (([String]::IsNullOrEmpty($check))) {
		setPingAnswer $response
	}
	else {
		$check
	}
}
Export-ModuleMember -function Ping-Host

Function isInputNull($param)
{
	if ($param) {
		return $false
	}
	else {
		return $true
	}
}

Function SetTracerouteParam($params)
{
	if (-not (isInputNull $params)) {
		if ($params.ContainsKey("Address")) {
			$kvalue = $params.Get_Item("Address")
			$params.Remove("Address")
			$params.Add("addr", $kvalue)
			$true
		}
		else {
			Write-Verbose "key not found"
			$false
		}
	}
	else {
		Throw "ERROR: params table is NULL"
	}
}

Function SetTracerouteErrorAnswer($ReturnCode, $Response, $Data)
{
	setKempAPIReturnObject $ReturnCode $Response $Data
}

Function SetTracerouteOkAnswer($ReturnCode, $Response, $Data)
{
	$TrTempObj = [ordered]@{}
	$TrTempObj.PSTypeName = "TracerouteData"

	$go = $true
	$new_string = $Data
	$pos2 = 0
	$hop = 1
	while ($go -eq $true)
	{
		$pos = $new_string.IndexOf("#")
		if ($pos -lt 0) {
			$go = $false
		}
		elseif ($pos2 -gt 0) {
			$tmp = $new_string.Substring(0, $pos)
			$hopString = "Hop " + $hop
			$TrTempObj.Add($hopString, $tmp)
			$tmp = $new_string.Substring($pos + 1)
			$new_string = $tmp
			$hop += 1
		}
		else {
			$tmp = $new_string.Substring($pos + 1)
			if (-not ([string]::IsNullOrEmpty($tmp))) {
				$new_string = $tmp
				$pos2 = $new_string.IndexOf("#")
				$tmp = $new_string.Substring(0, $pos2)
				$TrTempObj.TracerouteTo = $tmp
				$tmp = $new_string.Substring($pos2 + 1)
				$new_string = $tmp
			}
		}
	}
	$TrObj = New-Object -TypeName PSObject -Prop $TrTempObj

	setKempAPIReturnObject $ReturnCode $Response $TrObj
}

Function setTracerouteAnswer($LmAnswer)
{
	if ($LmAnswer) {
		$retCode  = 200
		$response = "Traceroute success"
		$data     = $LmAnswer

		SetTracerouteOkAnswer $retCode $response $data
	}
	else {
		$retCode  = 422
		$response = "Traceroute failure"
		$data     = $null

		SetTracerouteErrorAnswer $retCode $response $data
	}
}

Function Test-NetworkRoute
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[ValidateNotNullOrEmpty()]
		[String]$Address,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$cmd = "logging/traceroute"

	if (-not (SetTracerouteParam $propertyTable)) {
		Throw "Error: missing address to test network route"
	}

	try {
		$response = sendRequest2LM $cmd $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$response = $_.Exception.Message
	}

	$check = checkDiagnosticCmdletsErrors $response
	if (([String]::IsNullOrEmpty($check))) {
		setTracerouteAnswer $response
	}
	else {
		$check
	}
}
Export-ModuleMember -function Test-NetworkRoute

Function setLmDebugInformationAnswer($cmd, $response)
{
	if ($cmd -and $response) {
		switch ($cmd)
		{
			"ps" {
				$data = $response
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "PsData"
				$tmpObj.ProcessesData = $data
				$psObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $psObject
			}

			"meminfo" {
				$data = $response
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "MemInfoData"
				$tmpObj.MeminfoData = $data
				$psObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $psObject
			}

			"ifconfig" {
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "IfconfigData"
				$tmpObj.Interfaces = $response
				$IfconfigObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $IfconfigObject
			}

			"netstat" {
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "NetstatData"
				$tmpObj.Netstat = $response
				$NetstatObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $NetstatObject
			}

			"interrupts" {
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "InterruptsData"
				$tmpObj.Interrupts = $response
				$InterruptsObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $InterruptsObject
			}

			"partitions" {
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "PartitionsData"
				$tmpObj.Partitions = $response
				$PartitionsObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $PartitionsObject
			}

			"cpuinfo" {
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "CpuinfoData"
				$tmpObj.Cpuinfo = $response
				$CpuinfoObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $CpuinfoObject
			}

			"df" {
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "DfData"
				$tmpObj.Df = $response
				$DfObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $DfObject
			}

			"lspci" {
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "LsPciData"
				$tmpObj.LsPci = $response
				$LsPciObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $LsPciObject
			}

			"lsmod" {
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "LsModData"
				$tmpObj.LsMod = $response
				$LsModObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $LsModObject
			}

			"slabinfo" {
				$data = $response.SlabInfo.data
				$version = $response.SlabInfo.version
				$tmpObj = [ordered]@{}
				$tmpObj.PSTypeName = "SlabinfoData"
				$tmpObj.Slabinfo = $data
				$tmpObj.Version = $version
				$SlabinfoObject = New-Object -TypeName PSObject -Prop $tmpObj

				$answer = setKempAPIReturnObject 200 "cmd successfully executed" $SlabinfoObject
			}

			default {
				Write-Verbose $cmd": Not implemented yet"
			}
		}
	}
	else {
		Throw "ERROR: null input cmd [$cmd], response [$response]"
	}
	$answer
}

Function Get-LmDebugInformation
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateSet("ps","meminfo","ifconfig", "netstat", "interrupts", "partitions", "cpuinfo", "df", "lspci", "lsmod", "slabinfo")]
		[string]$Param,

		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$cmd = "logging/$Param"

	try {
		$response = sendRequest2LM $cmd $null $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$response = $_.Exception.Message
	}

	$check = checkDiagnosticCmdletsErrors $response
	if (([String]::IsNullOrEmpty($check))) {
		$answer = setLmDebugInformationAnswer $Param $response
		$answer
	}
	else {
		$check
	}
}
Export-ModuleMember -function Get-LmDebugInformation

Function setLmResetStatisticAnswer($response)
{
	if ($response) {
		if ($response.Contains("ok")) {
			$answer = setKempAPIReturnObject 200 "cmd successfully executed" $null
		}
		else {
			$answer = setKempAPIReturnObject 422 "cmd failure" $null
		}
	}
	else {
		$answer = setKempAPIReturnObject 422 "cmd failure: no answer from the LM" $null
	}
	$answer
}

Function Reset-LogStatistics
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[ValidateNotNullOrEmpty()] 
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)
	$propertyTable = Convert-BoundParameters -hashtable $psboundparameters

	$cmd = "logging/resetstats"

	try {
		$response = sendRequest2LM $cmd $propertyTable $LoadBalancer $LBPort $Credential $SubjectCN $CertificateStoreLocation
	}
	catch {
		$response = $_.Exception.Message
	}

	$check = checkDiagnosticCmdletsErrors $response
	if (([String]::IsNullOrEmpty($check))) {
		$answer = setLmResetStatisticAnswer $response
		$answer
	}
	else {
		$check
	}
}
Export-ModuleMember -function Reset-LogStatistics

#endregion - diagnostic

#region - SAML

# Internal use only
Function checkInputFile($strFileName = "strFileName is mandatory, please provide a value.")
{
	if (([String]::IsNullOrEmpty($strFileName))) {
		return $false
	}

	if ($strFileName -eq "strFileName is mandatory, please provide a value.") {
		return $false
	}

	if (Test-Path $strFileName) {
		# File exists
		return $true
	}
	else {
		# File does not exist
		return $false
	}
}

# Internal use only
Function checkOutputFile($strFileName = "strFileName is mandatory, please provide a value.")
{
	if (([String]::IsNullOrEmpty($strFileName))) {
		return $false
	}

	if ($strFileName -eq "strFileName is mandatory, please provide a value.") {
		return $false
	}

	$parent = Split-Path -Path $strFileName -Parent
	if (([String]::IsNullOrEmpty($parent))) {
		return $false
	}

	if (Test-Path $parent) {
		# File exists
		return $true
	}
	else {
		# File does not exist
		return $false
	}
}

Function Install-SAMLIdpMetafile
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Domain,

		[Parameter(Mandatory=$true)]
		[string]$Path,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$fileCheck = checkInputFile $Path
	if ($fileCheck -eq $false) {
		setKempAPIReturnObject 401 "The input file does not exist." $null
		return
	}

	$p = Get-ChildItem -Path $Path
	$fn = $p.FullName
	Write-Verbose "fn [$fn]"

	$params = @{}
	$params.Add("domain", $Domain)

	try {
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$lmResponse = Send-LBMessage -Command uploadsamlidpmd -ParameterValuePair $params -File $fn -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$lmResponse = Send-LBMessage -Command uploadsamlidpmd -ParameterValuePair $params -File $fn -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$lmResponse = Send-LBMessage -Command uploadsamlidpmd -ParameterValuePair $params -File $fn -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
	}
	catch {
		$lmResponse = $_.Exception.Message
	}

	if ($lmResponse -eq "ok") {
		setKempAPIReturnObject 200 "Command successfully executed" $null
	}
	else {
		Write-Verbose "lmResponse [$lmResponse]"
		if ($lmResponse -is [array]) {

			$lmaType = $($lmResponse[1].GetType()).Name

			if ($lmaType -eq "XmlElement") {
				$errStr = $($lmResponse[1]).Error

				$r_lmResponse = $errStr
			}
			else {
				$r_lmResponse = $lmResponse[0]
			}
		}
		else {
			$r_lmResponse = $lmResponse
		}
		Write-Verbose "r_lmResponse [$r_lmResponse]"

		$retCode = 401
		if ($r_lmResponse.Contains("Unauthorized") -or
		    $r_lmResponse.Contains("Can't find certificate") -or
		    $r_lmResponse.Contains("The remote name could not be resolved")) {
			$errorString = $r_lmResponse
		}
		elseif ($r_lmResponse.Contains("Domain does not exist")) {
			$retCode = 422
			$errorString = $r_lmResponse
		}
		elseif ($r_lmResponse.Contains("Invalid address format")) {
			$errorString = "Invalid address format"
		}
		elseif ($r_lmResponse.Contains("Unable to connect")) {
			$errorString = "Unable to connect to the remote server"
		}
		else {
			$errorString = "Unknow error"
		}
		setKempAPIReturnObject $retCode "$errorString" $null
	}
	return
}
Export-ModuleMember -function Install-SAMLIdpMetafile

Function Set-SAMLSPEntity
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Domain,

		[ValidateNotNullOrEmpty()]
		[string]$IdpEntityId,

		[ValidateNotNullOrEmpty()]
		[string]$IdpSsoUrl,

		[ValidateNotNullOrEmpty()]
		[string]$IdpLogoffUrl,

		[ValidateNotNullOrEmpty()]
		[string]$IdpCert,

		[ValidateNotNullOrEmpty()]
		[string]$SpEntityId,

		[ValidateNotNullOrEmpty()]
		[string]$SpCert,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$params = [ordered]@{}
	$params.Add("domain", $Domain)

	$count = 0
	if (-not ([String]::IsNullOrEmpty($IdpEntityId))) {
		$params.Add("idp_entity_id", $IdpEntityId)
		$count += 1
	}
	if (-not ([String]::IsNullOrEmpty($IdpSsoUrl))) {
		$params.Add("idp_sso_url", $IdpSsoUrl)
		$count += 1
	}
	if (-not ([String]::IsNullOrEmpty($IdpLogoffUrl))) {
		$params.Add("idp_logoff_url", $IdpLogoffUrl)
		$count += 1
	}
	if (-not ([String]::IsNullOrEmpty($IdpCert))) {
		$params.Add("idp_cert", $IdpCert)
		$count += 1
	}
	if (-not ([String]::IsNullOrEmpty($SpEntityId))) {
		$params.Add("sp_entity_id", $SpEntityId)
		$count += 1
	}
	if (-not ([String]::IsNullOrEmpty($SpCert))) {
		$params.Add("sp_cert", $SpCert)
		$count += 1
	}

	if ($count -eq 0) {
		setKempAPIReturnObject 401 "ERROR: you need to set at least one parameter other than Domain" $null
	}

	try {
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$lmResponse = Send-LBMessage -Command moddomain -ParameterValuePair $params -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$lmResponse = Send-LBMessage -Command moddomain -ParameterValuePair $params -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$lmResponse = Send-LBMessage -Command moddomain -ParameterValuePair $params -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
	}
	catch {
		$lmResponse = $_.Exception.Message
	}

	if ($lmResponse.Domain) {
		$r_lmResponse = $lmResponse.Domain | Convert-XmlToPSObject
		$props = $($r_lmResponse.psobject).Properties
		if ($props) {
			$tmpObj = [ordered]@{}
			$tmpObj.PSTypeName = "DomainData"
			ForEach($item in $props) {
				$tmpObj.Add($($item.Name), $($item.Value))
			}
			$data = New-Object -TypeName PSObject -Prop $tmpObj
			$answer = setKempAPIReturnObject 200 "Command successfully executed." $data
		}
		else {
			$answer = setKempAPIReturnObject 401 "System error." $null
		}
		$answer
		return
	}
	else {
		if ($lmResponse -is [array]) {
			if ($($lmResponse[1]).Error) {
				$r_lmResponse = $lmResponse[0] + " " + $($lmResponse[1]).Error
				$retCode = $($lmResponse[1]).stat
			}
			else {
				$r_lmResponse = $lmResponse[0]
				$retCode = 401
			}
			setKempAPIReturnObject $retCode "$r_lmResponse" $null
			return
		}
		else {
			$r_lmResponse = $lmResponse
			setKempAPIReturnObject 401 "$r_lmResponse" $null
			return
		}
	}
}
Export-ModuleMember -function Set-SAMLSPEntity

Function Get-SAMLDomain
{
	[cmdletbinding(DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Domain,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN
	)

	$params = [ordered]@{}
	$params.Add("domain", $Domain)

	try {
		if (-not ([String]::IsNullOrEmpty($SubjectCN)))
		{
			if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation)))
			{
				$lmResponse = Send-LBMessage -Command showdomain -ParameterValuePair $params -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
			}
			else {
				$lmResponse = Send-LBMessage -Command showdomain -ParameterValuePair $params -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
			}
		}
		else {
			$lmResponse = Send-LBMessage -Command showdomain -ParameterValuePair $params -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
		}
	}
	catch {
		$lmResponse = $_.Exception.Message
	}

	if ($lmResponse.Domain) {
		# OK
		$r_lmResponse = $lmResponse.Domain | Convert-XmlToPSObject
		$props = $($r_lmResponse.psobject).Properties
		if ($props) {
			$tmpObj = [ordered]@{}
			$tmpObj.PSTypeName = "DomainData"
			ForEach($item in $props) {
				$tmpObj.Add($($item.Name), $($item.Value))
			}
			$data = New-Object -TypeName PSObject -Prop $tmpObj
			$answer = setKempAPIReturnObject 200 "Command successfully executed." $data
		}
		else {
			$answer = setKempAPIReturnObject 401 "System error." $null
		}
		$answer
		return
	}
	else {
		# ERROR
		if ($lmResponse -is [array]) {
			if ($($lmResponse[1]).Error) {
				$r_lmResponse = $lmResponse[0] + " " + $($lmResponse[1]).Error
				$retCode = $($lmResponse[1]).stat
			}
			else {
				$r_lmResponse = $lmResponse[0]
				$retCode = 401
			}
			setKempAPIReturnObject $retCode "$r_lmResponse" $null
			return
		}
		else {
			$r_lmResponse = $lmResponse
			setKempAPIReturnObject 401 "$r_lmResponse" $null
			return
		}
	}
}
Export-ModuleMember -function Get-SAMLDomain

Function Export-SAMLSPCert
{
	[cmdletbinding(SupportsShouldProcess=$true, DefaultParameterSetName='Credential')]
	Param(
		[Parameter(Mandatory=$true)]
		[string]$Domain,

		[Parameter(Mandatory=$true)]
		[string]$CertificateFilePath,

		[ValidateNotNullOrEmpty()]
		[string]$LoadBalancer = $LoadBalancerAddress,

		[ValidateNotNullOrEmpty()]
		[ValidateRange(3,65530)]
		[int]$LBPort = $LBAccessPort,

		[Parameter(ParameterSetName="Credential")]
			[ValidateNotNullOrEmpty()]
			[System.Management.Automation.Credential()]$Credential = $script:cred,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$CertificateStoreLocation = $script:CertificateStoreLocation,

		[Parameter(ParameterSetName="Certificate")]
			[ValidateNotNullOrEmpty()]
			[String]$SubjectCN = $script:SubjectCN,

		[switch]$Force
	)

	PROCESS {
		$fileCheck = checkOutputFile $CertificateFilePath
		if ($fileCheck -eq $false) {
			setKempAPIReturnObject 401 "The destination folder does not exist." $null
			return
		}

		$params = @{}
		$params.Add("domain", $Domain)

		try {
			if (($Force) -or $PsCmdlet.ShouldProcess($CertificateFilePath, "Overwrite"))
			{
				if (-not ([String]::IsNullOrEmpty($SubjectCN))) {
					if (-not ([String]::IsNullOrEmpty($CertificateStoreLocation))) {
						$lmResponse = Send-LBMessage -Command downloadsamlspcert -ParameterValuePair $params -File $CertificateFilePath -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN -CertificateStoreLocation $CertificateStoreLocation
					}
					else {
						$lmResponse = Send-LBMessage -Command downloadsamlspcert -ParameterValuePair $params -File $CertificateFilePath -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -SubjectCN $SubjectCN
					}
				}
				else {
					$lmResponse = Send-LBMessage -Command downloadsamlspcert -ParameterValuePair $params -File $CertificateFilePath -Output -LoadBalancer $LoadBalancer -LBPort $LBPort -Credential $Credential
				}
			}
		}
		catch {
			$lmResponse = $_.Exception.Message
		}
		if ([bool]$WhatIfPreference.IsPresent) {
			return
		}

		if ($lmResponse.Contains("OK")) {
			setKempAPIReturnObject 200 "Command successfully executed" $null
			return
		}

		if ($lmResponse.Contains("ERROR: The specified file already exists")) {
			setKempAPIReturnObject 401 "$lmResponse" $null
			return
		}

		if ($lmResponse -is [array]) {

			$lmaType = $($lmResponse[1].GetType()).Name

			if ($lmaType -eq "XmlElement") {
				$errStr = $($lmResponse[1]).Error

				$r_lmResponse = $errStr
			}
			else {
				$r_lmResponse = $lmResponse[0]
			}
		}
		else {
			$r_lmResponse = $lmResponse
		}
		Write-Verbose "r_lmResponse [$r_lmResponse]"

		$retCode = 401
		if ($r_lmResponse.Contains("Unauthorized") -or
		    $r_lmResponse.Contains("Can't find certificate") -or
		    $r_lmResponse.Contains("The remote name could not be resolved")) {
			$errorString = $r_lmResponse
		}
		elseif ($r_lmResponse.Contains("Domain does not exist")) {
			$retCode = 422
			$errorString = $r_lmResponse
		}
		elseif ($r_lmResponse.Contains("Invalid address format")) {
			$errorString = "Invalid address format"
		}
		elseif ($r_lmResponse.Contains("Unable to connect")) {
			$errorString = "Unable to connect to the remote server"
		}
		else {
			$errorString = "Unknow error"
		}
		setKempAPIReturnObject $retCode "$errorString" $null
		return
	}
}
Export-ModuleMember -function Export-SAMLSPCert

#endregion - SAML


# SIG # Begin signature block
# MIIcDQYJKoZIhvcNAQcCoIIb/jCCG/oCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCDPDNGZh5QucGQA
# eXUPHsM7amtjaj3U/KpP+Ams96QtBKCCCuMwggVWMIIEPqADAgECAhAZGjLLdZyX
# uM+sEY3VEn9JMA0GCSqGSIb3DQEBCwUAMIHKMQswCQYDVQQGEwJVUzEXMBUGA1UE
# ChMOVmVyaVNpZ24sIEluYy4xHzAdBgNVBAsTFlZlcmlTaWduIFRydXN0IE5ldHdv
# cmsxOjA4BgNVBAsTMShjKSAyMDA2IFZlcmlTaWduLCBJbmMuIC0gRm9yIGF1dGhv
# cml6ZWQgdXNlIG9ubHkxRTBDBgNVBAMTPFZlcmlTaWduIENsYXNzIDMgUHVibGlj
# IFByaW1hcnkgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkgLSBHNTAeFw0xNDAzMDQw
# MDAwMDBaFw0yNDAzMDMyMzU5NTlaMIGRMQswCQYDVQQGEwJVUzEdMBsGA1UEChMU
# U3ltYW50ZWMgQ29ycG9yYXRpb24xHzAdBgNVBAsTFlN5bWFudGVjIFRydXN0IE5l
# dHdvcmsxQjBABgNVBAMTOVN5bWFudGVjIENsYXNzIDMgRXh0ZW5kZWQgVmFsaWRh
# dGlvbiBDb2RlIFNpZ25pbmcgQ0EgLSBHMjCCASIwDQYJKoZIhvcNAQEBBQADggEP
# ADCCAQoCggEBANAYAu7too0IWGMPJtfdInuI9uTH7DsmGHjTx6QgU42DfKU/fqXI
# K0ffDfWm2cMdJZNgz3zc6gMsvnh/XEhtpwLZSfih6+uaYXyfwCbW3BXYuBB8ILpe
# 9Cj2qOqnXHzGnJCQNDy2Iqz+ugw6HtZehLZb8KOBcHiKjUZSe/zbSfMpExF0T40W
# s8LjoC3HAwSdzMNy4Q4M+wKO8SYXe26u+Lczi6ZhS0Xf8iVEx/ewmCM23Ch5Cuib
# coio2Oiue38KZEWl8FeSmncGRR7rn+hm83p9koFfAC0euPZWE1piDbdHoY9y74Ne
# guCUmOGspa2GN+Cn07qxPnrrRajxwUR94gMCAwEAAaOCAW0wggFpMBIGA1UdEwEB
# /wQIMAYBAf8CAQAwLwYDVR0fBCgwJjAkoCKgIIYeaHR0cDovL3Muc3ltY2IuY29t
# L3BjYTMtZzUuY3JsMBYGA1UdJQEB/wQMMAoGCCsGAQUFBwMDMA4GA1UdDwEB/wQE
# AwIBBjAuBggrBgEFBQcBAQQiMCAwHgYIKwYBBQUHMAGGEmh0dHA6Ly9zLnN5bWNk
# LmNvbTBfBgNVHSAEWDBWMFQGBFUdIAAwTDAjBggrBgEFBQcCARYXaHR0cHM6Ly9k
# LnN5bWNiLmNvbS9jcHMwJQYIKwYBBQUHAgIwGRoXaHR0cHM6Ly9kLnN5bWNiLmNv
# bS9ycGEwKQYDVR0RBCIwIKQeMBwxGjAYBgNVBAMTEVN5bWFudGVjUEtJLTEtNjI5
# MB0GA1UdDgQWBBQWZt5KNONQpxGGA7FsqcaszVlumzAfBgNVHSMEGDAWgBR/02Wn
# wt3su/AwCfNDOfoCrzMxMzANBgkqhkiG9w0BAQsFAAOCAQEAP1sZ8/oT1XU4Klru
# n1qgTKkdxcyU7t4V/vUQbqQbpWSDVBhYxAsooYXDTnTl/4l8/tXtPLpxn1YCJo8W
# Koj+sKMnIs5L4jiOAKY6hl+d5T6o3mRJQXRBIf0HyIQX2h1lMILLJk851gQnpIGx
# S0nDI4t+AjIYJ7erC/MYcrak7mcGbzimWI3g8X5dpGDGqOVQX+DouuKPmVi2taCo
# dvGi8RyIQXJ+UpebCjaZjVD3Aes85/AiauU1jGM2ihqx2WdmX5ca76ggnfAvumzO
# 2ZSFAPFY8X3JfCK1B10CxuYLv6uTk/8nGI4zNn5XNPHDrwTBhPFWs+iHgzb40wox
# 3G4sbTCCBYUwggRtoAMCAQICECeDjyzMAJ09C7Adbyi1uUkwDQYJKoZIhvcNAQEL
# BQAwgZExCzAJBgNVBAYTAlVTMR0wGwYDVQQKExRTeW1hbnRlYyBDb3Jwb3JhdGlv
# bjEfMB0GA1UECxMWU3ltYW50ZWMgVHJ1c3QgTmV0d29yazFCMEAGA1UEAxM5U3lt
# YW50ZWMgQ2xhc3MgMyBFeHRlbmRlZCBWYWxpZGF0aW9uIENvZGUgU2lnbmluZyBD
# QSAtIEcyMB4XDTE2MTAwNTAwMDAwMFoXDTE3MTAwNTIzNTk1OVowgdYxEzARBgsr
# BgEEAYI3PAIBAxMCVVMxGTAXBgsrBgEEAYI3PAIBAhMIRGVsYXdhcmUxHTAbBgNV
# BA8TFFByaXZhdGUgT3JnYW5pemF0aW9uMRAwDgYDVQQFEwc1MDg0MDMyMQswCQYD
# VQQGEwJVUzERMA8GA1UECAwITmV3IFlvcmsxETAPBgNVBAcMCE5ldyBZb3JrMR8w
# HQYDVQQKDBZLRU1QIFRlY2hub2xvZ2llcyBJbmMuMR8wHQYDVQQDDBZLRU1QIFRl
# Y2hub2xvZ2llcyBJbmMuMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
# zUcerldV4VKZ+xPtYfy3/HwbSLJu8HS2urs4QUEF41PJYl50P/+ePYKpuOcksc2l
# n45YO5/zrKTcjgzxMxJhrNDDAelPeZjiDlkidj6a79msxMXjRzIAI8A4b5SpKBKc
# 7GVANCt0HZlEWjYegiaVjA6fRJFTRuJVr+K1fc1M79mgARNPCLOuy21N7d+MNvOl
# nfBlvRGJZC2CQeeXHjhh/q8RdPmVqXliG8zkIIX+wq+kyVSO1ngYsDJZ4iMmjpCA
# QzdT564zK8potK4fjPblYDiiEtUR1wGg6g4l2X18Mci21of4RbWBKQ/hxQm6oprA
# XBraFsYnwwwt8QSl9+UtmQIDAQABo4IBkDCCAYwwLgYDVR0RBCcwJaAjBggrBgEF
# BQcIA6AXMBUME1VTLURFTEFXQVJFLTUwODQwMzIwCQYDVR0TBAIwADAOBgNVHQ8B
# Af8EBAMCB4AwKwYDVR0fBCQwIjAgoB6gHIYaaHR0cDovL3N3LnN5bWNiLmNvbS9z
# dy5jcmwwYAYDVR0gBFkwVzBVBgVngQwBAzBMMCMGCCsGAQUFBwIBFhdodHRwczov
# L2Quc3ltY2IuY29tL2NwczAlBggrBgEFBQcCAjAZDBdodHRwczovL2Quc3ltY2Iu
# Y29tL3JwYTAWBgNVHSUBAf8EDDAKBggrBgEFBQcDAzAfBgNVHSMEGDAWgBQWZt5K
# NONQpxGGA7FsqcaszVlumzAdBgNVHQ4EFgQUu/UpUEJhq9Dx+3CC/n6LPC3FyRYw
# WAYIKwYBBQUHAQEETDBKMB8GCCsGAQUFBzABhhNodHRwOi8vc3cuc3ltY2QuY29t
# MCcGCCsGAQUFBzAChhtodHRwOi8vc3cxLnN5bWNiLmNvbS9zdy5jcnQwDQYJKoZI
# hvcNAQELBQADggEBAF/gxzrK67CSW740OGSXAC4/NzRVcP5RmONNEXIjmYEowtgN
# UtXAns9olC+uzborP1Pq7MvZKMC0CW5P8GsloeqVGjRQ2IPFiEvLAagQO5HVzDMA
# NZeuTNS5At5i+MkqkW+sLoVH+tBVDhbn17sH2mX0wXid4NCOojyVA1FarE0gup+v
# XjhwBlXQPRU2K49cmd6dryN1GGufmiXJC25fggZ+2lc0A+j1mBfN9lsMVe7ZYgvW
# o1WyWM/K66ga8FC9MIpv2tzVQc5Oy0UysQtnHykfRuAA5yXHaiXejx2uF5/5LWR1
# s53Zchz0LXsZ+ndn4Utg0e2T9iRyuW7Sw7jek4sxghCAMIIQfAIBATCBpjCBkTEL
# MAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMR8wHQYD
# VQQLExZTeW1hbnRlYyBUcnVzdCBOZXR3b3JrMUIwQAYDVQQDEzlTeW1hbnRlYyBD
# bGFzcyAzIEV4dGVuZGVkIFZhbGlkYXRpb24gQ29kZSBTaWduaW5nIENBIC0gRzIC
# ECeDjyzMAJ09C7Adbyi1uUkwDQYJYIZIAWUDBAIBBQCgfDAQBgorBgEEAYI3AgEM
# MQIwADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEEAYI3AgELMQ4w
# DAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0BCQQxIgQg/z36xfoNpBEmCmldUxxLGZp8
# vgu0DmfP3zGHXaF8+EcwDQYJKoZIhvcNAQEBBQAEggEAZlPzambZbqb7M2UWTt2f
# 5JNycrsIEDT9xzuO0IMVwcHKxgf+nMyZvEnc1fRbTSptYdLfk76CRwMLp0X3PU32
# SL0Pi01pKJiS39FzkPvAZ8stp4QoCFZVQfZWDBF70tpORa84Zvt95Vw+NKVieHV5
# 5Qw0yBoaagps8h1eCx/XsdcE5XlLbVeuKzXFfZ3tL++k7eFaD/94iV2Lq8itfeUk
# vZ19GfK69PlXy5ZkPssayOr32RjgMr8vEwfgJLfWqrY+thHL3IUcaLpjvQ+TJSyL
# C5V/9xDbkvv/FUK9UYf8tiiVw8qxo1N6RetV3momRVfW8Ej5gbvfSLPv4Q5+ZYCy
# jaGCDiwwgg4oBgorBgEEAYI3AwMBMYIOGDCCDhQGCSqGSIb3DQEHAqCCDgUwgg4B
# AgEDMQ0wCwYJYIZIAWUDBAIBMIH/BgsqhkiG9w0BCRABBKCB7wSB7DCB6QIBAQYL
# YIZIAYb4RQEHFwMwITAJBgUrDgMCGgUABBSFuW+IbMQQsymT0+C9Y5BGoof4qwIV
# AIc7jHfC0BF4ecjyAb1BPXno7i0bGA8yMDE3MDcxMDE4MTQ1N1owAwIBHqCBhqSB
# gzCBgDELMAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9u
# MR8wHQYDVQQLExZTeW1hbnRlYyBUcnVzdCBOZXR3b3JrMTEwLwYDVQQDEyhTeW1h
# bnRlYyBTSEEyNTYgVGltZVN0YW1waW5nIFNpZ25lciAtIEcyoIIKizCCBTgwggQg
# oAMCAQICEHsFsdRJaFFE98mJ0pwZnRIwDQYJKoZIhvcNAQELBQAwgb0xCzAJBgNV
# BAYTAlVTMRcwFQYDVQQKEw5WZXJpU2lnbiwgSW5jLjEfMB0GA1UECxMWVmVyaVNp
# Z24gVHJ1c3QgTmV0d29yazE6MDgGA1UECxMxKGMpIDIwMDggVmVyaVNpZ24sIElu
# Yy4gLSBGb3IgYXV0aG9yaXplZCB1c2Ugb25seTE4MDYGA1UEAxMvVmVyaVNpZ24g
# VW5pdmVyc2FsIFJvb3QgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkwHhcNMTYwMTEy
# MDAwMDAwWhcNMzEwMTExMjM1OTU5WjB3MQswCQYDVQQGEwJVUzEdMBsGA1UEChMU
# U3ltYW50ZWMgQ29ycG9yYXRpb24xHzAdBgNVBAsTFlN5bWFudGVjIFRydXN0IE5l
# dHdvcmsxKDAmBgNVBAMTH1N5bWFudGVjIFNIQTI1NiBUaW1lU3RhbXBpbmcgQ0Ew
# ggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQC7WZ1ZVU+djHJdGoGi61Xz
# sAGtPHGsMo8Fa4aaJwAyl2pNyWQUSym7wtkpuS7sY7Phzz8LVpD4Yht+66YH4t5/
# Xm1AONSRBudBfHkcy8utG7/YlZHz8O5s+K2WOS5/wSe4eDnFhKXt7a+Hjs6Nx23q
# 0pi1Oh8eOZ3D9Jqo9IThxNF8ccYGKbQ/5IMNJsN7CD5N+Qq3M0n/yjvU9bKbS+GI
# mRr1wOkzFNbfx4Dbke7+vJJXcnf0zajM/gn1kze+lYhqxdz0sUvUzugJkV+1hHk1
# inisGTKPI8EyQRtZDqk+scz51ivvt9jk1R1tETqS9pPJnONI7rtTDtQ2l4Z4xaE3
# AgMBAAGjggF3MIIBczAOBgNVHQ8BAf8EBAMCAQYwEgYDVR0TAQH/BAgwBgEB/wIB
# ADBmBgNVHSAEXzBdMFsGC2CGSAGG+EUBBxcDMEwwIwYIKwYBBQUHAgEWF2h0dHBz
# Oi8vZC5zeW1jYi5jb20vY3BzMCUGCCsGAQUFBwICMBkaF2h0dHBzOi8vZC5zeW1j
# Yi5jb20vcnBhMC4GCCsGAQUFBwEBBCIwIDAeBggrBgEFBQcwAYYSaHR0cDovL3Mu
# c3ltY2QuY29tMDYGA1UdHwQvMC0wK6ApoCeGJWh0dHA6Ly9zLnN5bWNiLmNvbS91
# bml2ZXJzYWwtcm9vdC5jcmwwEwYDVR0lBAwwCgYIKwYBBQUHAwgwKAYDVR0RBCEw
# H6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0yMDQ4LTMwHQYDVR0OBBYEFK9j1sqj
# ToVy4Ke8QfMpojh/gHViMB8GA1UdIwQYMBaAFLZ3+mlIR59TEtXC6gcydgfRlwcZ
# MA0GCSqGSIb3DQEBCwUAA4IBAQB16rAt1TQZXDJF/g7h1E+meMFv1+rd3E/zociB
# iPenjxXmQCmt5l30otlWZIRxMCrdHmEXZiBWBpgZjV1x8viXvAn9HJFHyeLojQP7
# zJAv1gpsTjPs1rSTyEyQY0g5QCHE3dZuiZg8tZiX6KkGtwnJj1NXQZAv4R5NTtzK
# EHhsQm7wtsX4YVxS9U72a433Snq+8839A9fZ9gOoD+NT9wp17MZ1LqpmhQSZt/gG
# V+HGDvbor9rsmxgfqrnjOgC/zoqUywHbnsc4uw9Sq9HjlANgCk2g/idtFDL8P5dA
# 4b+ZidvkORS92uTTw+orWrOVWFUEfcea7CMDjYUq0v+uqWGBMIIFSzCCBDOgAwIB
# AgIQVFjyqtdB1kS8hKl7oJZS5jANBgkqhkiG9w0BAQsFADB3MQswCQYDVQQGEwJV
# UzEdMBsGA1UEChMUU3ltYW50ZWMgQ29ycG9yYXRpb24xHzAdBgNVBAsTFlN5bWFu
# dGVjIFRydXN0IE5ldHdvcmsxKDAmBgNVBAMTH1N5bWFudGVjIFNIQTI1NiBUaW1l
# U3RhbXBpbmcgQ0EwHhcNMTcwMTAyMDAwMDAwWhcNMjgwNDAxMjM1OTU5WjCBgDEL
# MAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMR8wHQYD
# VQQLExZTeW1hbnRlYyBUcnVzdCBOZXR3b3JrMTEwLwYDVQQDEyhTeW1hbnRlYyBT
# SEEyNTYgVGltZVN0YW1waW5nIFNpZ25lciAtIEcyMIIBIjANBgkqhkiG9w0BAQEF
# AAOCAQ8AMIIBCgKCAQEAmfP82AQJA4b511ymk8BCfOp8Y89dAOKO88CQ348p9Rjq
# lLeS5dewoHOB6OkKm0p8Af+dj6Q5pw7qRfQiDDpw7TlFi+TFG1zwRWhGJAVjdpsc
# /J5sKrFW5Yp/UnGu8jXVRiMGHM9ILR20zbjZdiOOHP8+v7sGXGkHpmUO+F6ufS7t
# Ta4178nXAEL9KJUOn11yQgm8w9pE0u3MR4Tk/MotrFi+rveu2UQNCLfCd9YaQ3DR
# bgPeUpLEEAhx2boiVfIfvO2bnTviXh1Mg/+XD3sL51WDTtIN677X7K5uR7mf36XW
# UbwEVe3/J3BMye0qSxPhsblMD8kB7lVlX2kCeGbLPwIDAQABo4IBxzCCAcMwDAYD
# VR0TAQH/BAIwADBmBgNVHSAEXzBdMFsGC2CGSAGG+EUBBxcDMEwwIwYIKwYBBQUH
# AgEWF2h0dHBzOi8vZC5zeW1jYi5jb20vY3BzMCUGCCsGAQUFBwICMBkaF2h0dHBz
# Oi8vZC5zeW1jYi5jb20vcnBhMEAGA1UdHwQ5MDcwNaAzoDGGL2h0dHA6Ly90cy1j
# cmwud3Muc3ltYW50ZWMuY29tL3NoYTI1Ni10c3MtY2EuY3JsMBYGA1UdJQEB/wQM
# MAoGCCsGAQUFBwMIMA4GA1UdDwEB/wQEAwIHgDB3BggrBgEFBQcBAQRrMGkwKgYI
# KwYBBQUHMAGGHmh0dHA6Ly90cy1vY3NwLndzLnN5bWFudGVjLmNvbTA7BggrBgEF
# BQcwAoYvaHR0cDovL3RzLWFpYS53cy5zeW1hbnRlYy5jb20vc2hhMjU2LXRzcy1j
# YS5jZXIwKAYDVR0RBCEwH6QdMBsxGTAXBgNVBAMTEFRpbWVTdGFtcC0yMDQ4LTUw
# HQYDVR0OBBYEFAm1wf6WcpcpQ5rJ4AK6rvj9L7r2MB8GA1UdIwQYMBaAFK9j1sqj
# ToVy4Ke8QfMpojh/gHViMA0GCSqGSIb3DQEBCwUAA4IBAQAXswqI6VxaXiBrOwoV
# smzFqYoyh9Ox9BxTroW+P5v/17y3lIW0x1J+lOi97WGy1KeZ5MPJk8E1PQvoaApd
# Vpi9sSI70UR617/wbVEyitUj3zgBN/biUyt6KxGPt01sejMDG3xrCZQXu+TbWNQh
# E2Xn7NElyix1mpx//Mm7KmirxH20z6PJbKfZxACciQp3kfRNovsxO4Zu9uYfUAOG
# m7/LQqvmdptyWhEBisbvpW+V592uuuYiZfAYWRsRyc2At9iXRx9CCPiscR+wRlOz
# 1LLVo6tQdUgSF4Ktz+BBTzJ+zZUcv5GKCD2kp2cClt8kTKXQQcCCYKOKFzJL07zP
# pLSMMYICWjCCAlYCAQEwgYswdzELMAkGA1UEBhMCVVMxHTAbBgNVBAoTFFN5bWFu
# dGVjIENvcnBvcmF0aW9uMR8wHQYDVQQLExZTeW1hbnRlYyBUcnVzdCBOZXR3b3Jr
# MSgwJgYDVQQDEx9TeW1hbnRlYyBTSEEyNTYgVGltZVN0YW1waW5nIENBAhBUWPKq
# 10HWRLyEqXugllLmMAsGCWCGSAFlAwQCAaCBpDAaBgkqhkiG9w0BCQMxDQYLKoZI
# hvcNAQkQAQQwHAYJKoZIhvcNAQkFMQ8XDTE3MDcxMDE4MTQ1N1owLwYJKoZIhvcN
# AQkEMSIEIE7k1q20YbkC81rrOxFYDiwgYHheGKAOQDlrNSf5Su9aMDcGCyqGSIb3
# DQEJEAIvMSgwJjAkMCIEIM96wXrQR+zV/cNoIgMbEtTvB4tvK0xea6Qfj/LPS61n
# MAsGCSqGSIb3DQEBAQSCAQCAz/c4it3RK0Hxcctou4c61/Q8PI22VqTOg6Abw3wA
# qGiRcFqM3UTjpQrtBQw54jB8P+4nR9zXhQPf2hpg5wwUHa5aytEj4sl7OGSjWl3g
# sOwYXw/MqFKC2tJ4C0JHamyrshYCt8p7FsGFepViZEnQgZv1oiIHAfw3woRKausP
# UEdmg/Fih4k0hvOxEA4eYn4QguMXRhxSi+GVcr4qZyyWUkZTlyiNwGpHVvnF4d0r
# WCmUWGxNR5GSvFDwhwWpAqTkTu+JOFxjrHxY8E2BTJV2humlW8lscmvL3jt2DkID
# aB4jewFUdJy8hGJH+ZXOuTTtRcwhhe/P8efRMHT+Fa45
# SIG # End signature block
